package com.mobilegame.dominoes.Actor;

import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.g2d.Batch;
import com.badlogic.gdx.graphics.g2d.NinePatch;
import com.badlogic.gdx.graphics.g2d.TextureAtlas;
import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.scenes.scene2d.Actor;
import com.mobilegame.dominoes.DominoGame;

/**
 * Created by Doodle on 2021/1/28.
 */

public class DominoActor extends Actor {
    private TextureAtlas textureAtlas;
    private int[] values;
    private NinePatch dominoBgNine;
    private NinePatch dominoShadowNine;
    private TextureRegion[] dominoPointRegions;

    private float dominoBgX, dominoBgY, dominoShadowX, dominoShadowY, point1X, point1Y, point2X, point2Y;
    private float dominoBgW, dominoBgH, dominoShadowW, dominoShadowH, pointW, pointH;

    public DominoActor(TextureAtlas textureAtlas, int[] values) {
        if (textureAtlas == null) {
            return;
        }
        this.textureAtlas = textureAtlas;
        this.values = values;

        dominoPointRegions = new TextureRegion[2];
        //骨牌下边界阴影
        setDominoShadowRegion();
        //骨牌背景
        setDominoBgRegion();
        //骨牌上面的点
        setDominoPoint(values);

        setSize(dominoBgW, dominoBgH + dominoShadowH / 2f);
    }

    public void setDominoBgRegion() {
        TextureRegion dominobg = textureAtlas.findRegion(DominoGame.dominoeName + "board-dominoes");
        dominoBgNine = new NinePatch(dominobg, 20, 20, 20, 20);
        dominoBgW = 89;
        dominoBgH = 177;
    }

    public void setDominoShadowRegion() {
        TextureRegion dominoShadow = textureAtlas.findRegion(DominoGame.dominoeName + "button");
        dominoShadowNine = new NinePatch(dominoShadow, 20, 20, 20, 1);
        dominoShadowW = 89;
        dominoShadowH = dominoShadow.getRegionHeight();
    }

    public void setDominoPoint(int[] values) {
        TextureRegion point1Region = textureAtlas.findRegion(DominoGame.dominoeName + "point-" + values[0]);
        TextureRegion point2Region = textureAtlas.findRegion(DominoGame.dominoeName + "point-" + values[1]);
        dominoPointRegions[0] = point1Region;
        dominoPointRegions[1] = point2Region;
        pointW = 60;
        pointH = 60;
    }

    public void refresh() {
        setDominoShadowRegion();
        setDominoBgRegion();
        setDominoPoint(values);
    }

    @Override
    public void setPosition(float x, float y) {
        super.setPosition(x, y);
        dominoShadowX = x;
        dominoShadowY = y;
        dominoBgX = x;
        dominoBgY = y + 6;
        float h = dominoBgH / 2f;
        point1X = dominoBgX + (dominoBgW - pointW) / 2f;
        point1Y = dominoBgY + (h - pointH) / 2f + h;
        point2X = point1X;
        point2Y = dominoBgY + (h - pointH) / 2f;
    }

    @Override
    public void draw(Batch batch, float parentAlpha) {
        Color oldColor = batch.getColor();
        batch.setColor(getColor());
        if (dominoShadowNine != null) {
            dominoShadowNine.draw(batch, dominoShadowX, dominoShadowY, dominoShadowW, dominoShadowH);
        }
        if (dominoBgNine != null) {
            dominoBgNine.draw(batch, dominoBgX, dominoBgY, dominoBgW, dominoBgH);
        }
        if (dominoPointRegions[0] != null) {
            batch.draw(dominoPointRegions[0], point1X, point1Y, pointW, pointH);
        }

        if (dominoPointRegions[1] != null) {
            batch.draw(dominoPointRegions[1], point2X, point2Y, pointW, pointH);
        }
        batch.setColor(oldColor);
    }
}
