package com.mobilegame.dominoes.Actor;

import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.scenes.scene2d.Group;
import com.badlogic.gdx.scenes.scene2d.actions.Actions;
import com.badlogic.gdx.scenes.scene2d.actions.SequenceAction;
import com.badlogic.gdx.scenes.scene2d.ui.Image;
import com.badlogic.gdx.scenes.scene2d.ui.Label;
import com.badlogic.gdx.scenes.scene2d.utils.TextureRegionDrawable;
import com.badlogic.gdx.utils.Align;
import com.mobilegame.dominoes.assets.Assets;
import com.mobilegame.dominoes.audio.AudioManager;

/**
 * Created by Doodle on 2021/3/1.
 */

public class PointActor extends Group {
    Label pointLabel;
    Image pointBG;
    final TextureRegionDrawable pointBg;

    public PointActor(boolean isGreen) {
        TextureRegion bgTexture = Assets.dominoes.findRegion("7");
        pointBg = new TextureRegionDrawable(new TextureRegion(bgTexture));

        pointBG = new Image(pointBg);

        setBGRegion(isGreen);
        pointBG.setSize(pointBG.getWidth(), pointBG.getHeight());
        addActor(pointBG);

        Label.LabelStyle labelStyle = new Label.LabelStyle();
        labelStyle.font = Assets.instance.fonts[2];

        setSize(pointBG.getWidth(), pointBG.getHeight());

        pointLabel = new Label("+10", labelStyle);
        pointLabel.setAlignment(Align.center);
        pointLabel.setPosition(getWidth() / 2f, getHeight() / 2f - 2, Align.center);
        pointLabel.setFontScale(0.7f);
        setLabelColor(isGreen);
        addActor(pointLabel);

        setOrigin(Align.center);
    }

    public void setBGRegion(boolean isGreen) {
        if (isGreen) {
            pointBG.setColor(138 / 255f, 253 / 255f, 101 / 255f, 1);
        } else {
            pointBG.setColor(1, 240 / 255f, 161 / 255f, 1);
        }
    }

    public void setLabelColor(boolean isGreen) {
        if (isGreen) {
            pointLabel.setColor(9 / 255f, 123 / 255f, 49 / 255f, 1);
        } else {
            pointLabel.setColor(192 / 255f, 96 / 255f, 39 / 255f, 1);
        }
    }

    public void setText(String string) {
        pointLabel.setText(string);
    }

    public void setColor(boolean isGreen) {
        setBGRegion(isGreen);
        setLabelColor(isGreen);
    }

    public void animationStart(float endX, float endY, Runnable runnable) {
        setScale(0);
        setVisible(true);
        SequenceAction sequenceAction = Actions.sequence(
                Actions.scaleTo(1.1f, 1.1f, 0.2f),
                Actions.scaleTo(1, 1, 0.2f),
                Actions.run(new Runnable() {
                    @Override
                    public void run() {
                        AudioManager.PlaySound(AudioManager.AudioType.score);
                    }
                }),
                Actions.delay(0.2f),
                Actions.parallel(
                        Actions.moveTo(endX - 20, endY - 40, 0.5f),
                        Actions.scaleTo(0.2f, 0.2f, 0.5f)
                )
        );

        if (runnable != null) {
            sequenceAction.addAction(Actions.run(runnable));
        }

        sequenceAction.addAction(Actions.hide());

        clearActions();
        addAction(sequenceAction);
    }
}
