package com.mobilegame.dominoes.Animation;

import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.math.Interpolation;
import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.Group;
import com.badlogic.gdx.scenes.scene2d.Stage;
import com.badlogic.gdx.scenes.scene2d.actions.Actions;
import com.badlogic.gdx.scenes.scene2d.actions.SequenceAction;
import com.badlogic.gdx.scenes.scene2d.ui.Label;
import com.badlogic.gdx.utils.Align;
import com.badlogic.gdx.utils.ArrayMap;
import com.mobilegame.dominoes.Actor.AddPointActor;
import com.mobilegame.dominoes.assets.Configuration;
import com.mobilegame.dominoes.handles.DominoManager;
import com.mobilegame.dominoes.handles.GameConfig;
import com.mobilegame.dominoes.handles.PlayerManager;
import com.mobilegame.dominoes.objects.DominoCardPlayer2;

/**
 * Created by Doodle on 2021/2/5.
 */

public class ComputerHandFinishAnimations extends AnimationBase {
    private final float offset = 30;
    private final int rowNum = 4;

    private PlayerManager[] playerManagers;
    private ArrayMap<Integer, Actor> dominoObjects;
    private DominoManager[] dominoManagers;
    private Stage stage;
    private DominoPosition dp;
    private Label[] pointLabel;
    private int winners;

    private float x = 0;
    private float y = 0;

    private float pointLabelX, pointLabelY;


    public ComputerHandFinishAnimations(AnimationManager animationManager,
                                        PlayerManager[] playerManagers, ArrayMap<Integer, Actor> dominoObjects,
                                        DominoManager[] dominoManagers, Stage stage, Label[] pointLabel,
                                        int winners) {
        super(animationManager);
        this.playerManagers = playerManagers;
        this.dominoObjects = dominoObjects;
        this.dominoManagers = dominoManagers;
        this.pointLabel = pointLabel;
        this.winners = winners;
        this.stage = stage;

        Group parentG = pointLabel[winners].getParent();
        pointLabelX = pointLabel[winners].getX() + parentG.getX() + pointLabel[winners].getWidth() / 2f;
        pointLabelY = pointLabel[winners].getY() + parentG.getY() + pointLabel[winners].getHeight() / 2f;
    }

    private void addAnimation(Actor a, int index, Runnable run) {
        a.clearActions();
        a.addAction(
                Actions.sequence(
                        Actions.delay(index * 0.28f),
                        Actions.moveBy(0, 50, 0.3333f, Interpolation.sineIn),
                        Actions.run(run),
                        Actions.removeActor()
                )
        );
    }

    private void addShowDominoAnimation(Actor a, final int index, final float midX, final float midY, float groupX, float groupY, final float midW, final float midH, final int point) {
        final float mx = midX + groupX;
        final float my = midY + groupY;
        a.clearActions();
        SequenceAction sequenceAction = Actions.sequence(Actions.parallel(
                Actions.moveTo(midX, midY, 0.2333f),
                Actions.scaleTo(midW, midH),
                Actions.run(new Runnable() {
                    @Override
                    public void run() {
                        if (winners == 0) {
                            addPointAnimation(index, mx, my, midW, midH, point, playerManagers[1].idsInHand.size, 0);
                        } else {
                            if (index == playerManagers[1].idsInHand.size - 1) {
                                addPlayAnimation();
                            }
                        }
                    }
                })
        ));
        a.addAction(sequenceAction);
    }

    private final Color c = new Color(251 / 255f, 197 / 255f, 1 / 255f, 1);

    private void addPointAnimation(int index, float x, float y, float sw, float sh, final int point, int size, float delay) {
        AddPointActor addPointActor = new AddPointActor();
        addPointActor.setOrigin(Align.center);
        addPointActor.setScale(0, 0);
        addPointActor.setPosition(x, y, Align.center);
        addPointActor.setPoint(point + "");

        addPointActor.setBgColor(c);
        addPointActor.clearActions();

        float px = pointLabelX - addPointActor.getWidth() / 2f;
        float py = pointLabelY - addPointActor.getHeight() / 2f;

        SequenceAction sequenceAction = Actions.sequence(
                Actions.delay(delay),
                Actions.delay(0.5f, Actions.scaleTo(sw, sh, 0.2f)),
                Actions.moveTo(px, py, 0.8f, Interpolation.sineOut),
                Actions.scaleTo(0, 0, 0.2f),
                Actions.run(new Runnable() {
                    @Override
                    public void run() {
                        addPointFunc();
                    }
                })
        );

        if (index == size - 1) {
            sequenceAction.addAction(Actions.run(runnable));
        }

        sequenceAction.addAction(Actions.removeActor());

        addPointActor.addAction(sequenceAction);
        stage.addActor(addPointActor);
    }

    public void addPointFunc() {

    }

//    private void addComputerAnimation() {
//
//    }

//    private void addPlayersAnimation(int index, float x, float y, float sw, float sh, final int point) {
//        AddPointActor addPointActor = new AddPointActor();
//        addPointActor.setOrigin(Align.center);
//        addPointActor.setScale(0, 0);
//        addPointActor.setPosition(x, y, Align.center);
//        addPointActor.setPoint(point + "");
//
//        addPointActor.setBgColor(Color.GREEN);
//        addPointActor.clearActions();
//
//        GameConfig.points[0] += point;
//        SequenceAction sequenceAction = Actions.sequence(
//                Actions.delay(0.5f, Actions.scaleTo(sw, sh, 0.2f)),
//                Actions.moveTo(pointLabelX, pointLabelY, 0.8f, Interpolation.sineOut),
//                Actions.scaleTo(0, 0, 0.2f),
//                Actions.run(new Runnable() {
//                    @Override
//                    public void run() {
//                        pointLabel.setText(GameConfig.points[0] + "");
//                    }
//                })
//        );
//
//        if (index == idsInHand.size - 1) {
//            sequenceAction.addAction(Actions.run(runnable));
//        }
//
//        sequenceAction.addAction(Actions.removeActor());
//
//        addPointActor.addAction(sequenceAction);
//        stage.addActor(addPointActor);
//    }

    private DominoPosition getDominoPositionForLine(Actor a, int size) {
        if (dp == null) {
            dp = new DominoPosition();
            dp.width = a.getWidth();

            dp.width = (720 - (size + 1) * offset) / size;  //计算可以显示的牌宽度
            if (dp.width > a.getWidth()) {
                dp.width = a.getWidth();
            }
            dp.height = a.getHeight() / a.getWidth() * dp.width;    //通过当前可以显示的牌宽度计算当前牌高度

            float ws = dp.width * size + (size + 1) * offset;
            float hs = dp.height;

            dp.x = Configuration.width / 2f - ws / 2f;
            dp.y = 1000 + hs / 2f - dp.height;
        }

        return dp;
    }

    private DominoPosition getDominoPosition(Actor a, int size) {
        if (dp == null) {
            dp = new DominoPosition();
            dp.width = a.getWidth();

            dp.width = (720 - (size + 1) * offset) / size;  //计算可以显示的牌宽度
            if (dp.width > a.getWidth()) {
                dp.width = a.getWidth();
            }
            dp.height = a.getHeight() / a.getWidth() * dp.width;    //通过当前可以显示的牌宽度计算当前牌高度

            float ws;
            float hs;
            if (size <= rowNum) {
                ws = dp.width * size + (size + 1) * offset;
                hs = dp.height;
            } else {
                int rowN = rowNum;
                int listN = size / rowN;
                if (size % rowN > 0) {
                    listN += 1;
                }

                ws = rowNum * dp.width + (rowNum + 1) * offset;
                hs = listN * dp.height + (listN - 1) * offset;
            }
            dp.x = Configuration.width / 2f - ws / 2f;
            dp.y = 800 + hs / 2f - dp.height;
        }

        return dp;
    }

    @Override
    public void animationStart() {
        int size = playerManagers[1].idsInHand.size;
        if (size > 0) {
            for (int i = 0; i < size; ++i) {
                int id = playerManagers[1].idsInHand.get(i);
                final Actor a = dominoObjects.get(id);
                final Group g = a.getParent();
                final int finalI = i;
                addAnimation(a, i, new Runnable() {
                    @Override
                    public void run() {
                        DominoManager d = dominoManagers[playerManagers[1].idsInHand.get(finalI)];
                        int point = d.values[0] + d.values[1];
                        if (GameConfig.gameType == GameConfig.ALL_FIVES_TYPE) {
                            point = allfiveScore(point);
                        }
                        GameConfig.points[winners] += point;

                        Actor b = new DominoCardPlayer2(playerManagers[1].idsInHand.get(finalI), d.values, null);
                        DominoPosition dp = getDominoPositionForLine(b, playerManagers[1].idsInHand.size);

                        b.setPosition(a.getX(), a.getY());
                        float scaleX = a.getWidth() / b.getWidth();
                        float scaleY = a.getHeight() / b.getHeight();
                        b.setScale(scaleX, scaleY);
                        g.addActor(b);

                        x = x + offset;
                        scaleX = dp.width / b.getWidth();
                        scaleY = dp.height / b.getHeight();
                        addShowDominoAnimation(b, finalI, dp.x + x - g.getX(), dp.y + y - g.getY(), g.getX(), g.getY(), scaleX, scaleY, point);
                        x += dp.width;
//                        if (finalI != 0 && finalI % (rowNum - 1) == 0) {
//                            x = 0;
//                            y -= dp.height + offset;
//                        }
                    }
                });
            }
        } else {
            addPlayAnimation();
        }
    }

    public void addPlayAnimation() {
        for (int i = 0, s = playerManagers[0].idsInHand.size; i < s; ++i) {
            int id = playerManagers[0].idsInHand.get(i);
            final Actor a = dominoObjects.get(id);
            Group g = a.getParent();
            DominoManager d = dominoManagers[playerManagers[0].idsInHand.get(i)];
            int point = d.values[0] + d.values[1];
            if (GameConfig.gameType == GameConfig.ALL_FIVES_TYPE) {
                point = allfiveScore(point);
            }
            GameConfig.points[winners] += point;
            addPointAnimation(i, a.getX() + g.getX(), a.getY() + g.getY(), 1, 1, point, s, i * 0.2f);
        }
    }

    private int allfiveScore(int point) {
        int a = point / 5;
        return a * 5;
    }

    @Override
    public void animationFinish() {
    }

    @Override
    public void animationMidStart() {

    }

    public class DominoPosition {
        public float x;
        public float y;
        public float width;
        public float height;
    }
}
