package com.mobilegame.dominoes.Animation;

import com.badlogic.gdx.math.Interpolation;
import com.badlogic.gdx.scenes.scene2d.Group;
import com.badlogic.gdx.scenes.scene2d.actions.Actions;
import com.badlogic.gdx.scenes.scene2d.actions.SequenceAction;
import com.badlogic.gdx.utils.IntArray;
import com.mobilegame.dominoes.handles.DominoManager;
import com.mobilegame.dominoes.objects.DominoCard;

/**
 * Created by Doodle on 2021/2/5.
 */

public class FinishAnimation2 extends AnimationBase {
    DominoManager[] dominoManagers;
    IntArray layedIds;
    Group board;

    public FinishAnimation2(AnimationManager animationManager,
                            DominoManager[] dominoManagers, IntArray layedIds, Group board) {
        super(animationManager);
        this.dominoManagers = dominoManagers;
        this.layedIds = layedIds;
        this.board = board;
    }

    public void bubbleSort(IntArray layedIds) {
        int endI = layedIds.size - 1;
        while (endI > 0) {
            for (int j = 0; j < endI; ++j) {
                if (dominoManagers[layedIds.get(j)].dominoBody.getY() > dominoManagers[layedIds.get(j + 1)].dominoBody.getY()) {
                    endI = j;
                    layedIds.swap(j, j + 1);
                }

                if (j >= endI) {
                    return;
                }
            }
        }
    }

    @Override
    public void animationStart() {
//        float time = 0.15f;
        int size = layedIds.size;
//        if (size > 9) {
        float time = 0.1f;
//        }

        float maxX = 0;
        for (int i = 0; i < size; ++i) {
            DominoCard dominoCard = dominoManagers[layedIds.get(i)].dominoBody;
            if (maxX < dominoCard.getX()) {
                maxX = dominoCard.getX();
            }
        }

        bubbleSort(layedIds);
        int i = 0;
        int left = 0;
        int right = layedIds.size - 1;

        while (left <= right) {
            if (left != right) {
                DominoCard dominoCard2 = dominoManagers[layedIds.get(right)].dominoBody;
                dominoCard2.clearActions();
                dominoCard2.addAction(
                        Actions.sequence(
                                Actions.delay(i * time),
                                Actions.moveTo(maxX + 720, dominoCard2.getY(), 0.1f, Interpolation.linear),  //0.03667f
                                Actions.hide(),
                                Actions.removeActor()
                        )
                );
            }

            DominoCard dominoCard = dominoManagers[layedIds.get(left)].dominoBody;
            dominoCard.clearActions();
            SequenceAction sequenceAction = Actions.sequence(
                    Actions.delay(i * time),
                    Actions.moveTo(maxX + 720, dominoCard.getY(), 0.1f, Interpolation.linear),  //0.03667f
                    Actions.hide()
            );

            if (left == right || left + 1 == right) {
                sequenceAction.addAction(Actions.run(runnable));
            }
            sequenceAction.addAction(Actions.removeActor());
            dominoCard.addAction(sequenceAction);

            i++;
            left++;
            right--;
        }
    }

    @Override
    public void animationFinish() {
        for (int i = 0; i < layedIds.size; ++i) {
            DominoCard dominoCard = dominoManagers[layedIds.get(i)].dominoBody;
            if (dominoCard != null) {
                dominoCard.clearActions();
                dominoCard.remove();
            }
        }
    }

    @Override
    public void animationMidStart() {
        for (int i = 0; i < layedIds.size; ++i) {
            DominoCard dominoCard = dominoManagers[layedIds.get(i)].dominoBody;
            dominoCard.setVisible(false);
        }
    }
}
