package com.mobilegame.dominoes.Animation;

import com.badlogic.gdx.math.Interpolation;
import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.Group;
import com.badlogic.gdx.scenes.scene2d.Stage;
import com.badlogic.gdx.scenes.scene2d.actions.Actions;
import com.badlogic.gdx.scenes.scene2d.actions.SequenceAction;
import com.badlogic.gdx.scenes.scene2d.ui.Image;
import com.badlogic.gdx.utils.Align;
import com.badlogic.gdx.utils.Array;
import com.badlogic.gdx.utils.ArrayMap;
import com.mobilegame.dominoes.Actor.AddPointActor;
import com.mobilegame.dominoes.assets.Configuration;
import com.mobilegame.dominoes.handles.DominoManager;
import com.mobilegame.dominoes.handles.GameConfig;
import com.mobilegame.dominoes.handles.PlayerManager;
import com.mobilegame.dominoes.objects.DominoCardBase;
import com.mobilegame.dominoes.objects.DominoCardPlayer2;

/**
 * Created by Doodle on 2021/2/22.
 */

public class FinishAnimation3 extends AnimationBase {
    private final float offsetScaleW = 4;
    private final int rowNum = 4;
    private float offset = 0;

    private PlayerManager[] playerManagers;
    private ArrayMap<Integer, Actor> dominoObjects;
    private DominoManager[] dominoManagers;
    private Stage stage;
    private DominoPosition dp;
    private Image[] pointLabel;
    private int winners;

    private float x = 0;
    private float y = 0;

    private AddPointActor greenPointActor;
    private AddPointActor yellowPointActor;
    private Array<Actor> computerMoveActors;
    private int winPoint = 0;

    private float dominoMoveTime = 0.28f;

//    private boolean isAddPoint = false;
    public static boolean isComputerHandFinishAddAnimations = false;

    public FinishAnimation3(AnimationManager animationManager,
                            PlayerManager[] playerManagers, ArrayMap<Integer, Actor> dominoObjects,
                            DominoManager[] dominoManagers, Stage stage, Image[] processes,
                            int winners) {
        super(animationManager);

        this.playerManagers = playerManagers;
        this.dominoObjects = dominoObjects;
        this.dominoManagers = dominoManagers;
        this.pointLabel = processes;
        this.winners = winners;
        this.stage = stage;
        if (winners == 0) {
            greenPointActor = new AddPointActor(AddPointActor.GREEN, true);
            greenPointActor.setOrigin(Align.center);
            greenPointActor.setPosition(360, 660, Align.center);
            greenPointActor.setScale(0);
            greenPointActor.clearActions();
            greenPointActor.addAction(
                    Actions.scaleTo(1, 1, 0.2f)
            );
            for (int i = 0, s = playerManagers[1].idsInHand.size; i < s; ++i) {
                DominoManager d = dominoManagers[playerManagers[1].idsInHand.get(i)];
                int point = d.values[0] + d.values[1];
                winPoint += point;
            }
        } else {
            yellowPointActor = new AddPointActor(AddPointActor.YELLOW, true);
            yellowPointActor.setOrigin(Align.center);
            yellowPointActor.setPosition(360, 660, Align.center);
            yellowPointActor.setScale(0);
            yellowPointActor.clearActions();
            yellowPointActor.addAction(
                    Actions.scaleTo(1, 1, 0.2f)
            );

            for (int i = 0, s = playerManagers[0].idsInHand.size; i < s; ++i) {
                DominoManager d = dominoManagers[playerManagers[0].idsInHand.get(i)];
                int point = d.values[0] + d.values[1];
                winPoint += point;
            }
        }

        computerMoveActors = new Array<>();
    }

    @Override
    public void animationStart() {
        //判断电脑是否有手牌
        int cHandSize = playerManagers[1].idsInHand.size;
        //电脑有手牌
        if (cHandSize > 0) {
            //置灰电脑手牌
            grayHandDomino(dominoObjects, playerManagers, 1);
            //设置电脑手牌动画时间
            if (cHandSize > 7) {
                dominoMoveTime = 0.14f;
            }
            addComputerAnimation();
            //电脑无手牌
        } else {
            addPlayAnimation();
        }
    }

    private void grayHandDomino(ArrayMap<Integer, Actor> dominoObjects,
                                PlayerManager[] playerManagers,
                                int winner) {
        for (int j = 0; j < playerManagers[winner].dominoesInHand; j++) {
            //置灰操作
            ((DominoCardBase) dominoObjects.get(playerManagers[winner].idsInHand.get(j))).gray(true);
        }
    }

    private void addPlayAnimation() {
        isComputerHandFinishAddAnimations = true;
        for (int i = 0, s = playerManagers[0].idsInHand.size; i < s; ++i) {
            Actor a = dominoObjects.get(playerManagers[0].idsInHand.get(i));
            Group g = a.getParent();
            DominoManager d = dominoManagers[playerManagers[0].idsInHand.get(i)];
            int point = d.values[0] + d.values[1];
            addPointAnimation(i, a.getX(Align.center) + g.getX(), a.getY(Align.center) + g.getY(), point, s);
        }
    }

    private void clearPlayAnimation() {
        for (int i = 0, s = playerManagers[0].idsInHand.size; i < s; ++i) {
            Actor a = dominoObjects.get(playerManagers[0].idsInHand.get(i));
            a.clearActions();
        }
    }

    private void addComputerAnimation() {
        for (int i = 0, s = playerManagers[1].idsInHand.size; i < s; ++i) {
            final Actor a = dominoObjects.get(playerManagers[1].idsInHand.get(i));
            final int finalI = i;
            addAnimation(a, i, new Runnable() {
                @Override
                public void run() {
                    addShowDominoAnimation(a, finalI);
                }
            });
        }
    }

    private void clearComputerAnimation() {
        for (int i = 0, s = playerManagers[1].idsInHand.size; i < s; ++i) {
            Actor a = dominoObjects.get(playerManagers[1].idsInHand.get(i));
            a.clearActions();
            a.setVisible(false);
        }
    }

    private void addAnimation(Actor a, int index, Runnable run) {
        a.clearActions();
        a.addAction(
                Actions.sequence(
                        Actions.delay(index * dominoMoveTime),
                        Actions.moveBy(0, 50, 0.3333f, Interpolation.sineIn),
                        Actions.run(run),
                        Actions.visible(false)
                )
        );
    }

    float sscale = 0.45f;

    private void addPointAnimation(final int index, float startX, float startY, final int point, final int size) {
        final AddPointActor addPointActor;
        final AddPointActor currentPointActor;
        float endX;
        float endY;
        if (winners == 0) {
            addPointActor = new AddPointActor(AddPointActor.GREEN, false);
            endX = greenPointActor.getX(Align.center) - addPointActor.getWidth() / 2f;
            endY = greenPointActor.getY(Align.center) - addPointActor.getHeight() / 2f;
            stage.addActor(addPointActor);
            stage.addActor(greenPointActor);
            currentPointActor = greenPointActor;
//
        } else {
            addPointActor = new AddPointActor(AddPointActor.YELLOW, false);
            endX = yellowPointActor.getX(Align.center) - addPointActor.getWidth() / 2f;
            endY = yellowPointActor.getY(Align.center) - addPointActor.getHeight() / 2f;
            stage.addActor(addPointActor);
            stage.addActor(yellowPointActor);
            currentPointActor = yellowPointActor;
        }
        addPointActor.setOrigin(Align.center);
        addPointActor.setScale(0f);
        addPointActor.setVisible(false);
        addPointActor.setPosition(startX, startY, Align.center);
        addPointActor.setPoint(point + "");

        addPointActor.clearActions();
        SequenceAction sequenceAction = Actions.sequence(

        );


//        if (winners == 0) {
//            sequenceAction.addAction(Actions.delay(0.28f * (size - index)));
//        }

        sequenceAction.addAction(Actions.show());
//        sequenceAction.addAction(Actions.moveBy(0, -150 * fanzhuan, 0.2f));
//        sequenceAction.addAction(Actions.moveBy(0, 25 * fanzhuan, 0.2f));
        sequenceAction.addAction(Actions.scaleTo(0.8f, 0.8f, 0.2f));
        sequenceAction.addAction(Actions.scaleTo(0.6f, 0.6f, 0.2f));
        sequenceAction.addAction(Actions.delay(0.5f));

        sequenceAction.addAction(Actions.parallel(
                Actions.moveBy(20, 50, 0.25f * sscale, Interpolation.sineOut),
                Actions.scaleTo(0.5f, 0.5f, 0.2f * sscale)
        ));
        sequenceAction.addAction(Actions.moveTo(endX, endY, 0.4f, Interpolation.sineOut));
        sequenceAction.addAction(Actions.addAction(
                Actions.scaleTo(1.3f, 1.3f, 0.17f)
                , currentPointActor));
        sequenceAction.addAction(Actions.delay(0.1f));
        sequenceAction.addAction(Actions.run(new Runnable() {
            @Override
            public void run() {
                if (index == 0) {
                    currentPointActor.setPoint(winPoint + "");
                }
            }
        }));
        sequenceAction.addAction(Actions.addAction(
                Actions.scaleTo(1, 1, 0.17f)
                , currentPointActor));
        sequenceAction.addAction(Actions.delay(0.3f, Actions.run(new Runnable() {
            @Override
            public void run() {
                if (index == 0) {
                    if (GameConfig.gameType == GameConfig.ALL_FIVES_TYPE) {
                        winPoint = allfiveScore(winPoint);
                    }
                    currentPointActor.setPoint(winPoint + "");
                }
            }
        })));
        if (index == size - 1) {
            sequenceAction.addAction(Actions.run(new Runnable() {
                @Override
                public void run() {
                    sumPointAnimation();
                }
            }));
        }

        sequenceAction.addAction(Actions.removeActor());

        addPointActor.addAction(sequenceAction);

    }

    private void addPointAnimation(int index, float x, float y, float sw, float sh, final int point, int size) {
        AddPointActor addPointActor;
        if (winners == 0) {
            addPointActor = new AddPointActor(AddPointActor.GREEN, false);
        } else {
            addPointActor = new AddPointActor(AddPointActor.YELLOW, false);
        }
        addPointActor.setOrigin(Align.center);
        addPointActor.setScale(0, 0);
        addPointActor.setPosition(x, y, Align.center);
        addPointActor.setPoint(point + "");

        addPointActor.clearActions();

        float dx;
        float dy;
        if (winners == 0) {
            dx = greenPointActor.getX(Align.center);
            dy = greenPointActor.getY(Align.center);
        } else {
            dx = yellowPointActor.getX(Align.center);
            dy = yellowPointActor.getY(Align.center);
        }

        SequenceAction sequenceAction = Actions.sequence(
                Actions.delay(1, Actions.scaleTo(sw, sh, 0.2f)),
                Actions.delay(0.5f),
                Actions.scaleTo(0.3f, 0.3f, 0.2f),
                Actions.delay(0.2f),
                Actions.moveTo(dx, dy, 0.8f, Interpolation.sineOut),
                Actions.run(new Runnable() {
                    @Override
                    public void run() {
//                        GameConfig.points[winners] += point;
//                        pointLabel[winners].setText(GameConfig.points[winners] + "");
//                        addPointFunc();
                        if (winners == 0) {
                            greenPointActor.setPoint(point + "");
                        } else {
                            yellowPointActor.setPoint(point + "");
                        }

                    }
                })
        );

        if (index == size - 1) {
            sequenceAction.addAction(Actions.run(runnable));
        }

        sequenceAction.addAction(Actions.removeActor());

        addPointActor.addAction(sequenceAction);
        stage.addActor(addPointActor);
    }

    public void addPointFunc() {

    }

    private DominoPosition getDominoPositionForLine(Actor a, float width, int size) {
        if (dp == null) {
            dp = new DominoPosition();
            dp.width = 720 * offsetScaleW / (size * (offsetScaleW + 1) + 1);//计算可以显示的牌宽度
            if (dp.width > width) {
                dp.width = width;
            }
            offset = dp.width / offsetScaleW;

            dp.height = a.getHeight() / a.getWidth() * dp.width;    //通过当前可以显示的牌宽度计算当前牌高度

            float ws = dp.width * size + (size + 1) * offset;
            float hs = dp.height;

            dp.x = Configuration.width / 2f - ws / 2f - Configuration.widthOffset / 2f;
            dp.y = a.getParent().getY() - dp.height * 5 / 4f;
        }
        return dp;
    }

    private DominoPosition getDominoPosition(Actor a, int size) {
        if (dp == null) {
            dp = new DominoPosition();
            dp.width = a.getWidth();

            dp.width = (720 - (size + 1) * offset) / size;  //计算可以显示的牌宽度
            if (dp.width > a.getWidth()) {
                dp.width = a.getWidth();
            }
            dp.height = a.getHeight() / a.getWidth() * dp.width;    //通过当前可以显示的牌宽度计算当前牌高度

            float ws;
            float hs;
            if (size <= rowNum) {
                ws = dp.width * size + (size + 1) * offset;
                hs = dp.height;
            } else {
                int rowN = rowNum;
                int listN = size / rowN;
                if (size % rowN > 0) {
                    listN += 1;
                }

                ws = rowNum * dp.width + (rowNum + 1) * offset;
                hs = listN * dp.height + (listN - 1) * offset;
            }
            dp.x = Configuration.width / 2f - ws / 2f;
            dp.y = 800 + hs / 2f - dp.height;
        }

        return dp;
    }

    private void addShowDominoAnimation(Actor a, final int index) {
        Group computerHandGroup = a.getParent();

        DominoManager d = dominoManagers[playerManagers[1].idsInHand.get(index)];
        int point = d.values[0] + d.values[1];
        final DominoCardPlayer2 b = new DominoCardPlayer2(playerManagers[1].idsInHand.get(index), d.values, null);
        computerMoveActors.add(b);
        b.setPosition(a.getX(), a.getY());
        b.setScale(a.getWidth() / b.getWidth(), a.getHeight() / b.getHeight());
        computerHandGroup.addActor(b);

        DominoPosition dp = getDominoPositionForLine(b, a.getWidth(), playerManagers[1].idsInHand.size);

        x = x + offset;
        final float scaleX = dp.width / b.getWidth();
        final float scaleY = dp.height / b.getHeight();

        final float mx = dp.x + x;
        final float my = dp.y + y;
        b.clearActions();
        final int finalPoint = point;
        SequenceAction sequenceAction = Actions.sequence(
                Actions.parallel(
                        Actions.moveTo(mx - computerHandGroup.getX(), my - computerHandGroup.getY(), 0.2333f),
                        Actions.scaleTo(scaleX, scaleY, 0.2333f)
                ),
                Actions.delay(dominoMoveTime * (playerManagers[1].idsInHand.size - index)),
                Actions.run(new Runnable() {
                    @Override
                    public void run() {
                        isComputerHandFinishAddAnimations = true;
                        if (winners == 0) {
                            b.gray(false);
                            addPointAnimation(index, mx + b.getWidth() / 2f, my + b.getHeight() / 2f, finalPoint, playerManagers[1].idsInHand.size);
                        } else {
                            b.gray(true);
                            if (index == playerManagers[winners].idsInHand.size - 1) {
                                addPlayAnimation();
                            }
                        }
                    }
                })
        );
        b.addAction(sequenceAction);

        x += dp.width;
    }


    private void sumPointAnimation() {
        if (greenPointActor != null) {
            Group parentG = pointLabel[0].getParent();
            float pointLabelX = pointLabel[0].getX() + parentG.getX() - greenPointActor.getWidth() / 2f + 20;
            float pointLabelY = pointLabel[0].getY() + parentG.getY() - greenPointActor.getHeight() / 2f + 25;
            greenPointActor.clearActions();
            greenPointActor.addAction(
                    Actions.sequence(
                            Actions.delay(1f * sscale),
                            Actions.parallel(
                                    Actions.moveTo(pointLabelX, pointLabelY, 0.8f * sscale),
                                    Actions.scaleTo(0.2f, 0.2f, 0.8f * sscale)
                            ),
                            Actions.delay(0.1f, Actions.hide()),
                            Actions.run(
                                    new Runnable() {
                                        @Override
                                        public void run() {
//                                            GameConfig.points[winners] += winPoint;
                                            addPointFunc();
                                        }
                                    }
                            ),
                            Actions.run(runnable)
                    )
            );
        } else if (yellowPointActor != null) {
            Group parentG = pointLabel[1].getParent();
            float pointLabelX = pointLabel[1].getX() + parentG.getX() - yellowPointActor.getWidth() / 2f;
            float pointLabelY = pointLabel[1].getY() + parentG.getY() - yellowPointActor.getHeight() / 2f + 20;
            yellowPointActor.clearActions();
            yellowPointActor.addAction(
                    Actions.sequence(
                            Actions.delay(1f * sscale),
                            Actions.parallel(
                                    Actions.moveTo(pointLabelX, pointLabelY, 0.8f * sscale),
                                    Actions.scaleTo(0.2f, 0.2f, 0.8f * sscale)
                            ),
                            Actions.delay(0.1f, Actions.hide()),
                            Actions.run(
                                    new Runnable() {
                                        @Override
                                        public void run() {
//                                            isAddPoint = true;
//                                            GameConfig.points[winners] += winPoint;
                                            addPointFunc();
                                        }
                                    }
                            ),
                            Actions.run(runnable)
                    )
            );
        }
    }

    private int allfiveScore(int point) {
        int x;

        int a = point % 5;
        int b = point / 5;

        if (a >= 3) {
            b++;
        }
        x = b * 5;

        return x;
    }

    @Override
    public void animationFinish() {
        isComputerHandFinishAddAnimations = false;
//        if (GameConfig.gameType == GameConfig.ALL_FIVES_TYPE) {
//            winPoint = allfiveScore(winPoint);
//        }
//        if (!isAddPoint) {
//            GameConfig.points[winners] += winPoint;
//        }
    }

    private void clearComputerMoveActor() {

        for (int i = 0, s = computerMoveActors.size; i < s; ++i) {
            Actor a = computerMoveActors.get(i);
            a.remove();
        }
        computerMoveActors.clear();
    }

    @Override
    public void animationMidStart() {
        float x = 0;
        float y = 0;
        clearComputerMoveActor();

        int size = playerManagers[1].idsInHand.size;
        if (size > 0) {
            if (size > 7) {
                dominoMoveTime = 0.14f;
            }
            for (int i = 0; i < size; ++i) {
                int id = playerManagers[1].idsInHand.get(i);
                Actor a = dominoObjects.get(id);
                a.clearActions();
                a.setVisible(false);
                Group p = a.getParent();

                DominoManager d = dominoManagers[id];
                int point = d.values[0] + d.values[1];
                final DominoCardPlayer2 b = new DominoCardPlayer2(id, d.values, null);
                p.addActor(b);
                DominoPosition dp = getDominoPositionForLine(b, a.getWidth(), playerManagers[1].idsInHand.size);

                x = x + offset;
                final float scaleX = dp.width / b.getWidth();
                final float scaleY = dp.height / b.getHeight();

                final float mx = dp.x + x;
                final float my = dp.y + y;
                b.clearActions();
                final int finalPoint = point;
                final int finalI = i;
                b.setPosition(mx - p.getX(), my - p.getY());
                b.setScale(scaleX, scaleY);

                SequenceAction sequenceAction = Actions.sequence(
                        Actions.delay(dominoMoveTime),
                        Actions.run(new Runnable() {
                            @Override
                            public void run() {
                                isComputerHandFinishAddAnimations = true;
                                if (winners == 0) {
                                    b.gray(false);
                                    addPointAnimation(finalI, mx + b.getWidth() / 2f, my + b.getHeight() / 2f, finalPoint, playerManagers[1].idsInHand.size);
                                } else {
                                    b.gray(true);
                                    if (finalI == playerManagers[winners].idsInHand.size - 1) {
                                        addPlayAnimation();
                                    }
                                }
                            }
                        })
                );
                b.addAction(sequenceAction);
                x += dp.width;
            }
        } else {
            addPlayAnimation();
        }
    }

    public class DominoPosition {
        public float x;
        public float y;
        public float width;
        public float height;
    }
}
