package com.mobilegame.dominoes.Animation;

import com.badlogic.gdx.math.Interpolation;
import com.badlogic.gdx.math.Vector2;
import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.Group;
import com.badlogic.gdx.scenes.scene2d.actions.Actions;
import com.badlogic.gdx.scenes.scene2d.actions.SequenceAction;
import com.badlogic.gdx.utils.Align;
import com.mobilegame.dominoes.assets.Configuration;

/**
 * Created by Doodle on 2021/2/7.
 */

public class StartGameAnimation extends AnimationBase {
    Vector2[] playPosition;
    Vector2[] computerPosition;

    Group plays;
    Group computers;

    public StartGameAnimation(AnimationManager animationManager, Group[] playerPanels) {
        super(animationManager);
        plays = playerPanels[0];
        computers = playerPanels[1];

        playPosition = new Vector2[plays.getChildren().size];
        computerPosition = new Vector2[computers.getChildren().size];

        initVector2(plays, playPosition);
        initVector2(computers, computerPosition);
    }

    private void initVector2(Group group, Vector2[] vector2s) {
        Actor[] actors = group.getChildren().begin();
        for (int i = 0, n = group.getChildren().size; i < n; i++) {
            vector2s[i] = new Vector2(actors[i].getX(), actors[i].getY());
        }
        group.getChildren().end();
    }

    private void addPlayAnimation() {
        Actor[] actors = plays.getChildren().begin();
        for (int i = 0, n = plays.getChildren().size; i < n; i++) {
            Actor a = actors[i];
            a.setOriginY(Align.center);
            a.setScale(1.05f);
            a.moveBy(-Configuration.width, 0);
            a.clearActions();
            a.addAction(
                    Actions.sequence(
                            Actions.delay(0.15f * (n - i - 1)),
                            Actions.moveBy(+Configuration.width + 18, 0, 0.3f, Interpolation.sineOut),
                            Actions.parallel(
                                    Actions.moveBy(-18, 0, 0.2f),
                                    Actions.scaleTo(1, 1, 0.2f)
                            )
                    )
            );
        }
        plays.getChildren().end();
    }

    private void addComputerAnimation() {
        Actor[] actors = computers.getChildren().begin();
        for (int i = 0, n = computers.getChildren().size; i < n; i++) {
            Actor a = actors[i];
            a.moveBy(-Configuration.width, -600);
            a.clearActions();
            SequenceAction sequenceAction = Actions.sequence(
                    Actions.delay(0.115f * (n - i - 1)),
                    Actions.moveBy(Configuration.width, 700, 0.4333f),
                    Actions.moveBy(0, -100, 0.3333f, Interpolation.sineOut)
            );

            if (i == computers.getChildren().size - 1) {
//                sequenceAction.addAction(Actions.run(runnable));
            }
            a.addAction(sequenceAction);
        }
        computers.getChildren().end();
    }

    @Override
    public void animationStart() {
        addComputerAnimation();
        addPlayAnimation();
    }

    @Override
    public void animationFinish() {

    }

    @Override
    public void animationMidStart() {

    }
}
