package com.mobilegame.dominoes.GameFunction;

import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.g2d.NinePatch;
import com.badlogic.gdx.scenes.scene2d.Group;
import com.badlogic.gdx.scenes.scene2d.actions.Actions;
import com.badlogic.gdx.scenes.scene2d.ui.Image;
import com.badlogic.gdx.scenes.scene2d.ui.Label;
import com.badlogic.gdx.utils.Align;
import com.mobilegame.dominoes.assets.Assets;
import com.mobilegame.dominoes.handles.DominoManager;

/**
 * Created by Doodle on 2020/12/7.
 * 计牌器逻辑
 */

public class DominoNumber {
    private int[] demoNumber;
    private Group showGroup;
    private Label[] numberLabel;
    private Image[] demoBlocks;
    private final Color shad = new Color(1,1,1, 0.3f);
    private final Color curd = new Color(0, 232/255f, 228/255f, 1);
//    private final Color curd = new Color(1, 216/255f, 140/255f, 1);

    public DominoNumber() {
        demoNumber = new int[7];
        numberLabel = new Label[7];
        demoBlocks = new Image[7];
        showGroup = new Group();

        Image dominoBG = new Image(new NinePatch(Assets.design.findRegion("bg"), 1, 12, 12, 12));
        dominoBG.setSize(290, 68);
        showGroup.addActor(dominoBG);

        Label.LabelStyle style1 = new Label.LabelStyle();
        style1.font = Assets.instance.fonts[0];
        style1.fontColor = Color.WHITE;

        float x = 10;
        float y = 8;
        for (int i = 0; i < demoNumber.length; ++i) {
            demoNumber[i] = 7;

            demoBlocks[i] = new Image(Assets.design.findRegion(i+""));
            demoBlocks[i].setPosition(x, y);
            showGroup.addActor(demoBlocks[i]);

            numberLabel[i] = new Label("7", style1);
            numberLabel[i].setAlignment(Align.center);
            numberLabel[i].setPosition(demoBlocks[i].getX(Align.center), demoBlocks[i].getY(Align.top) + 15, Align.center);
            numberLabel[i].setFontScale(0.5f);
            showGroup.addActor(numberLabel[i]);

            x += 10 + demoBlocks[0].getWidth();
        }
    }

    //更新记牌器
    public void updateDemoNumber(DominoManager[] dominoManagers, int pickedId) {
        clearColor();
        if (dominoManagers[pickedId].isDouble()) {
            int index = dominoManagers[pickedId].values[0];
            demoNumber[index]--;
            setCurrColor(index);
        } else {
            int index = dominoManagers[pickedId].values[0];
            demoNumber[index]--;
            setCurrColor(index);
            index = dominoManagers[pickedId].values[1];
            demoNumber[index]--;
            setCurrColor(index);
        }
        updateShow();
    }

    //记牌器展示逻辑
    public Group getShowGroup() {
        return showGroup;
    }

    public void updateShow() {
        for (int i = 0; i < 7; ++i) {
            if(demoNumber[i] <=0){
                setShadColor(i);
                demoNumber[i] = 0;
            }
            numberLabel[i].setText(demoNumber[i] + "");
        }
    }

    private void setShadColor(int index){
        setColor(shad, index);
    }

    private void setCurrColor(int index){
        if(index < 0 || index > 6 || demoNumber[index] <= 0) {
            return;
        }
        demoBlocks[index].clearActions();
        demoBlocks[index].addAction(
                Actions.sequence(
                        Actions.color(curd, 0.2f),
                        Actions.delay(1f),
                        Actions.color(Color.WHITE, 0.2f)
                )

        );

        numberLabel[index].clearActions();
        numberLabel[index].addAction(
                Actions.sequence(
                        Actions.color(curd, 0.2f),
                        Actions.delay(1f),
                        Actions.color(Color.WHITE, 0.2f)
                )
        );
//        final int cu = index;
//        setColor(curd, index);
//        demoBlocks[index].clearActions();
//        demoBlocks[index].addAction(
//                Actions.delay(1f, Actions.run(new Runnable() {
//                    @Override
//                    public void run() {
//                        setColor(Color.WHITE, cu);
//                    }
//                }))
//        );

    }

    public void clearColor(){
        for(int i=0; i<7; ++i){
            setColor(Color.WHITE, i);
        }
    }

    @Override
    public String toString() {
        StringBuffer stringBuffer = new StringBuffer();
        for (int i = 0; i < demoNumber.length; ++i) {
            stringBuffer.append(" index: " + i + " value: " + demoNumber[i]);
        }
        return stringBuffer.toString();
    }

    public void setColor(Color color, int i){
        if(i < 0 || i > 6){
            return;
        }
        numberLabel[i].clearActions();
        numberLabel[i].setColor(color);
        demoBlocks[i].clearActions();
        demoBlocks[i].setColor(color);
    }
}
