package com.mobilegame.dominoes.assets;

import com.badlogic.gdx.utils.IntMap;
import com.mobilegame.dominoes.DominoGame;
import com.mobilegame.dominoes.constants.Constant;

public class SettingData {
    private int coin = 0;
    private int sound;
    private int music;
    private int notification;
    private boolean rate = false;
    private long game_time;
    private long install_time;
    private int orientation;
    private int playAs = 0;
    private int playAsLast = 0;
    private int help;
    private int theme;
    private int level = 1;
    public IntMap<History> history = new IntMap<>();
    private int[] levelState = new int[20];
    private long[] playTime = new long[20];
    private int exitScreen;

    public SettingData() {
        coin = 0;
        levelState[0] = Constant.LEVEL_UNLOCK;
        levelState[Constant.LEVEL_TWO_PLAYERS - 1] = Constant.LEVEL_UNLOCK;
    }

    public boolean isRate() {
        return rate;
    }

    public void rate() {
        this.rate = true;
    }

    public int getSound() {
        return sound;
    }

    public void setSound(int sound) {
        this.sound = sound;
    }

    public int getMusic() {
        return music;
    }

    public void setMusic(int music) {
        this.music = music;
    }

    public int getNotification() {
        return notification;
    }

    public void setNotification(int notification) {
        this.notification = notification;
    }

    public int getLevel() {
        if (Configuration.unlock) return Constant.LEVEL_TOTAL;
        return level;
    }

    public void setExitScreen(int exitScreen) {
        this.exitScreen = exitScreen;
    }

    public int getExitScreen() {
        return exitScreen;
    }

    public void unlockLevel() {
        if (level < Constant.LEVEL_TOTAL) {
            level++;
            levelState[level - 1] = Constant.LEVEL_UNLOCK;
        }
    }

    public int getLevelState(int level) {
        if (level < 1 || level > Constant.LEVEL_TOTAL) return -1;
        return levelState[level - 1];
    }

    public void setLevelState(int level, int state) {
        if (level < 1 || level > Constant.LEVEL_TOTAL) return;
        if (state == Constant.LEVEL_WIN && levelState[level - 1] == Constant.LEVEL_ENTER) {
            levelState[level - 1] = Constant.LEVEL_FIRST_WIN;
        } else {
            levelState[level - 1] = state;
        }
    }

    public void addHistory(int level, History value) {
        history.put(level, value);
    }

    public void removeHistory(int level) {
        history.remove(level);
    }

    public History getHistory(int level) {
        return history.get(level);
    }

    public void setOrientation(int orientation) {
        this.orientation = orientation;
    }

    public int getOrientation() {
        return orientation;
    }

    public boolean cost(int cost) {
        if (coin >= cost) {
            coin -= cost;
            Configuration.saveData();
            return true;
        }
        return false;
    }

    public int getCoin() {
        if (coin > 9999999) {
            return 9999999;
        }
        return coin;
    }

    public void setPlayAs(int playAs) {
        this.playAs = playAs;
    }

    public int getPlayAs() {
        return playAs;
    }

    public void addCoin(int offset) {
        if (offset > 0) {
            coin += offset;
            Configuration.saveData();
        }
    }

    public void setPlayTime(int level, long value) {
        if (value < 0) return;
        if (playTime[level] == 0 || value < playTime[level]) {
            this.playTime[level] = value;
        }
    }

    public long getPlayTime(int level) {
        return playTime[level];
    }

    public void setTheme(int theme) {
        this.theme = theme;
    }

    public int getTheme() {
        return theme;
    }

    public int getHelp() {
        return help;
    }

    public void setHelp(int help) {
        this.help = help;
    }

    public long getInstall_time() {
        return install_time;
    }

    public void setInstall_time(long install_time) {
        this.install_time = install_time;
    }

    public long getGame_time() {
        return game_time;
    }

    public void setGame_time(long game_time) {
        this.game_time = game_time;
        if(install_time == 0){
            install_time = game_time;
        }
    }

    public void setPlayAsLast(int playAsLast) {
        this.playAsLast = playAsLast;
    }

    public int getPlayAsLast() {
        return playAsLast;
    }

    @Override
    public String toString() {
        try {
            return DominoGame.getJson().toJson(this);
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }

}
