package com.mobilegame.dominoes.audio;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.files.FileHandle;
import com.mobilegame.dominoes.assets.Assets;
import com.mobilegame.dominoes.data.SettingDataUtil;

import java.util.ArrayDeque;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Queue;
import java.util.Random;
import java.util.concurrent.ArrayBlockingQueue;

public class AudioProcess {

    public static HashMap<String, SoundAsset> soundAssets = new HashMap<>();
    public static HashMap<String, MusicAsset> musicAssets = new HashMap<>();
    public static HashMap<String, String> musics = new HashMap<>();
    private static String currMusic = "";
    private static HashSet<String> currSoundLoop = new HashSet<>();
    private static boolean onFocus = true;
    private static boolean prepared = false;

    public static void prepare() { // 准备哪些资源将要加载
        clear();
        loadMusicFromData();
        loadSound();

        soundQueue = new ArrayBlockingQueue<>(2);// 4
        musicQueue = new ArrayDeque<>();
        new SoundThread().start();
        prepared = true;

    }

    public static boolean isPrepared() {
        return prepared;
    }

    public static void setOnFocus(boolean flag) {
//        Log.log("AudioProcess", "setOnFocus()--->" + flag);
        onFocus = flag;
        if (!prepared) return;
        if (onFocus) {
            resumeMusic();
            resumeAllSound();
        } else {
            pauseAllSound();
            pauseAllMusic();
        }
    }

    public static void setPitch(long soundId, float pitch, String name) {
        SoundAsset sa = getSoundAsset(name);
        if (sa != null) {
            sa.setPitch(soundId, pitch);
        }
    }

    public static void setVolume(long soundId, float pitch, String name) {
        SoundAsset sa = getSoundAsset(name);
        if (sa != null) {
            sa.setVolume(soundId, pitch);
//            sa.setPitch(soundId, pitch);
        }
    }


    public static SoundAsset getSoundAssets(String name) {
        return soundAssets.get(name);
    }

    static class SoundThread extends Thread {
        @Override
        public void run() {
            super.run();
            loadSound();
        }
    }


    public static void loadSound() {
        soundAssets.put("change_dec", Asset.registerSoundAsset("audio/sound/change_dec.ogg"));
        soundAssets.put("deck_appear", Asset.registerSoundAsset("audio/sound/deck_appear.ogg"));
        soundAssets.put("draw0", Asset.registerSoundAsset("audio/sound/draw0.ogg"));
        soundAssets.put("draw1", Asset.registerSoundAsset("audio/sound/draw1.ogg"));
        soundAssets.put("drawboneyard0", Asset.registerSoundAsset("audio/sound/drawboneyard0.ogg"));
        soundAssets.put("gameEnd", Asset.registerSoundAsset("audio/sound/gameEnd.ogg"));
        soundAssets.put("handEnd", Asset.registerSoundAsset("audio/sound/handEnd.ogg"));
        soundAssets.put("locked", Asset.registerSoundAsset("audio/sound/locked.ogg"));
        soundAssets.put("score", Asset.registerSoundAsset("audio/sound/score.ogg"));
        soundAssets.put("unlock", Asset.registerSoundAsset("audio/sound/unlock.ogg"));
        soundAssets.put("button", Asset.registerSoundAsset("audio/sound/button.ogg"));
        soundAssets.put("tiger", Asset.registerSoundAsset("audio/sound/tiger.ogg"));
        Iterator<SoundAsset> iterator = soundAssets.values().iterator();
        while (iterator.hasNext()) {
            iterator.next().load();
        }
    }

    private static long soundStartTime = 0;
    private static final long soundTimeSpan = 100;

    public static Queue<String> soundQueue;
    public static Queue<String> musicQueue;

    public static void playSound(String name, float v) {
        if (name != null) {
            playsound(name, false);
        }
    }
    public static void playSound(String name) {
        if (name != null) {
            playsound(name, false);
        }
    }


    public static long playSoundLoop(String name,float v) {
        stopSoundLoop(name);
        if (name != null) {
            if (currSoundLoop.add(name)) {
                return playsound(name, true,v);
            }
        }
        return -1;
    }

    public static void stopSoundLoop(String name) {
        currSoundLoop.remove(name);
        if (name != null) {
            if (soundAssets.get(name) != null) {
                soundAssets.get(name).stop();
            }
        }
    }


    public static void stopAllSound() {
        Iterator<String> iterator = currSoundLoop.iterator();
        while (iterator.hasNext()) {
            String soundName = iterator.next();
            if (soundName != null) {
                if (soundAssets.get(soundName) != null) {
                    soundAssets.get(soundName).stop();
                }
            }
        }
        currSoundLoop.clear();
    }


    public static void pauseAllSound() {
        Iterator<String> iterator = currSoundLoop.iterator();
        while (iterator.hasNext()) {
            String soundName = iterator.next();
            if (soundName != null) {
                if (soundAssets.get(soundName) != null) {
                    soundAssets.get(soundName).pause();
                }
            }
        }
    }


    public static void resumeAllSound() {
        Iterator<String> iterator = currSoundLoop.iterator();
        while (iterator.hasNext()) {
            String soundName = iterator.next();
            if (soundName != null) {
                if (soundAssets.get(soundName) != null) {
                    soundAssets.get(soundName).resume();
                }
            }
        }
    }

    public static void playAllSound() {
        Iterator<String> iterator = currSoundLoop.iterator();
        while (iterator.hasNext()) {
            String soundName = iterator.next();
            if (soundName != null) {
                if (soundName.contains("draw")) {
                    playsound(soundName, true);
                } else {
                    playsound(soundName, true);
                }
            }
        }
    }

    private static long playsound(String name, boolean loop) {
        if (soundQueue == null)
            return -1;

        if (!(SettingDataUtil.getSound())) return -1;

        SoundAsset sa = getSoundAsset(name);
        if (!loop) {
//            Gdx.app.log("audio3",name);
            if (!soundQueue.offer(name)) {
//                Gdx.app.log("audio4",name);
                return -1;
            }
        } else {

            return sa.loop();
        }
        return -1;
    }

    private static long playsound(String name, boolean loop,float v) {
        if (soundQueue == null)
            return -1;
        if (!(SettingDataUtil.getSound() )) return -1;
        SoundAsset sa = getSoundAsset(name);
        if(sa!=null){
            sa.setVolume(v);
        }
        if (!loop) {
            if (!soundQueue.offer(name)) {
                return -1;
            }
        } else {
            return sa.loop();
        }
        return -1;
    }


    public static void update() {
        if (soundQueue.isEmpty())
            return;
        if (System.currentTimeMillis() - soundStartTime > soundTimeSpan) {
            String name = soundQueue.poll();// soundQueue.clear();
            SoundAsset sa = getSoundAsset(name);
            if (sa != null) {
                sa.play();
            }
            soundStartTime = System.currentTimeMillis();
        }
    }

    private static SoundAsset getSoundAsset(String name) {
        return soundAssets.get(name);

    }

    public static void reset() {
        if (soundQueue != null) {
            soundQueue.clear();
        }
    }

    public static void clear() {
        if (soundQueue != null) {
            soundQueue.clear();
        }
        soundAssets.clear();
        musicAssets.clear();
        prepared = false;
        currMusic = "";
    }


    private static void pausemusic(String musicName) {
        musicAssets.get(musicName).pauseMusic();
    }

    public static void stopMusic() {
        if (currMusic != null) {
            stopmusic(currMusic);
            //           currMusic = "";
        }
    }

    public static void playMusic(){

//        if(musicQueue.size()<6){
//            inputMusicqueue();
//        }

        if(!isMusicPlaying()){
            String str = "lps";
            if(musicAssets.get(str)!=null) {
//                Gdx.app.log("test1",str);
                musicAssets.get(str).playMusic();
                currMusic = str;
            }
        }

    }

    public static void pauseMusic(){

        if(musicAssets.get(currMusic) != null){
            musicAssets.get(currMusic).pauseMusic();
        }


    }

    public static void resumeMusic(){

        if(musicAssets.get(currMusic) != null){
            musicAssets.get(currMusic).resumeMusic();
        }

    }



    private static void inputMusicqueue(){

        int x = (15 - musicQueue.size())/3;

        Random r = new Random();

        ArrayList<Integer> arrayList = new ArrayList<>();

        for(int i = 0 ;i < x;i++){

            for (int m = 0;m < 5;m++){
                arrayList.add(m);
            }

            for(int j = 0 ;j < 5;j++){

                if(arrayList.size()>0){
                    int tar = r.nextInt(arrayList.size());
//                    String s = "bgm"+arrayList.get(tar);
                    String s = "lps";
                    arrayList.remove(tar);
                    musicQueue.offer(s);
                }

            }
        }

    }

    private static boolean isMusicPlaying(){

        Iterator<MusicAsset> iterator = musicAssets.values().iterator();
        while (iterator.hasNext()) {
            if(  iterator.next().isPlaying()){
                return true;
            }
        }
        return false;
    }

    public static void stopMusic(String name) {
        if (!currMusic.equals(name))
            return;
        String musicName = musics.get(name);
        if (musicName != null) {
            stopmusic(musicName);
        }

    }

    private static void stopmusic(String musicName) {
        if (musicAssets.get(musicName) != null) {
            musicAssets.get(musicName).stopMusic();
//            musicAssets.get(musicName).dispose(Assets.instance.assetManager);
        }
    }

    public static void pauseAllMusic() {
        Iterator<MusicAsset> iteratorMusic = musicAssets.values().iterator();
        while (iteratorMusic.hasNext()) {
            iteratorMusic.next().pauseMusic();
        }
    }

    public static void stopAllMusic() {
        Iterator<MusicAsset> iteratorMusic = musicAssets.values().iterator();
        while (iteratorMusic.hasNext()) {
            iteratorMusic.next().stopMusic();
        }
//        currMusic = "";
    }

    public static FileHandle[] getAllSoundName() {
        return Gdx.files.internal("sfx").list();
    }

    public static FileHandle[] getAllMusciName() {
        return Gdx.files.internal("music").list();
    }

    public static void destory() {
        for (int i = 0; i < musicAssets.size(); i++) {
            musicAssets.get(i).music.dispose();
        }

        for (int i = 0; i < soundAssets.size(); i++) {
            soundAssets.get(i).sound.dispose();
        }
    }

    private static void loadMusicFromData() {

        try {
            musics.put("lps", "audio/bgm/lps.ogg");

            MusicAsset ml = new MusicAsset(musics.get("lps"));
            musicAssets.put("lps", ml);


            Iterator<MusicAsset> iterator = musicAssets.values().iterator();
            while (iterator.hasNext()) {
                iterator.next().load();
            }

        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
