package com.mobilegame.dominoes.audio;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.assets.AssetManager;
import com.badlogic.gdx.audio.Music;
import com.mobilegame.dominoes.DominoGame;

public class MusicAsset extends Asset {
    Music music;
    boolean load = false;

    public MusicAsset(Music music, String name) {
        this.music = music;
        this.name = name;
    }

    public MusicAsset(String name) {
        this.name = name;
//        music = Gdx.audio.newMusic(Gdx.files.internal(name));
    }

    public void playMusicLoop() {
        try {
            if (!music.isLooping()) {
                music.setLooping(true);
            }
            music.setVolume(0.4F);
            music.play();
        } catch (Exception e) {
            e.printStackTrace();
        }

    }

    public void playMusic() {
        try {
            if(music != null){
                music.dispose();
            }
            music = Gdx.audio.newMusic(Gdx.files.internal(name));
            music.setVolume(0.7F);
            music.play();
            music.setLooping(true);
//            music.setOnCompletionListener(new Music.OnCompletionListener(){
//                @Override
//                public void onCompletion(Music music) {
//                    playMusic();
//                }
//            });
        } catch (Exception e) {
            e.printStackTrace();
        }

    }

    public void pauseMusic() {
        try {
            if (music != null) {
                music.pause();
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void resumeMusic() {
        try {
            if (music != null) {
                music.play();
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void stopMusic() {
        try {
            if (music != null) {
                music.stop();
                music.dispose();
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public boolean isPlaying(){

        if(music!=null){
            return music.isPlaying();
        }
        else {
            return false;
        }
    }

    @Override
    public void load() {
        // TODO Auto-generated method stub
        if (load)
            return;
        load = true;
        music = Gdx.audio.newMusic(Gdx.files.internal(name));
    }

    @Override
    public void loading(AssetManager assetManager) {
        // TODO Auto-generated method stub
        if (assetManager.isLoaded(name, Music.class)) {
            assetManager.unload(name);
            assetManager.load(name, Music.class);
        } else {
            assetManager.load(name, Music.class);
        }
    }

    @Override
    public void finished(AssetManager assetManager) {
        // TODO Auto-generated method stub
        if (assetManager.isLoaded(name)) {
            music = assetManager.get(name, Music.class);
        }
    }

    @Override
    public void dispose(AssetManager assetManager) {
        if (music != null) {
            music.dispose();
            music = null;
            load = false;
            //	Log.log("Music", "diapose new music: " + name);
        }
    }

}
