package com.mobilegame.dominoes.data;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.Preferences;
import com.mobilegame.dominoes.assets.Configuration;
import com.mobilegame.dominoes.constants.Constant;


public class SettingDataUtil {
    private static Preferences preferences;
    private static boolean prepared = false;

    public static void prepare() {
        preferences = Gdx.app.getPreferences("setting_data");
        prepared = true;
    }

    public static boolean isRate() {
        if(!prepared) return false;
        return preferences.getBoolean("rate", false);
    }

    public static void rate() {
        if(!prepared) return ;
        preferences.putBoolean("rate", true);
        preferences.flush();
    }

    public static boolean getSound() {
        if(preferences == null)
            preferences = Gdx.app.getPreferences("setting_data");
        return preferences.getBoolean("sound",true);
    }

    public static void setSound(boolean sound) {
        if(preferences == null)
            preferences = Gdx.app.getPreferences("setting_data");
        preferences.putBoolean("sound", sound);
        preferences.flush();
    }

    public static boolean getMusic() {
        if(preferences == null)
            preferences = Gdx.app.getPreferences("setting_data");
        return preferences.getBoolean("music",true);
    }

    public static void setMusic(boolean sound) {
        if(preferences == null){
            preferences = Gdx.app.getPreferences("setting_data");
        }
        preferences.putBoolean("music", sound);
        preferences.flush();
    }

    public static int getNotification() {
        if(!prepared) return Constant.ON;
        return preferences.getInteger("notification", Constant.ON);
    }

    public static void setNotification(int notification) {
        if(!prepared) return;
        preferences.putInteger("notification", notification);
        preferences.flush();
    }

    public static void setPlayAs(int playAs) {
        if(!prepared) return;
        preferences.putInteger("playAs", playAs);
        preferences.flush();
    }

    public static int getPlayAs() {
        if(!prepared) return Constant.WHITE;
        return preferences.getInteger("playAs", Constant.ON);
    }

    public static void setPlayTime(int level, long value) {
        if(!prepared) return;
        if (value <= 0) return;
        long old = preferences.getLong("game_time_" + level, 0);
        if (old == 0 || value < old) {
            preferences.putLong("game_time_" + level, value);
        }
        preferences.flush();
    }

    public static long getPlayTime(int level) {
        return preferences.getLong("game_time_" + level, 0);
    }

    public static void setExitScreen(int exitScreen) {
        if(!prepared) return;
        preferences.putInteger("exit_screen", exitScreen);
        preferences.flush();
    }

    public static int getExitScreen() {
        if(!prepared) return Constant.MENUSCREEN;
        return preferences.getInteger("exit_screen", Constant.MENUSCREEN);

    }
    public static void unlockLevel(int level) {
        if (level < Constant.LEVEL_TOTAL) {
            level++;
            if(getLevelState(level)==Constant.LEVEL_LOCK) {
                setLevelState(level, Constant.LEVEL_UNLOCK);
            }
        }
        preferences.flush();
    }

    public static int getLevelState(int level) {
        if (level < 1 || level > Constant.LEVEL_TOTAL) return -1;
        int levelState = preferences.getInteger("level_state_" + level, Constant.LEVEL_LOCK);
        if ((level == 1 || level == Constant.LEVEL_TWO_PLAYERS) && levelState == Constant.LEVEL_LOCK) {
            preferences.putInteger("level_state_" + level, Constant.LEVEL_UNLOCK);
            return Constant.LEVEL_UNLOCK;
        }
        if(Configuration.unlock && levelState == Constant.LEVEL_LOCK){
            return Constant.LEVEL_UNLOCK;
        }
        boolean wrong = false;
        if(level>1 && level<=Constant.LEVEL_TOTAL){
            int preLevelState = preferences.getInteger("level_state_"+(level-1),Constant.LEVEL_LOCK);
            if(levelState != Constant.LEVEL_LOCK &&(preLevelState!=Constant.LEVEL_WIN && preLevelState !=Constant.LEVEL_FIRST_WIN)){
                wrong = true;
            }
        }
        if(wrong){
            setLevelState(level,Constant.LEVEL_LOCK);
            return Constant.LEVEL_LOCK;
        }

        return levelState;
    }

    public static void setLevelState(int level, int state) {
        if (level < 1 || level > Constant.LEVEL_TOTAL) return;
        if (state == Constant.LEVEL_WIN && getLevelState(level) == Constant.LEVEL_ENTER) {
            preferences.putInteger("level_state_" + level, Constant.LEVEL_FIRST_WIN);
        } else {
            preferences.putInteger("level_state_" + level, state);
        }
        preferences.flush();
    }

    public static void setPlayAsLast(int playAsLast) {
        preferences.putInteger("playAsLast", playAsLast);
        preferences.flush();
    }

    public static int getPlayAsLast() {
        return preferences.getInteger("playAsLast", Constant.WHITE);
    }

    public static long getInstall_time() {
        return preferences.getLong("install_time", 0);
    }

    public static long getGame_time() {
        return preferences.getLong("game_time", 0);
    }

    public static void setGame_time(long game_time) {
        preferences.putLong("game_time", game_time);
        long install_time = getInstall_time();
        if (install_time == 0) {
            setInstallTime(game_time);
        }
        preferences.flush();
    }
    public static void setInstallTime(long installTime){
        preferences.putLong("install_time", installTime);
        preferences.flush();
    }

    public static void setTheme(int theme) {
        preferences.putInteger("theme_id", theme);
        preferences.flush();
    }

    public static int getTheme() {
        return preferences.getInteger("theme_id",0);
    }

    public static int getHelp() {
        return preferences.getInteger("help",Constant.ON);
    }

    public static void setHelp(int help) {
        preferences.putInteger("help",help);
        preferences.flush();
    }

    public static void setOrientation(int orientation) {
        preferences.putInteger("orientation",orientation);
        preferences.flush();
    }

    public static int getOrientation() {
        if(!prepared) return Constant.LANDSCAPE;
        return preferences.getInteger("orientation",Constant.LANDSCAPE);
    }


    public static void reset(){
        preferences.clear();
    }

    public static void flush(){
        preferences.flush();
    }

}
