package com.mobilegame.dominoes.dialogs;

import com.badlogic.gdx.math.Interpolation;
import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.Group;
import com.badlogic.gdx.scenes.scene2d.InputEvent;
import com.badlogic.gdx.scenes.scene2d.Touchable;
import com.badlogic.gdx.scenes.scene2d.actions.Actions;
import com.badlogic.gdx.scenes.scene2d.utils.ClickListener;
import com.badlogic.gdx.utils.Align;
import com.badlogic.gdx.utils.Array;
import com.mobilegame.dominoes.DominoGame;
import com.mobilegame.dominoes.assets.Configuration;
import com.mobilegame.dominoes.utils.ActorUtil;
import com.mobilegame.dominoes.utils.LayerUtils;
import com.mobilegame.dominoes.utils.listener.ButtonListener2;

import java.util.Iterator;

/**
 * Created by lilexiang on 17-8-23.
 */
public class BaseDialog implements DialogInterface {
    protected String TAG = "";
    protected Actor layer;
    Group group;
    public Actor closeArea;
    protected Array<DialogListener> listeners = new Array<>();
    protected float layerAlpha = 0.7f;

    public BaseDialog(String name) {
        if (name != null && !name.equals("")) {
            group = DominoGame.parseScene(name);
            group.setTouchable(Touchable.childrenOnly);
        } else {
            group = new Group();
        }
        group.setPosition(360, 640, Align.center);
        init();
        layout();
    }

    protected void init() {
        layer = LayerUtils.createLayer(layerAlpha);
        layer.setName("layer");
        layer.setTouchable(Touchable.enabled);
        layer.addListener(new ClickListener() {
            @Override
            public void touchUp(InputEvent event, float x, float y, int pointer, int button) {
                super.touchUp(event, x, y, pointer, button);
//                close();
            }
        });
        Actor bg = group.findActor("bg");
        if (bg != null) bg.setTouchable(Touchable.enabled);
        final Actor btnClose = group.findActor("btn_close");
        if (btnClose != null) {
            btnClose.setTouchable(Touchable.disabled);
            closeArea = ActorUtil.getTouchArea(btnClose, 40);
            group.addActor(closeArea);
            closeArea.setTouchable(Touchable.enabled);
            closeArea.addListener(new ButtonListener2() {
                @Override
                public void clicked(InputEvent event, float x, float y) {
                    super.clicked(event, x, y);
                    close();
                }

                @Override
                public boolean touchDown(InputEvent event, float x, float y, int pointer, int button) {
                    btnClose.addAction(
                            Actions.sequence(
                                    Actions.scaleTo(0.8f, 0.8f)
                            ));
                    return super.touchDown(event, x, y, pointer, button);

                }

                @Override
                public void touchUp(InputEvent event, float x, float y, int pointer, int button) {
                    btnClose.addAction(
                            Actions.sequence(
                                    Actions.scaleTo(1, 1)
                            ));
                    super.touchUp(event, x, y, pointer, button);
                }
            });
        }
    }

    public void layout() {
        layer.setSize(Configuration.width, Configuration.height);
        layer.setPosition(360, 640, Align.center);
    }

    final public void setGroup(Group parent) {
        parent.addActor(layer);
        parent.addActor(group);
        show();
        update();
    }

    final public void setGroupBottom(Group parent) {
        parent.addActor(layer);
        parent.addActor(group);
        showBottom();
        update();
    }

    protected void showBottom() {
        DominoGame.pauseInputProcessor();
//        group.setScale(0.3F, 0.3F);
//        group.setTouchable(Touchable.childrenOnly);
//        group.setOrigin(Align.center);
//        start();
//        group.addAction(
//                Actions.sequence(
//                        Actions.moveBy(0,-700,0.01f),
//                        Actions.moveBy(0,700,0.25f),
//                        Actions.run(new Runnable() {
//                            @Override
//                            public void run() {
//                                start();
//                            }
//                        }
//                        )
//                )
//        );
    }

    protected void show() {
        DominoGame.pauseInputProcessor();
//        layer.addAction(Actions.alpha(layerAlpha));
        layer.addAction(
                Actions.alpha(0.75f, 0.3f)
        );
        group.setScale(0.3F, 0.3F);
        group.setTouchable(Touchable.childrenOnly);
        group.setOrigin(Align.center);
        group.addAction(Actions.sequence(Actions.scaleTo(1, 1, 0.3F, Interpolation.swingOut), Actions.run(new Runnable() {
            @Override
            public void run() {
                start();
            }
        })));
    }

    protected void start() {
        DominoGame.resumeInputProcessor();
    }


    public void update() {

    }

    public void bottomClose() {
        layer.remove();
        group.remove();
        Iterator<DialogListener> iterator = listeners.iterator();
        while (iterator.hasNext()) {
            iterator.next().closed();
        }
    }

    public void close() {
        layer.addAction(
                Actions.fadeOut(0.3f)
        );
        group.addAction(
                Actions.sequence(
                        Actions.scaleTo(0.1f, 0.1f, 0.2F),
                        Actions.run(new Runnable() {
                            @Override
                            public void run() {
                                group.remove();
                                layer.remove();
                            }
                        })));
//        layer.remove();
        Iterator<DialogListener> iterator = listeners.iterator();
        while (iterator.hasNext()) {
            iterator.next().closed();
        }
    }


    public void addListener(DialogListener listener) {
        listeners.add(listener);
    }

    @Override
    public void resize(int width, int height) {
    }
}
