package com.mobilegame.dominoes.dialogs;

import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.g2d.NinePatch;
import com.badlogic.gdx.math.MathUtils;
import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.Group;
import com.badlogic.gdx.scenes.scene2d.InputEvent;
import com.badlogic.gdx.scenes.scene2d.actions.Actions;
import com.badlogic.gdx.scenes.scene2d.ui.Image;
import com.badlogic.gdx.scenes.scene2d.ui.Table;
import com.badlogic.gdx.scenes.scene2d.utils.ClickListener;
import com.badlogic.gdx.utils.Align;
import com.badlogic.gdx.utils.Array;
import com.badlogic.gdx.utils.IntArray;
import com.mobilegame.dominoes.DominoGame;
import com.mobilegame.dominoes.assets.Assets;
import com.mobilegame.dominoes.assets.Configuration;
import com.mobilegame.dominoes.audio.AudioManager;
import com.mobilegame.dominoes.data.Classes.Design;
import com.mobilegame.dominoes.data.DiamondDataUtil;
import com.mobilegame.dominoes.data.GameDataCsv;
import com.mobilegame.dominoes.handles.GameConfig;
import com.mobilegame.dominoes.utils.MyRandom;

public class BoneYard extends Group {
    Table[] table = new Table[GameDataCsv.getDesignDominoesSize()];
    Array<Image>[] dominoes = new Array[GameDataCsv.getDesignDominoesSize()];

    private Image[] bgImage;
    private int board_id[];
    private Image shadow;
    private Actor block;

    private Group actionGroup;

    IntArray marks = new IntArray();
    private BoneYardListener listener;

    public static boolean isSelect = true;

    public BoneYard(IntArray dominoIds, BoneYardListener listener) {
        this.listener = listener;
        actionGroup = new Group();
        actionGroup.setWidth(Configuration.width);
        marks.addAll(dominoIds);

        bgImage = new Image[GameDataCsv.getDesignBgSize()];
        board_id = new int[GameDataCsv.getDesignBgSize()];

        shadow = new Image(new NinePatch(Assets.instance.design.findRegion("card_bg")));
        shadow.setColor(1, 1, 1, 0);
        shadow.setSize(Configuration.width * 2, Configuration.height * 2);

        addActor(shadow);
        addActor(actionGroup);

        block = new Actor();
        block.setSize(Configuration.width, Configuration.height);
        int n = 0;
        for (int i = 1; i <= GameDataCsv.getDesignSize(); i++) {
            Design design = GameDataCsv.getDesign(i);
            if (design.type.contains("background")) {
//                bgImage[n] = new Image(new NinePatch(Assets.instance.design.findRegion(design.Topic_name+"_boneyard"),20,20,30,30));
                bgImage[n] = new Image(new NinePatch(Assets.instance.design.findRegion("boneYardBG"), 45, 45, 45, 45));
                bgImage[n].setColor(setColor(design.ID));
                board_id[n] = design.ID;
                bgImage[n].setSize(720, 441);
                bgImage[n].setX((actionGroup.getWidth() - bgImage[n].getWidth()) / 2f);
                bgImage[n].moveBy(1, -15);
                bgImage[n].setVisible(false);
                actionGroup.addActor(bgImage[n]);
                n++;
            }
        }
        setSize(Configuration.width, 441);

        initDominoes();
        showBg(DominoGame.bgNum);
    }

    private Color setColor(int topicID) {
        Color color = new Color();
        if (topicID == 10) {
            color.r = 22 / 255f;
            color.g = 86 / 255f;
            color.b = 158 / 255f;
            color.a = 1;
        } else if (topicID == 11) {
            color.r = 14 / 255f;
            color.g = 124 / 255f;
            color.b = 138 / 255f;
            color.a = 1;
        } else if (topicID == 12) {
            color.r = 124 / 255f;
            color.g = 36 / 255f;
            color.b = 98 / 255f;
            color.a = 1;
        } else if (topicID == 13) {  //hei
            color.r = 33 / 255f;
            color.g = 37 / 255f;
            color.b = 41 / 255f;
            color.a = 1;
        } else if (topicID == 14) { //pi
            color.r = 119 / 255f;
            color.g = 62 / 255f;
            color.b = 25 / 255f;
            color.a = 1;
        } else {
            color.r = 173 / 255f;
            color.g = 99 / 255f;
            color.b = 38 / 255f;
            color.a = 1;
        }
        return color;
    }

    private void initDominoes() {
        if (marks.size < 14) {
            while (marks.size < 14) {
                marks.add(-1);
            }
            int count = marks.size;
            int num = count;
            while (num > 1) {
                num--;
                int index = MyRandom.getInstance().nextInt(num + 1);
                int value = marks.items[index];
                marks.items[index] = marks.items[num];
                marks.items[num] = value;
            }
        }

        for (int j = 0; j < GameDataCsv.getDesignDominoesSize(); j++) {
            table[j] = new Table();
            dominoes[j] = new Array<Image>();
        }

        for (int i = 0; i < marks.size; i++) {

            final Image[] image = new Image[GameDataCsv.getDesignDominoesSize()];

            for (int j = 0; j < GameDataCsv.getDesignDominoesSize(); j++) {

                Design design = GameDataCsv.getDesign(j + 1);
                image[j] = new Image(Assets.instance.dominoes.findRegion(design.Topic_name + "-board-dominoes2"));
                dominoes[j].add(image[j]);
                table[j].add(image[j]).width(image[j].getWidth()).height(image[j].getHeight()).pad(4);
            }

            if (i == MathUtils.ceil(marks.size / 2) - 1) {
                for (int j = 0; j < GameDataCsv.getDesignDominoesSize(); j++) {
                    table[j].row();
                }
            }
            final int index = i;
            final int dominoId = marks.get(i);
            if (dominoId == -1) {
                for (int j = 0; j < GameDataCsv.getDesignDominoesSize(); j++) {
                    image[j].setVisible(false);
                }
            }

            for (int j = 0; j < GameDataCsv.getDesignDominoesSize(); j++) {
                image[j].addListener(new ClickListener() {
                    @Override
                    public boolean touchDown(InputEvent event, float x, float y, int pointer, int button) {
                        if (pointer > 0 || !isSelect) {
                            return false;
                        }
                        AudioManager.PlaySound(AudioManager.AudioType.drawboneyard0);
                        selected(index, dominoId, (Image) event.getListenerActor());
                        return super.touchDown(event, x, y, pointer, button);
                    }
                });
            }

        }

        for (int j = 0; j < GameDataCsv.getDesignDominoesSize(); j++) {
            table[j].setPosition(getWidth() / 2, getHeight() / 2);
            actionGroup.addActor(table[j]);
        }
    }

    private void selected(int index, int id, Image image) {

        Image[] selectImage = new Image[dominoes.length];
        for (int i = 0; i < dominoes.length; i++) {
            selectImage[i] = dominoes[i].get(index);

            if (selectImage[i] != null) {
                selectImage[i].setVisible(false);
            }
        }

        listener.selected(id, image);

    }

    public void show(IntArray dominoIds) {

        for (int i = 0; i < dominoes.length; i++) {
            if (DiamondDataUtil.getUseDesign(DesignDialogNew.DesignType.dominoes) == i + 1) {
                table[i].setVisible(true);
            } else {
                table[i].setVisible(false);
            }
        }

        for (int j = 0; j < dominoes.length; j++) {
            for (Image image : dominoes[j]) {
                image.setVisible(false);
            }
        }

        for (int i = 0; i < dominoIds.size; i++) {
            int index = marks.indexOf(dominoIds.get(i));
            if (index != -1) {
                Image[] temp = new Image[dominoes.length];
                for (int j = 0; j < dominoes.length; j++) {
                    temp[j] = dominoes[j].get(index);
                    if (temp[j] != null) {
                        temp[j].setVisible(true);
                    }
                }
            }
        }
        setPosition(Configuration.width / 2, Configuration.height / 2 + 50, Align.center);
        shadow.setPosition(-getX(), -getY());

    }

    public void showAction() {
        removeActor(block);
        actionGroup.setX(-Configuration.width - 200);
//        shadow.addAction(
//                Actions.sequence(
//                        Actions.fadeOut(0.01f),
//                        Actions.delay(GameConfig.aniTime),
//                        Actions.fadeIn(0.01f)
//                )
//        );
        actionGroup.addAction(
                Actions.sequence(
                        Actions.delay(GameConfig.aniTime + 0.1f),
                        Actions.run(new Runnable() {
                            @Override
                            public void run() {
                                AudioManager.PlaySound(AudioManager.AudioType.deck_appear);
                            }
                        }),
                        Actions.moveTo(-Configuration.widthOffset / 2f, 0, 0.2f)
                )
        );
    }

    public void TutorshowAction() {
        removeActor(block);
//        shadow.clearActions();
//        shadow.addAction(
//                Actions.sequence(
//                        Actions.fadeOut(0.01f)
//
//                )
//        );
    }

    public void exitAction() {
        addActor(block);
        actionGroup.addAction(
                Actions.sequence(
                        Actions.moveBy(-Configuration.width, 0, 0.3f),
                        Actions.run(new Runnable() {
                            @Override
                            public void run() {
                                remove();
                            }
                        })
                )
        );
    }

    public interface BoneYardListener {
        void selected(int id, Image image);
    }

    public void showBg(int m) {

        for (int i = 0; i < board_id.length; i++) {
            if (m == board_id[i] && bgImage[i] != null) {
                bgImage[i].setVisible(true);

            } else {
                bgImage[i].setVisible(false);
            }
        }
    }

    public Image getBoneImage(int index) {
//        return dominoes[i].get(index);
        return null;
    }
}
