package com.mobilegame.dominoes.dialogs;

import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.Group;
import com.badlogic.gdx.scenes.scene2d.InputEvent;
import com.badlogic.gdx.scenes.scene2d.Touchable;
import com.badlogic.gdx.scenes.scene2d.actions.Actions;
import com.badlogic.gdx.scenes.scene2d.ui.Label;
import com.badlogic.gdx.scenes.scene2d.ui.ScrollPane;
import com.badlogic.gdx.scenes.scene2d.ui.Table;
import com.badlogic.gdx.utils.Align;
import com.mobilegame.dominoes.assets.Configuration;
import com.mobilegame.dominoes.data.Classes.Design;
import com.mobilegame.dominoes.data.DiamondDataUtil;
import com.mobilegame.dominoes.data.GameDataCsv;
import com.mobilegame.dominoes.objects.DesignDominoesGroup;
import com.mobilegame.dominoes.utils.listener.ButtonListener2;

/**
 * Created by Doodle on 2021/1/8.
 */

public class DesignDialogNew extends BaseDialog {
    private DesignListener listener;

    private Actor dominoeButton;
    private Actor backGroundButton;
    private Actor dominoClickSign;
    private Actor backgroundClickSign;
    private Label dominoLabel;
    private Label backgroundLabel;
    private Actor back;

    private Table backTable;
    private Table dominoTable;
    private ScrollPane backscrollPane;
    private ScrollPane dominoscrollPane;
    private Group bottom;

    private Actor[] dominoes;
    private Actor[] background;

    private Actor fenshuActor;
    private float fenshuX, fenshuY;

    public static int isUnlock = -1;

    public DesignDialogNew(String name) {
        super(name);

        group.setOrigin(Align.center);
        bottom = new Group();
        group.addActor(bottom);

        back = group.findActor("btn_close");

        fenshuActor = group.findActor("fenshu");
        fenshuX = fenshuActor.getX();
        fenshuY = fenshuActor.getY();

        dominoeButton = group.findActor("dominoe_nofocus", Touchable.enabled);
        dominoeButton.setColor(1, 1, 1, 0);
        if (dominoeButton != null) {
            dominoeButton.addListener(new ButtonListener2(false) {
                @Override
                public void clicked(InputEvent event, float x, float y) {
                    super.clicked(event, x, y);
                    if (listener != null) {
                        listener.dominose();
                    }
                }
            });
        }

        backGroundButton = group.findActor("backgroud_focus", Touchable.enabled);
        backGroundButton.setColor(1, 1, 1, 0);
        if (backGroundButton != null) {
            backGroundButton.addListener(new ButtonListener2(false) {
                @Override
                public void clicked(InputEvent event, float x, float y) {
                    super.clicked(event, x, y);
                    if (listener != null) {
                        listener.background();
                    }
                }
            });
        }

        dominoLabel = group.findActor("BitmapFontLabel_3");
        backgroundLabel = group.findActor("BitmapFontLabel_3_0");

        dominoClickSign = group.findActor("Image_16");
        backgroundClickSign = group.findActor("Image_16_0");

        backTable = new Table();
        backTable = backTable.top();
        dominoTable = new Table();
        dominoTable = dominoTable.top();

        backscrollPane = new ScrollPane(backTable, new ScrollPane.ScrollPaneStyle());
        backscrollPane.setSize(615, 600);
        backscrollPane.setPosition(360, 417, Align.bottom);

        backscrollPane.setScrollingDisabled(true, false);
        backscrollPane.setSmoothScrolling(true);
        backscrollPane.toFront();
        bottom.addActor(backscrollPane);

        dominoscrollPane = new ScrollPane(dominoTable, new ScrollPane.ScrollPaneStyle());
        dominoscrollPane.setSize(615, 652);
        dominoscrollPane.setPosition(360, 357, Align.bottom);

        dominoscrollPane.setScrollingDisabled(true, false);
        dominoscrollPane.setSmoothScrolling(true);
        dominoscrollPane.layout();
        dominoscrollPane.toFront();
        bottom.addActor(dominoscrollPane);

        showItems();
        changeDesignTyep(DesignType.dominoes);
        refreshNum();

        resize((int) Configuration.width, (int) Configuration.height);
    }

    private void setClickColor(Label clickLabel, Label noClickLabel) {
        if (clickLabel != null) {
            clickLabel.setColor(1, 242 / 255f, 127 / 255f, 1);
        }
        if (noClickLabel != null) {
            noClickLabel.setColor(26 / 255f, 73 / 255f, 128 / 255f, 1);
        }
    }

    private void showItems() {
        backTable.clear();
        dominoTable.clear();

        int rowCnt = 3;
        int backNum = 0;
        int dominoesNum = 0;

        dominoes = new Actor[9];
        background = new Actor[6];

        for (int i = 1; i <= GameDataCsv.getDesignSize(); i++) {
            Design design = GameDataCsv.getDesign(i);
            if (design.type.contains("dominoes")) {
                if (dominoesNum % rowCnt == 0)
                    dominoTable.row();

                DesignDominoesGroup actor = new DesignDominoesGroup(design);
                dominoes[dominoesNum] = actor;
                dominoesNum++;
                dominoTable.add(actor).width(actor.getWidth()).height(actor.getHeight()).pad(49, 53, 5, 53);
            } else {
                if (backNum % rowCnt == 0)
                    backTable.row();
                DesignDominoesGroup actor = new DesignDominoesGroup(design);
                background[backNum] = actor;
                backNum++;
                backTable.add(actor).width(actor.getWidth()).height(actor.getHeight()).pad(49, 53, 5, 53);
            }
        }
//        Design design = GameDataCsv.getDesign(1);
//        DesignDominoesGroup actor = new DesignDominoesGroup(design);
//        actor.setVisible(false);
//        dominoTable.row();
//        dominoTable.add(actor).width(actor.getWidth()).height(actor.getHeight()).pad(49, 53, 5, 53);

        backscrollPane.layout();
        dominoscrollPane.layout();
    }

    public void setListener(DesignListener listener) {
        this.listener = listener;
    }

    public void addListenr() {
        listener = new DesignListener() {
            @Override
            public void dominose() {
                changeDesignTyep(DesignType.dominoes);
            }

            @Override
            public void background() {
                changeDesignTyep(DesignType.background);
            }

            @Override
            public void back() {
                close();
            }
        };
    }

    public interface DesignListener {
        void dominose();

        void background();

        void back();
    }

    private void changeDesignTyep(DesignType designType) {

        if (designType == DesignType.dominoes) {
            dominoClickSign.setVisible(true);
            backgroundClickSign.setVisible(false);
            setClickColor(dominoLabel, backgroundLabel);

            backscrollPane.setVisible(false);
            dominoscrollPane.setVisible(true);

            for (int i = 0; i < dominoes.length; i++) {
                float x = dominoes[i].getX();
                float y = dominoes[i].getY();
                int row = i / 3;
                dominoes[i].setY(y - 200 - 50 * row);
                dominoes[i].addAction(
                        Actions.moveTo(x, y, 0.2f + 0.1f * row)
                );
            }
        } else {
            backgroundClickSign.setVisible(true);
            dominoClickSign.setVisible(false);
            setClickColor(backgroundLabel, dominoLabel);

            backscrollPane.setVisible(true);
            dominoscrollPane.setVisible(false);

            for (int i = 0; i < background.length; i++) {
                float x = background[i].getX();
                float y = background[i].getY();
                int row = i / 3;
                background[i].setY(y - 200 - 50 * row);
                background[i].addAction(
                        Actions.moveTo(x, y, 0.2f + 0.1f * row)
                );
            }
        }
    }

    public void refreshNum() {
        Label m = group.findActor("playerDiamondNum");
        m.setText("" + DiamondDataUtil.getDiamondNum());
    }

    public void refresh() {
        showItems();
        refreshNum();
    }

    @Override
    public void close() {
        super.close();
//        bottom.addAction(
//                Actions.run(new Runnable() {
//                    @Override
//                    public void run() {
//                        trueClose();
//                    }
//                })
//        );
        layer.addAction(
                Actions.sequence(
                        Actions.fadeOut(0.3f),
                        Actions.run(new Runnable() {
                            @Override
                            public void run() {
                                trueClose();
                            }
                        })
                )
        );
        back.setOrigin(Align.center);
        back.addAction(
                Actions.sequence(
                        Actions.scaleTo(0.01f, 0.01f, 0.25f)
                )
        );
    }

    @Override
    public void resize(int width, int height) {
        fenshuActor.setPosition(fenshuX + Configuration.widthOffset / 2f, fenshuY + Configuration.heightOffset / 2f);
    }

    @Override
    protected void showBottom() {
        super.showBottom();
        bottom.addAction(
                Actions.run(new Runnable() {
                    @Override
                    public void run() {
                        start();
                    }
                })
        );
    }

    public void trueClose() {
        super.bottomClose();
    }

    public enum DesignType {
        dominoes,
        background
    }
}
