package com.mobilegame.dominoes.dialogs;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.Pixmap;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.graphics.g2d.TextureAtlas;
import com.badlogic.gdx.math.Interpolation;
import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.InputEvent;
import com.badlogic.gdx.scenes.scene2d.Touchable;
import com.badlogic.gdx.scenes.scene2d.actions.Actions;
import com.badlogic.gdx.scenes.scene2d.ui.Image;
import com.badlogic.gdx.scenes.scene2d.ui.Label;
import com.badlogic.gdx.utils.Align;
import com.esotericsoftware.spine.AnimationState;
import com.esotericsoftware.spine.SkeletonData;
import com.esotericsoftware.spine.SkeletonJson;
import com.esotericsoftware.spine.SkeletonRenderer;
import com.esotericsoftware.spine.attachments.AtlasAttachmentLoader;
import com.mobilegame.dominoes.assets.Assets;
import com.mobilegame.dominoes.assets.Configuration;
import com.mobilegame.dominoes.data.FileUtil;
import com.mobilegame.dominoes.flurry.DeltaDNAManager;
import com.mobilegame.dominoes.handles.GameConfig;
import com.mobilegame.dominoes.utils.listener.ButtonListener2;
import com.spine.MySpineActor;
import com.spine.MySpineGroup;


/**
 * Created by Doodle on 2021/1/11.
 */

public class GameEndDialogNew extends BaseDialog {
    private GameEndListener listener;
    private Image bgImage;

    private Actor startNewActor;
    private Actor homeActor;
    private Actor scoreActor;
    private Actor titleLabel;
    private MySpineGroup jewelActor;
    private MySpineActor guangActor;
    private MySpineActor titleActor;

    public GameEndDialogNew(String name) {
        super(name);
        Texture bg = new Texture(Gdx.files.internal("image/endBG.png"), Pixmap.Format.RGBA8888, false);
        bg.setFilter(Texture.TextureFilter.Linear, Texture.TextureFilter.Linear);

        SkeletonJson sJsonAct1 = new SkeletonJson(new AtlasAttachmentLoader(Assets.newUI));
        SkeletonData skeletonDataLoading1 = sJsonAct1.readSkeletonData(Gdx.files.internal("animation/jewel.json"));
        jewelActor = new MySpineGroup(new SkeletonRenderer(), skeletonDataLoading1);
        jewelActor.setPosition(360, 770, Align.center);
        jewelActor.setAnimation("animation", true);

        Label label = group.findActor("label");
        if (GameConfig.points[0] > GameConfig.points[1]) {
            Assets.instance.assetManager.load("image/win.atlas", TextureAtlas.class);
            Assets.instance.assetManager.finishLoading();
            TextureAtlas winAtlas = Assets.instance.assetManager.get("image/win.atlas", TextureAtlas.class);
            SkeletonJson sJsonAct = new SkeletonJson(new AtlasAttachmentLoader(winAtlas));

            SkeletonData skeletonDataLoading2 = sJsonAct.readSkeletonData(Gdx.files.internal("animation/win1.json"));
            titleActor = new MySpineActor(new SkeletonRenderer(), skeletonDataLoading2);
            titleActor.setAnimation("animation", false);
            group.addActorAt(0, titleActor);

            SkeletonData skeletonDataLoading = sJsonAct.readSkeletonData(Gdx.files.internal("animation/win2.json"));
            guangActor = new MySpineActor(new SkeletonRenderer(), skeletonDataLoading);
            guangActor.setAnimation("animation", false);
            group.addActorAt(0, guangActor);

            jewelActor.setAnimation("animation", true);
            String text = "1 Gem Is Rewarded To Winner.\nWin Gems To Unlock New Designs!";
            label.setText(text);

        } else {
            Assets.instance.assetManager.load("image/faild.atlas", TextureAtlas.class);
            Assets.instance.assetManager.finishLoading();
            TextureAtlas faildAtlas = Assets.instance.assetManager.get("image/faild.atlas", TextureAtlas.class);
            SkeletonJson sJsonAct = new SkeletonJson(new AtlasAttachmentLoader(faildAtlas));

            SkeletonData skeletonDataLoading2 = sJsonAct.readSkeletonData(Gdx.files.internal("animation/faild1.json"));
            titleActor = new MySpineActor(new SkeletonRenderer(), skeletonDataLoading2);
            titleActor.setAnimation("animation", false);
            group.addActorAt(0, titleActor);

            SkeletonData skeletonDataLoading = sJsonAct.readSkeletonData(Gdx.files.internal("animation/faild2.json"));
            guangActor = new MySpineActor(new SkeletonRenderer(), skeletonDataLoading);
            guangActor.setAnimation("animation", false);
            group.addActor(guangActor);

            jewelActor.setAnimation("animation5", true);
            String text = "Win Gems To Unlock New Designs!";
            label.setText(text);
        }

        guangActor.getAnimationState().addListener(new AnimationState.AnimationStateAdapter() {
            @Override
            public void complete(AnimationState.TrackEntry entry) {
                if (entry.getAnimation().getName().equals("animation")) {
                    guangActor.setAnimation("animation2", true);
                }
            }
        });

        jewelActor.clearActions();
        jewelActor.setScale(0.295f);
        jewelActor.addAction(
                Actions.sequence(
                        Actions.scaleTo(1.1f, 1.1f, 0.3f, Interpolation.sineOut),
                        Actions.scaleTo(1, 1, 0.3333f, Interpolation.sineIn)
                )
        );
        group.addActor(jewelActor);

        bgImage = new Image(bg);

        group.addActorAt(0, bgImage);

        titleLabel = group.findActor("label");
        Color c = titleLabel.getColor();
        titleLabel.setColor(c.r, c.g, c.b, 0);
        titleLabel.clearActions();
        titleLabel.addAction(
                Actions.alpha(1, 0.3333f)
        );

        startNewActor = group.findActor("bt_play", Touchable.enabled);
        startNewActor.clearListeners();
        startNewActor.addListener(new ButtonListener2() {
            @Override
            public void clicked(InputEvent event, float x, float y) {
                super.clicked(event, x, y);
                if (listener != null) {
                    GameConfig.refreshDataForNewGame();
                    listener.startNew();

                    //////////////////////////// DNA ////////////////////////////////
                    DeltaDNAManager.addUiInteraction("playEnd", "button", "startNew", FileUtil.playEndStartNew);
                    /////////////////////////////////////////////////////////////////
                }
            }
        });
        startNewActor.setOrigin(Align.center);
        startNewActor.setScale(0.299f);
        startNewActor.setVisible(false);
        startNewActor.clearActions();
        startNewActor.addAction(
                Actions.sequence(
                        Actions.delay(0.1333f, Actions.show()),
                        Actions.scaleTo(1.1f, 1.1f, 0.3f, Interpolation.sineOut),
                        Actions.scaleTo(1.0f, 1.0f, 0.3333f, Interpolation.sineIn)
                )
        );

        homeActor = group.findActor("bt_home", Touchable.enabled);
        homeActor.clearListeners();
        homeActor.addListener(new ButtonListener2() {
            @Override
            public void clicked(InputEvent event, float x, float y) {
                super.clicked(event, x, y);
                if (listener != null) {
                    GameConfig.refreshDataForNewGame();
                    listener.home();

                    //////////////////////////// DNA ////////////////////////////////
                    DeltaDNAManager.addUiInteraction("playEnd", "button", "home", FileUtil.playEndHome);
                    /////////////////////////////////////////////////////////////////
                }
            }
        });
        homeActor.setOrigin(Align.center);
        homeActor.setScale(0.299f);
        homeActor.clearActions();
        homeActor.setVisible(false);
        homeActor.addAction(
                Actions.sequence(
                        Actions.delay(0.7f, Actions.show()),
                        Actions.scaleTo(1.1f, 1.1f, 0.3f, Interpolation.sineOut),
                        Actions.scaleTo(1.0f, 1.0f, 0.3333f, Interpolation.sineIn)
                )
        );

        scoreActor = group.findActor("bt_score", Touchable.enabled);
        scoreActor.clearListeners();
        scoreActor.addListener(new ButtonListener2() {
            @Override
            public void clicked(InputEvent event, float x, float y) {
                super.clicked(event, x, y);
                if (listener != null) {
                    listener.scoreView();

                    //////////////////////////// DNA ////////////////////////////////
                    DeltaDNAManager.addUiInteraction("playEnd", "button", "viewScores", FileUtil.playEndViewScore);
                    /////////////////////////////////////////////////////////////////
                }
            }
        });
        scoreActor.setOrigin(Align.center);
        scoreActor.setScale(0.299f);
        scoreActor.setVisible(false);
        scoreActor.clearActions();
        scoreActor.addAction(
                Actions.sequence(
                        Actions.delay(0.7f, Actions.show()),
                        Actions.scaleTo(1.1f, 1.1f, 0.3f, Interpolation.sineOut),
                        Actions.scaleTo(1.0f, 1.0f, 0.3333f, Interpolation.sineIn)
                )
        );

        resize((int) Configuration.width, (int) Configuration.height);
    }

    public void setListener(GameEndListener listener) {
        this.listener = listener;
    }

    @Override
    protected void show() {

    }

    @Override
    public void close() {
        if (listener != null) {
            listener.home();
        }
    }

    public interface GameEndListener {
        void startNew();

        void home();

        void scoreView();
    }

    @Override
    public void resize(int width, int height) {
        float offsetY = Configuration.heightOffset / 2f;
        float offsetX = Configuration.widthOffset / 2f;

        float scale = Configuration.height / 1280f;
        //骨牌动画适配
        homeActor.setY(scale * 176 - offsetY, Align.bottom);
        scoreActor.setY(homeActor.getY());
        startNewActor.setPosition(360, homeActor.getY(Align.top) + 20, Align.bottom);
        titleLabel.setPosition(360, scale * 527 - offsetY, Align.bottom);
        jewelActor.setPosition(360, scale * 770 - offsetY, Align.bottom);
        titleActor.setPosition(360, scale * 978 - offsetY, Align.bottom);
        guangActor.setPosition(360, scale * 70 - offsetY, Align.bottom);

        float bgWidth = Configuration.width;
        float bgHeight = Configuration.height;
        if (bgWidth / bgHeight > 720 / 1280f) {
            bgHeight = 1280 / 720f * Configuration.width;
        } else {
            bgWidth = 720 / 1280f * Configuration.height;
        }
        bgImage.setSize(bgWidth, bgHeight);
        bgImage.setPosition(-offsetX, -offsetY);
    }
}
