package com.mobilegame.dominoes.dialogs;

import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.math.Interpolation;
import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.Group;
import com.badlogic.gdx.scenes.scene2d.InputEvent;
import com.badlogic.gdx.scenes.scene2d.Touchable;
import com.badlogic.gdx.scenes.scene2d.actions.Actions;
import com.badlogic.gdx.scenes.scene2d.ui.Image;
import com.badlogic.gdx.scenes.scene2d.ui.Label;
import com.badlogic.gdx.scenes.scene2d.ui.ScrollPane;
import com.badlogic.gdx.utils.Align;
import com.badlogic.gdx.utils.Timer;
import com.mobilegame.dominoes.DominoGame;
import com.mobilegame.dominoes.assets.Assets;
import com.mobilegame.dominoes.assets.Configuration;
import com.mobilegame.dominoes.data.FileUtil;
import com.mobilegame.dominoes.flurry.DeltaDNAManager;
import com.mobilegame.dominoes.handles.DominoManager;
import com.mobilegame.dominoes.handles.GameConfig;
import com.mobilegame.dominoes.handles.PlayerManager;
import com.mobilegame.dominoes.objects.DominoCardPlayer2;
import com.mobilegame.dominoes.utils.listener.ButtonListener2;

/**
 * Created by Doodle on 2021/1/11.
 */

public class HandEndDialogA extends BaseDialog {
    private ScrollPane[] scrollPanes;
    private Group[] tables;
    private HandEndListener listener;
    private boolean gameEnd;
    private Group topG;
    private Actor nextButton;
    private Image bgImage;
    private Actor middleBg;

    private boolean isButtonClose = false;
    private Label title;

    public HandEndDialogA(String name) {
        super(name);
        Assets.instance.assetManager.load("image/endBG.png", Texture.class);
        Assets.instance.assetManager.finishLoading();
        Texture endBgTexture = Assets.instance.assetManager.get("image/endBG.png", Texture.class);
        bgImage = new Image(endBgTexture);
        bgImage.setPosition(0, 0);
        group.addActorAt(0, bgImage);

        middleBg = group.findActor("tuceng_1");

        Actor pL1 = group.findActor("pL1");
        pL1.setPosition(77, 170);
        Actor pL2 = group.findActor("pL2");
        pL2.setPosition(60, 58);
        Actor pL3 = group.findActor("null0");
        pL3.setPosition(200, 170);
        Actor pL4 = group.findActor("null1");
        pL4.setPosition(200, 58);

        topG = new Group();
        topG.setSize(720, 294);
        topG.setPosition(360, 1133, Align.center);
        topG.addActor(pL1);
        topG.addActor(pL2);
        topG.addActor(pL3);
        topG.addActor(pL4);
        group.addActor(topG);
        topG.clearActions();
        Color topGColor = topG.getColor();
        topG.setColor(topGColor.r, topGColor.g, topGColor.b, 0);
        topG.addAction(
                Actions.sequence(
                        Actions.delay(0.0333f),
                        Actions.alpha(1, 0.3333f)
                )
        );

        nextButton = group.findActor("btn_next", Touchable.enabled);
        if (nextButton != null) {
            nextButton.addListener(new ButtonListener2() {
                @Override
                public void clicked(InputEvent event, float x, float y) {
                    super.clicked(event, x, y);
                    if (listener != null) {
                        isButtonClose = true;
                        close();
                    }
                }
            });
        }
        nextButton.clearActions();
        nextButton.setOriginY(Align.center);
        nextButton.setScale(0.137f);
        nextButton.setVisible(false);
        nextButton.addAction(
                Actions.sequence(
                        Actions.delay(0.9333f, Actions.show()),
                        Actions.scaleTo(1.2f, 1.2f, 0.28f, Interpolation.sineOut),
                        Actions.scaleTo(1, 1, 0.3333f, Interpolation.sineIn)
                )
        );

        title = group.findActor("title");
        Color titleColor = title.getColor();
        title.setColor(titleColor.r, titleColor.g, titleColor.b, 0);
        title.clearActions();
        title.addAction(
                Actions.sequence(
                        Actions.delay(0.667f),
                        Actions.alpha(1, 0.3333f)
                )
        );

        Group scoreGroup = new Group();
        Color scoreGroupColor = title.getColor();
        scoreGroup.setColor(scoreGroupColor.r, scoreGroupColor.g, scoreGroupColor.b, 0);
        scoreGroup.addActor(group.findActor("points_0"));
        scoreGroup.addActor(group.findActor("points"));
        group.addActor(scoreGroup);
        scoreGroup.clearActions();
        scoreGroup.addAction(
                Actions.sequence(
                        Actions.delay(0.2333f),
                        Actions.alpha(1, 0.3333f)
                )
        );

        Label youLable = group.findActor("BitmapFontLabel_1_1");
        youLable.moveBy(-82.36f, 0);
        youLable.setVisible(false);
        youLable.clearActions();
        youLable.addAction(
                Actions.sequence(
                        Actions.delay(0.1333f, Actions.show()),
                        Actions.moveBy(82.36f, 0, 0.3333f)
                )
        );

        Label playersLabel = group.findActor("BitmapFontLabel_1_0_0");
        playersLabel.moveBy(82.36f, 0);
        playersLabel.setVisible(false);
        playersLabel.clearActions();
        playersLabel.addAction(
                Actions.sequence(
                        Actions.delay(0.1333f, Actions.show()),
                        Actions.moveBy(-82.36f, 0, 0.3333f)
                )
        );

        Group roundLeftGroup = new Group();
        roundLeftGroup.addActor(group.findActor("BitmapFontLabel_1_2"));
        roundLeftGroup.addActor(group.findActor("round0"));
        roundLeftGroup.moveBy(-82.36f, 0);
        roundLeftGroup.setVisible(false);
        roundLeftGroup.clearActions();
        roundLeftGroup.addAction(
                Actions.sequence(
                        Actions.delay(0.1667f, Actions.show()),
                        Actions.moveBy(82.36f, 0, 0.3333f)
                )
        );
        group.addActor(roundLeftGroup);

        Group roundRightGroup = new Group();
        roundRightGroup.addActor(group.findActor("BitmapFontLabel_1_2_0"));
        roundRightGroup.addActor(group.findActor("round1"));
        roundRightGroup.moveBy(82.36f, 0);
        roundRightGroup.setVisible(false);
        roundRightGroup.clearActions();
        roundRightGroup.addAction(
                Actions.sequence(
                        Actions.delay(0.1667f, Actions.show()),
                        Actions.moveBy(-82.36f, 0, 0.3333f)
                )
        );
        group.addActor(roundRightGroup);

        Group totalLeftGroup = new Group();
        totalLeftGroup.addActor(group.findActor("BitmapFontLabel_1_2_1"));
        totalLeftGroup.addActor(group.findActor("total0"));
        totalLeftGroup.moveBy(-82.36f, 0);
        totalLeftGroup.setVisible(false);
        totalLeftGroup.clearActions();
        totalLeftGroup.addAction(
                Actions.sequence(
                        Actions.delay(0.2f, Actions.show()),
                        Actions.moveBy(82.36f, 0, 0.3333f)
                )
        );
        group.addActor(totalLeftGroup);

        Group totalRightGroup = new Group();
        totalRightGroup.addActor(group.findActor("BitmapFontLabel_1_2_1_0"));
        totalRightGroup.addActor(group.findActor("total1"));
        totalRightGroup.moveBy(82.36f, 0);
        totalRightGroup.setVisible(false);
        totalRightGroup.clearActions();
        totalRightGroup.addAction(
                Actions.sequence(
                        Actions.delay(0.2f, Actions.show()),
                        Actions.moveBy(-82.36f, 0, 0.3333f)
                )
        );
        group.addActor(totalRightGroup);

        scrollPanes = new ScrollPane[2];

        tables = new Group[2];
        tables[0] = new Group();
        tables[1] = new Group();

        resize((int) Configuration.width, (int) Configuration.height);
    }

    @Override
    public void resize(int width, int height) {
        float offsetX = Configuration.widthOffset / 2f;
        float offsetY = Configuration.heightOffset / 2f;


        float middleBgH = height / 1280 * 695;
        if (middleBgH < 695) {
            middleBgH = 695;
        }

        middleBg.setSize(width, middleBgH);
        middleBg.setPosition(360, 638.5f, Align.center);

        title.setY(middleBg.getY(Align.top) - 75);

        float scale = height / 1280f;
        //骨牌动画适配
        nextButton.setPosition(360, scale * 158 - offsetY, Align.bottom);

        topG.setPosition(360, (height - offsetY - middleBg.getY(Align.top)) / 2f + middleBg.getY(Align.top), Align.center);

        bgImage.setSize(width, height);
        bgImage.setPosition(-offsetX, -offsetY);
    }

    public void setData(PlayerManager[] players, DominoManager[] dominoManagers, int winner, boolean gameEnd) {
        this.gameEnd = gameEnd;

        if (gameEnd) {
            group.findActor("next").setVisible(false);
        } else {
            group.findActor("ok").setVisible(false);
        }

//        if (GameConfig.gameType != GameConfig.ALL_FIVES_TYPE) {
//            Label l1 = group.findActor("allfiveWord");
//            l1.setText("The winner is credited with the total");
//
//            Label l2 = group.findActor("allfiveWord_2");
//            l2.setText("number of pips left from the opponent.");
//        }

        Label pointTarget = group.findActor("points");
        pointTarget.setText(GameConfig.targetPoints + "");

        if (winner == 0) {
            Actor bW = group.findActor("botWin");
            bW.setVisible(false);
            Actor yW = group.findActor("youWin");
            yW.clearActions();
            yW.addAction(
                    Actions.sequence(
                            Actions.delay(0.2f, Actions.show()),
                            Actions.scaleTo(1.5f, 1.5f, 0.2f),
                            Actions.scaleTo(1, 1, 0.2f)
                    )
            );
            yW.setVisible(false);
        } else {
            Actor yW = group.findActor("youWin");
            yW.setVisible(false);
            Actor bW = group.findActor("botWin");
            bW.clearActions();
            bW.addAction(
                    Actions.sequence(
                            Actions.delay(0.2f, Actions.show()),
                            Actions.scaleTo(1.5f, 1.5f, 0.2f),
                            Actions.scaleTo(1, 1, 0.2f)
                    )
            );
            bW.setVisible(false);
        }

        initData(players, dominoManagers, winner);
    }

    private void initData(PlayerManager[] players, DominoManager[] dominoManagers, int winner) {
        int[] actionValue = new int[2];

        for (int i = 0; i < 2; i++) {
            Label temp = group.findActor("round" + i);
            if (GameConfig.scores.size() > 0) {
                temp.setText(0 + "");
            }
            temp = group.findActor("total" + i);
            temp.setText(GameConfig.points[i] + "");

            Actor None = group.findActor("null" + i);
            if (players[i].dominoesInHand > 0) {
                None.setVisible(false);
                tables[i].clear();
                scrollPanes[i] = new ScrollPane(tables[i]);
                scrollPanes[i].setSize(250, 100);
                scrollPanes[i].setPosition(321, 160 - i * 112);

                scrollPanes[i].setScrollingDisabled(false, true);
                scrollPanes[i].setCancelTouchFocus(false);
                scrollPanes[i].setSmoothScrolling(true);
                scrollPanes[i].toFront();
                topG.addActor(scrollPanes[i]);

                float cardWidth = 0;
                for (int j = 0; j < players[i].dominoesInHand; j++) {
                    int dominoId = players[i].idsInHand.get(j);
                    DominoCardPlayer2 dominoCard = new DominoCardPlayer2(dominoId, dominoManagers[dominoId].values, null);
                    dominoCard.setScale(0.5f);
                    dominoCard.setSize(dominoCard.getWidth() * 0.5f, dominoCard.getHeight() * 0.5f);

                    int dominoesPoint = dominoManagers[dominoId].values[0] + dominoManagers[dominoId].values[1];
                    actionValue[1 - i] += dominoesPoint;
                    tables[i].addActor(dominoCard);

                    if (players[i].dominoesInHand > 5) {
                        dominoCard.setX(dominoCard.getWidth() * j + 8 * j);
                    } else {
                        dominoCard.setX(scrollPanes[i].getWidth() / 2 - (players[i].dominoesInHand - 1) * (dominoCard.getWidth() / 2 + 4) + dominoCard.getWidth() * j + 8 * j, Align.center);
                    }

                    dominoCard.setY(2);
                    cardWidth = dominoCard.getWidth();
                }
                tables[i].setWidth(players[i].dominoesInHand * (8 + cardWidth));
                float w = tables[i].getWidth();
                if (w < 250) {
                    tables[i].setX(321, Align.left);
                }

            } else {
                None.setVisible(true);
            }
        }

        for (int i = 0; i < 2; i++) {
            if (GameConfig.gameType == GameConfig.ALL_FIVES_TYPE) {
                actionValue[i] = allfiveScore(actionValue[i]);
            }

            Label temp2 = group.findActor("round" + i);
            final Label temp = group.findActor("total" + i);

            boolean isEndOfGame = false;
            final int startValue = GameConfig.points[i] - actionValue[i];
            float delay = 0.1f;
            float delayTotal;

            if (i == winner) {
                temp2.setText(actionValue[i] + "");
                if (actionValue[i] < 20) {
                    temp.setText(startValue + "");
                    for (int m = 1; m <= actionValue[i]; m++) {
                        final Label finalTemp = temp;
                        final int finalM = m;
                        Timer.instance().scheduleTask(new Timer.Task() {
                            @Override
                            public void run() {
                                int x = startValue + finalM;
                                finalTemp.setText(x + "");
                            }
                        }, delay * m);
                    }
                    delayTotal = delay * actionValue[i] + 0.3f;
                } else {
                    float[] delatTime = new float[actionValue[i] + 1];
                    delatTime[0] = 0.1f;
                    temp.setText(startValue + "");
                    for (int m = 1; m <= actionValue[i]; m++) {
                        final Label finalTemp = temp;
                        final int finalM = m;
                        if (m > actionValue[i] - 5) {
                            delatTime[m] = delatTime[m - 1] + 0.15f;
                        } else {
                            delatTime[m] = delatTime[m - 1] + 0.08f;
                        }
                        Timer.instance().scheduleTask(new Timer.Task() {
                            @Override
                            public void run() {
                                int x = startValue + finalM;
                                finalTemp.setText(x + "");
                            }
                        }, delatTime[m]);
                    }
                    delayTotal = delatTime[actionValue[i]] + 0.3f;
                }

                if (isEndOfGame) {
                    Group win = group.findActor("winningscore");
                    win.setOrigin(Align.center);
                    win.addAction(Actions.forever(
                            Actions.sequence(
                                    Actions.scaleTo(1.1f, 1.1f, 0.6f),
                                    Actions.scaleTo(0.95f, 0.95f, 0.6f)

                            )
                    ));
                    final Group winTotla = group.findActor("tPoints" + i);
                    winTotla.setOrigin(Align.center);
                    Timer.instance().scheduleTask(new Timer.Task() {
                        @Override
                        public void run() {
                            winTotla.addAction(Actions.forever(
                                    Actions.sequence(
                                            Actions.scaleTo(1.2f, 1.2f, 0.6f),
                                            Actions.scaleTo(0.90f, 0.90f, 0.6f)

                                    )));
                        }
                    }, delayTotal);
                }

            }
        }
    }

    public void setListener(HandEndListener listener) {
        this.listener = listener;
    }

    @Override
    public void close() {
        if (!isButtonClose) return;
        isButtonClose = false;
//        super.close();
        group.remove();
        layer.remove();
        if (listener != null) {
            if (gameEnd) {
                listener.ok();
            } else {
                listener.next();

                //////////////////////////// DNA ////////////////////////////////
                DeltaDNAManager.addUiInteraction("roundEnd", "button", "next", FileUtil.roundEndNext);
                /////////////////////////////////////////////////////////////////
            }
        }
        DominoGame.isShowEnd = false;
    }


    @Override
    protected void show() {
    }

    private int allfiveScore(int n) {
        int x;

        int a = n % 5;
        int b = n / 5;

        if (a >= 3) {
            b++;
        }
        x = b * 5;

        return x;
    }
}

