package com.mobilegame.dominoes.dialogs;

import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.math.Interpolation;
import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.Group;
import com.badlogic.gdx.scenes.scene2d.InputEvent;
import com.badlogic.gdx.scenes.scene2d.Touchable;
import com.badlogic.gdx.scenes.scene2d.actions.Actions;
import com.badlogic.gdx.scenes.scene2d.ui.Image;
import com.badlogic.gdx.scenes.scene2d.ui.Label;
import com.badlogic.gdx.scenes.scene2d.ui.ScrollPane;
import com.badlogic.gdx.scenes.scene2d.ui.Table;
import com.badlogic.gdx.utils.Align;
import com.badlogic.gdx.utils.Timer;
import com.mobilegame.dominoes.DominoGame;
import com.mobilegame.dominoes.assets.Assets;
import com.mobilegame.dominoes.assets.Configuration;
import com.mobilegame.dominoes.data.FileUtil;
import com.mobilegame.dominoes.flurry.DeltaDNAManager;
import com.mobilegame.dominoes.handles.DominoManager;
import com.mobilegame.dominoes.handles.GameConfig;
import com.mobilegame.dominoes.handles.PlayerManager;
import com.mobilegame.dominoes.objects.ScoreItem;
import com.mobilegame.dominoes.utils.listener.ButtonListener2;

/**
 * Created by Doodle on 2021/1/11.
 */

public class HandEndDialogNew extends BaseDialog {
    private HandEndListener listener;
    private Actor nextButton;
    private Image bgImage;
    private Actor middleBg;

    private boolean isButtonClose = false;
    private Label title;

    private boolean isGameEnd;


    public HandEndDialogNew(String name) {
        super(name);
        Assets.instance.assetManager.load("image/endBG.png", Texture.class);
        Assets.instance.assetManager.finishLoading();
        Texture endBgTexture = Assets.instance.assetManager.get("image/endBG.png", Texture.class);
        bgImage = new Image(endBgTexture);
        bgImage.setPosition(0, 0);
        group.addActorAt(0, bgImage);

        middleBg = group.findActor("tuceng_1");

        title = group.findActor("title");
        Color titleColor = title.getColor();
        title.setColor(titleColor.r, titleColor.g, titleColor.b, 0);
        title.clearActions();
        title.addAction(
                Actions.sequence(
                        Actions.delay(0.667f),
                        Actions.alpha(1, 0.3333f)
                )
        );

        nextButton = group.findActor("btn_next", Touchable.enabled);
        if (nextButton != null) {
            nextButton.addListener(new ButtonListener2() {
                @Override
                public void clicked(InputEvent event, float x, float y) {
                    super.clicked(event, x, y);
                    if (listener != null) {
                        isButtonClose = true;
                        close();
                    }
                }
            });

            nextButton.clearActions();
            nextButton.setOriginY(Align.center);
            nextButton.setScale(0.137f);
            nextButton.setVisible(false);
            nextButton.addAction(
                    Actions.sequence(
                            Actions.delay(0.9333f, Actions.show()),
                            Actions.scaleTo(1.2f, 1.2f, 0.28f, Interpolation.sineOut),
                            Actions.scaleTo(1, 1, 0.3333f, Interpolation.sineIn)
                    )
            );
        }

        resize((int) Configuration.width, (int) Configuration.height);
    }

    private void ordinaryDialog() {
        Group scoreGroup = new Group();
        Color scoreGroupColor = title.getColor();
        scoreGroup.setColor(scoreGroupColor.r, scoreGroupColor.g, scoreGroupColor.b, 0);
        scoreGroup.addActor(group.findActor("points_0"));
        scoreGroup.addActor(group.findActor("points"));
        group.addActor(scoreGroup);
        scoreGroup.clearActions();
        scoreGroup.addAction(
                Actions.sequence(
                        Actions.delay(0.2333f),
                        Actions.alpha(1, 0.3333f)
                )
        );

        Label youLable = group.findActor("BitmapFontLabel_1_1");
        youLable.moveBy(-82.36f, 0);
        youLable.setVisible(false);
        youLable.clearActions();
        youLable.addAction(
                Actions.sequence(
                        Actions.delay(0.1333f, Actions.show()),
                        Actions.moveBy(82.36f, 0, 0.3333f)
                )
        );

        Label playersLabel = group.findActor("BitmapFontLabel_1_0_0");
        playersLabel.moveBy(82.36f, 0);
        playersLabel.setVisible(false);
        playersLabel.clearActions();
        playersLabel.addAction(
                Actions.sequence(
                        Actions.delay(0.1333f, Actions.show()),
                        Actions.moveBy(-82.36f, 0, 0.3333f)
                )
        );

        Group roundLeftGroup = new Group();
        roundLeftGroup.addActor(group.findActor("BitmapFontLabel_1_2"));
        roundLeftGroup.addActor(group.findActor("round0"));
        roundLeftGroup.moveBy(-82.36f, 0);
        roundLeftGroup.setVisible(false);
        roundLeftGroup.clearActions();
        roundLeftGroup.addAction(
                Actions.sequence(
                        Actions.delay(0.1667f, Actions.show()),
                        Actions.moveBy(82.36f, 0, 0.3333f)
                )
        );
        group.addActor(roundLeftGroup);

        Group roundRightGroup = new Group();
        roundRightGroup.addActor(group.findActor("BitmapFontLabel_1_2_0"));
        roundRightGroup.addActor(group.findActor("round1"));
        roundRightGroup.moveBy(82.36f, 0);
        roundRightGroup.setVisible(false);
        roundRightGroup.clearActions();
        roundRightGroup.addAction(
                Actions.sequence(
                        Actions.delay(0.1667f, Actions.show()),
                        Actions.moveBy(-82.36f, 0, 0.3333f)
                )
        );
        group.addActor(roundRightGroup);

        Group totalLeftGroup = new Group();
        totalLeftGroup.addActor(group.findActor("BitmapFontLabel_1_2_1"));
        totalLeftGroup.addActor(group.findActor("total0"));
        totalLeftGroup.moveBy(-82.36f, 0);
        totalLeftGroup.setVisible(false);
        totalLeftGroup.clearActions();
        totalLeftGroup.addAction(
                Actions.sequence(
                        Actions.delay(0.2f, Actions.show()),
                        Actions.moveBy(82.36f, 0, 0.3333f)
                )
        );
        group.addActor(totalLeftGroup);

        Group totalRightGroup = new Group();
        totalRightGroup.addActor(group.findActor("BitmapFontLabel_1_2_1_0"));
        totalRightGroup.addActor(group.findActor("total1"));
        totalRightGroup.moveBy(82.36f, 0);
        totalRightGroup.setVisible(false);
        totalRightGroup.clearActions();
        totalRightGroup.addAction(
                Actions.sequence(
                        Actions.delay(0.2f, Actions.show()),
                        Actions.moveBy(-82.36f, 0, 0.3333f)
                )
        );
        group.addActor(totalRightGroup);
    }

    private final float lineHeight = 80;

    private void dialog() {
        Group table = new Group();
        table.setWidth(639);
        ScrollPane pane = new ScrollPane(table);
        pane.isTopScollPaneY = true;
        group.addActor(pane);
        TextureRegion lineTexture = Assets.newUI.findRegion("scoreLine");
//
        float y = 0;
        Table b = new ScoreItem(GameConfig.scores.size(), "Total score :", "" + GameConfig.points[0], "" + GameConfig.points[1], true);
        b.setPosition(319.5f, y, Align.bottom);
        table.addActor(b);
        y += lineHeight;

        b.clearActions();
        b.moveBy(0, -100);
        b.addAction(
                Actions.moveBy(0, 100, 0.15f)
        );

//        int playPoint = 0;
//        int compPoint = 0;
//        for (int i = 1; i < GameConfig.scores.size(); ++i) {
//            playPoint += GameConfig.scores.get(i - 1)[0];
//            compPoint += GameConfig.scores.get(i - 1)[1];
//        }

        for (int i = GameConfig.scores.size(), j=1; i > 0; i--, j++) {
            Image line = new Image(lineTexture);
            line.setPosition(319.5f, y, Align.center);
            y += line.getHeight();
            table.addActor(line);

            line.clearActions();
            line.moveBy(0, -100);
            line.addAction(
                    Actions.moveBy(0, 100, 0.15f + 0.05f * j)
            );

            Table a;
            a = new ScoreItem(i, "Round " + i + " :", "+" + GameConfig.scores.get(i - 1)[0], "+" + GameConfig.scores.get(i - 1)[1], false);
            a.setPosition(319.5f, y, Align.bottom);
            y += lineHeight;
            table.addActor(a);

            a.clearActions();
            a.moveBy(0, -100);
            a.addAction(
                    Actions.moveBy(0, 100, 0.15f + 0.05f * j)
            );
        }

//        for (int i = 50, j = 1; i > 0; i--, ++j) {
//            Image line = new Image(lineTexture);
//            line.setPosition(319.5f, y, Align.center);
//            y += line.getHeight();
//            table.addActor(line);
//
//            line.clearActions();
//            line.moveBy(0, -100);
//            line.addAction(
//                    Actions.moveBy(0, 100, 0.15f + 0.05f * j)
//            );
//
//            Table a;
//            a = new ScoreItem(i, "Round " + i + " :", "+" + 100, "+" + 100, false);
//            a.setPosition(319.5f, y, Align.bottom);
//            y += lineHeight;
//            table.addActor(a);
//
//            a.clearActions();
//            a.moveBy(0, -100);
//            a.addAction(
//                    Actions.moveBy(0, 100, 0.15f + 0.05f * j)
//            );
//        }

        table.setHeight(y);

        float height = 500;
        if (height > table.getHeight() + 10) {
            height = table.getHeight() + 10;
        }
        pane.setSize(639, height);
        pane.setPosition(360, middleBg.getY(Align.top) - 140, Align.top);
    }

    @Override
    public void resize(int width, int height) {
        float offsetX = Configuration.widthOffset / 2f;
        float offsetY = Configuration.heightOffset / 2f;


        float middleBgH = height / 1280 * 695;
        if (middleBgH < 695) {
            middleBgH = 695;
        }

        middleBg.setSize(width, middleBgH);
        middleBg.setPosition(360, 701, Align.center);

        title.setY(middleBg.getY(Align.top) - 75);

        float scale = height / 1280f;
        //骨牌动画适配
        nextButton.setPosition(360, scale * 158 - offsetY, Align.bottom);

        bgImage.setSize(width, height);
        bgImage.setPosition(-offsetX, -offsetY);
    }

    public void setData(PlayerManager[] players, DominoManager[] dominoManagers, int winner, boolean gameEnd) {
        this.isGameEnd = gameEnd;
        if (gameEnd) {
            group.findActor("next").setVisible(false);
            dialog();
            return;
        } else {
            group.findActor("ok").setVisible(false);
            ordinaryDialog();
        }

//        if (GameConfig.gameType != GameConfig.ALL_FIVES_TYPE) {
//            Label l1 = group.findActor("allfiveWord");
//            l1.setText("The winner is credited with the total");
//
////            Label l2 = group.findActor("allfiveWord_2");
////            l2.setText("number of pips left from the opponent.");
//        }

        Label pointTarget = group.findActor("points");
        pointTarget.setText(GameConfig.targetPoints + "");

        if (winner == 0) {
            Actor bW = group.findActor("botWin");
            bW.setVisible(false);
            Actor yW = group.findActor("youWin");
            yW.clearActions();
            yW.addAction(
                    Actions.sequence(
                            Actions.delay(0.2f, Actions.show()),
                            Actions.scaleTo(1.5f, 1.5f, 0.2f),
                            Actions.scaleTo(1, 1, 0.2f)
                    )
            );
            yW.setVisible(false);
        } else {
            Actor yW = group.findActor("youWin");
            yW.setVisible(false);
            Actor bW = group.findActor("botWin");
            bW.clearActions();
            bW.addAction(
                    Actions.sequence(
                            Actions.delay(0.2f, Actions.show()),
                            Actions.scaleTo(1.5f, 1.5f, 0.2f),
                            Actions.scaleTo(1, 1, 0.2f)
                    )
            );
            bW.setVisible(false);
        }

        initData(players, dominoManagers, winner);
    }

    private void initData(PlayerManager[] players, DominoManager[] dominoManagers, int winner) {
        int[] actionValue = new int[2];

        for (int i = 0; i < 2; i++) {
            Label temp = group.findActor("round" + i);
            if (GameConfig.scores.size() > 0) {
                temp.setText(0 + "");
            }
            temp = group.findActor("total" + i);
            temp.setText(GameConfig.points[i] + "");
        }

        for (int j = 0; j < players[1 - winner].dominoesInHand; j++) {
            int[] values = dominoManagers[players[1 - winner].idsInHand.get(j)].values;
            actionValue[winner] += values[0] + values[1];
        }

        for (int i = 0; i < 2; i++) {
            if (GameConfig.gameType == GameConfig.ALL_FIVES_TYPE) {
                actionValue[i] = allfiveScore(actionValue[i]);
            }

            Label temp2 = group.findActor("round" + i);
            final Label temp = group.findActor("total" + i);
            boolean isEndOfGame = false;
            final int startValue = GameConfig.points[i] - actionValue[i];
            float delay = 0.1f;
            float delayTotal;

            if (i == winner) {
                temp2.setText(actionValue[i] + "");
                if (actionValue[i] < 20) {
                    temp.setText(startValue + "");
                    for (int m = 1; m <= actionValue[i]; m++) {
                        final Label finalTemp = temp;
                        final int finalM = m;
                        Timer.instance().scheduleTask(new Timer.Task() {
                            @Override
                            public void run() {
                                int x = startValue + finalM;
                                finalTemp.setText(x + "");
                            }
                        }, delay * m);
                    }
                    delayTotal = delay * actionValue[i] + 0.3f;
                } else {
                    float[] delatTime = new float[actionValue[i] + 1];
                    delatTime[0] = 0.1f;
                    System.out.println("**********************endof: " + startValue);
                    temp.setText(startValue + "");
                    for (int m = 1; m <= actionValue[i]; m++) {
                        final Label finalTemp = temp;
                        final int finalM = m;
                        if (m > actionValue[i] - 5) {
                            delatTime[m] = delatTime[m - 1] + 0.15f;
                        } else {
                            delatTime[m] = delatTime[m - 1] + 0.08f;
                        }
                        Timer.instance().scheduleTask(new Timer.Task() {
                            @Override
                            public void run() {
                                int x = startValue + finalM;
                                finalTemp.setText(x + "");
                            }
                        }, delatTime[m]);
                    }
                    delayTotal = delatTime[actionValue[i]] + 0.3f;
                }

                if (isEndOfGame) {
                    Group win = group.findActor("winningscore");
                    win.setOrigin(Align.center);
                    win.addAction(Actions.forever(
                            Actions.sequence(
                                    Actions.scaleTo(1.1f, 1.1f, 0.6f),
                                    Actions.scaleTo(0.95f, 0.95f, 0.6f)

                            )
                    ));
                    final Group winTotla = group.findActor("tPoints" + i);
                    winTotla.setOrigin(Align.center);
                    Timer.instance().scheduleTask(new Timer.Task() {
                        @Override
                        public void run() {
                            winTotla.addAction(Actions.forever(
                                    Actions.sequence(
                                            Actions.scaleTo(1.2f, 1.2f, 0.6f),
                                            Actions.scaleTo(0.90f, 0.90f, 0.6f)

                                    )));
                        }
                    }, delayTotal);
                }

            }
        }
    }

    public void setListener(HandEndListener listener) {
        this.listener = listener;
    }

    @Override
    public void close() {
        if (!isButtonClose) return;
        isButtonClose = false;
//        super.close();
        group.remove();
        layer.remove();
        if (listener != null) {
            if (isGameEnd) {
                listener.ok();
            } else {
                listener.next();

                //////////////////////////// DNA ////////////////////////////////
                DeltaDNAManager.addUiInteraction("roundEnd", "button", "next", FileUtil.roundEndNext);
                /////////////////////////////////////////////////////////////////
            }
        }
        DominoGame.isShowEnd = false;
    }


    @Override
    protected void show() {
    }


    private int allfiveScore(int n) {
        int x;

        int a = n % 5;
        int b = n / 5;

        if (a >= 3) {
            b++;
        }
        x = b * 5;

        return x;
    }
}
