package com.mobilegame.dominoes.dialogs;

import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.InputEvent;
import com.badlogic.gdx.scenes.scene2d.Touchable;
import com.badlogic.gdx.scenes.scene2d.actions.Actions;
import com.badlogic.gdx.scenes.scene2d.utils.ClickListener;
import com.badlogic.gdx.utils.Align;
import com.mobilegame.dominoes.assets.Configuration;
import com.mobilegame.dominoes.audio.AudioManager;
import com.mobilegame.dominoes.data.FileUtil;
import com.mobilegame.dominoes.data.SettingDataUtil;
import com.mobilegame.dominoes.flurry.DeltaDNAManager;
import com.mobilegame.dominoes.utils.listener.ButtonListener2;

/**
 * Created by Doodle on 2021/1/8.
 */

public class SettingDialogNew extends BaseDialog {
    private SettingListener listener;
    private Actor musicClickBg;
    private Actor soundClickBg;
    private Actor musicClickDot;
    private Actor soundClickDot;
    private Actor musicNoClickDot;
    private Actor soundNoClickDot;

    private String settingName = "setting1";
    private String settingNameMusic = FileUtil.mainSettingMusic;
    private String settingNameSound = FileUtil.mainSettingSound;

    public SettingDialogNew(String name) {
        super(name);
        group.setTouchable(Touchable.enabled);
        group.setPosition(360, Configuration.height / 2f, Align.center);
        initButtons();
    }

    public void setListener(SettingListener listener) {
        this.listener = listener;
    }

    private void initButtons() {
        Actor btn = group.findActor("btn_quit", Touchable.enabled);
        if (btn != null) {
            settingName = "setting2";
            settingNameMusic = FileUtil.gameSettingMusic;
            settingNameSound = FileUtil.gameSettingSound;

            btn.addListener(new ButtonListener2() {
                @Override
                public void clicked(InputEvent event, float x, float y) {
                    super.clicked(event, x, y);
                    if (listener != null) {
                        listener.home();

                        //////////////////////////// DNA ////////////////////////////////
                        DeltaDNAManager.addUiInteraction("setting2", "button", "home", FileUtil.gameSettingHome);
                        /////////////////////////////////////////////////////////////////
                    }
                }
            });
        }

        btn = group.findActor("btn_restart", Touchable.enabled);
        if (btn != null) {
            btn.addListener(new ButtonListener2() {
                @Override
                public void clicked(InputEvent event, float x, float y) {
                    super.clicked(event, x, y);
                    if (listener != null) {
                        listener.startNew();

                        //////////////////////////// DNA ////////////////////////////////
                        DeltaDNAManager.addUiInteraction("setting2", "button", "restart", FileUtil.gameSettingRestart);
                        /////////////////////////////////////////////////////////////////
                    }
                }
            });
        }
        btn = group.findActor("btn_design", Touchable.enabled);
        if (btn != null) {
            btn.addListener(new ButtonListener2() {
                @Override
                public void clicked(InputEvent event, float x, float y) {
                    super.clicked(event, x, y);
                    if (listener != null) {
                        listener.openDesignDialog();

                        //////////////////////////// DNA ////////////////////////////////
                        DeltaDNAManager.addUiInteraction("setting2", "button", "help", FileUtil.gameSettingHelp);
                        /////////////////////////////////////////////////////////////////
                    }
                }
            });
        }

        musicClickBg = group.findActor("yuanjiaojuxing_2");
        soundClickBg = group.findActor("yuanjiaojuxing_2_0");
        musicClickDot = group.findActor("yuanjiaojuxing_3");
        soundClickDot = group.findActor("yuanjiaojuxing_3_0");
        musicNoClickDot = group.findActor("yuanjiaojuxing_3_kaobei_0");
        soundNoClickDot = group.findActor("yuanjiaojuxing_3_kaobei");

        Actor turnMusic = group.findActor("btn_music", Touchable.disabled);
        if (turnMusic != null) {
            Actor turnMusicActor = new Actor();
            turnMusicActor.setTouchable(Touchable.enabled);
            turnMusicActor.setSize(turnMusic.getWidth() + 50, turnMusic.getHeight() + 26);
            turnMusicActor.setPosition(turnMusic.getX(Align.center), turnMusic.getY(Align.center), Align.center);
            group.addActorBefore(turnMusic, turnMusicActor);

            turnMusicActor.addListener(new ClickListener() {
                @Override
                public void clicked(InputEvent event, float x, float y) {
                    super.clicked(event, x, y);

                    boolean music = !SettingDataUtil.getMusic();
                    SettingDataUtil.setMusic(music);
                    turnMusic(music);

                    //////////////////////////// DNA ////////////////////////////////
                    DeltaDNAManager.addUiInteraction(settingName, "button", "music", settingNameMusic);
                    /////////////////////////////////////////////////////////////////
                }
            });
        }

        Actor turnSonud = group.findActor("btn_sound", Touchable.disabled);
        if (turnSonud != null) {
            Actor turnSoundActor = new Actor();
            turnSoundActor.setTouchable(Touchable.enabled);
            turnSoundActor.setSize(turnSonud.getWidth() + 50, turnSonud.getHeight() + 26);
            turnSoundActor.setPosition(turnSonud.getX(Align.center), turnSonud.getY(Align.center), Align.center);
            group.addActorBefore(turnSonud, turnSoundActor);

            turnSoundActor.addListener(new ClickListener() {
                @Override
                public void clicked(InputEvent event, float x, float y) {
                    super.clicked(event, x, y);
                    boolean sound = !SettingDataUtil.getSound();
                    SettingDataUtil.setSound(sound);
                    turnSonud(sound);

                    //////////////////////////// DNA ////////////////////////////////
                    DeltaDNAManager.addUiInteraction(settingName, "button", "sound", settingNameSound);
                    /////////////////////////////////////////////////////////////////
                }
            });
        }

        if (group.findActor("player_words") != null) {
            group.findActor("player_words").setVisible(false);
        }

        turnMusic(SettingDataUtil.getMusic());
        turnSonud(SettingDataUtil.getSound());
    }

    private void turnMusic(boolean isMusic) {
        if (isMusic) {
            AudioManager.playMusic();
            musicClickDot.setVisible(true);
            musicClickBg.setVisible(true);
            musicNoClickDot.setVisible(false);
        } else {
            AudioManager.stopMusic();
            musicClickDot.setVisible(false);
            musicClickBg.setVisible(false);
            musicNoClickDot.setVisible(true);
        }
    }

    private void turnSonud(boolean isSound) {
        if (isSound) {
            soundClickDot.setVisible(true);
            soundClickBg.setVisible(true);
            soundNoClickDot.setVisible(false);
        } else {
            soundClickDot.setVisible(false);
            soundClickBg.setVisible(false);
            soundNoClickDot.setVisible(true);
        }
    }

    @Override
    protected void show() {
        super.show();
        Actor btn = group.findActor("btn_quit", Touchable.enabled);
        if (btn != null) {
            btn.setOrigin(Align.center);
            btn.setScale(0.6f);
            btn.addAction(
                    Actions.sequence(
                            Actions.scaleTo(1.1f, 1.1f, 0.3f),
                            Actions.scaleTo(1.0f, 1.0f, 0.1f)
                    )
            );
        }

        btn = group.findActor("btn_restart", Touchable.enabled);
        if (btn != null) {
            btn.setOrigin(Align.center);
            btn.setScale(0.6f);
            btn.addAction(
                    Actions.sequence(
                            Actions.scaleTo(1.1f, 1.1f, 0.3f),
                            Actions.scaleTo(1.0f, 1.0f, 0.1f)
                    )
            );
        }
        btn = group.findActor("btn_design", Touchable.enabled);
        if (btn != null) {
            btn.setOrigin(Align.center);
            btn.setScale(0.6f);
            btn.addAction(
                    Actions.sequence(
                            Actions.scaleTo(1.1f, 1.1f, 0.3f),
                            Actions.scaleTo(1.0f, 1.0f, 0.1f)
                    )
            );
        }
    }

    public interface SettingListener {
        void startNew();

        void home();

        void openDesignDialog();

    }
}
