package com.mobilegame.dominoes.dialogs;

import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.Group;
import com.badlogic.gdx.scenes.scene2d.InputEvent;
import com.badlogic.gdx.scenes.scene2d.Touchable;
import com.badlogic.gdx.scenes.scene2d.utils.ActorGestureListener;
import com.badlogic.gdx.scenes.scene2d.utils.ClickListener;

/**
 * Created by Doodle on 2021/3/18.
 */

public abstract class TipBaseDialog extends BaseDialog {
    Actor left, right;
    int page = 1, pageSize = 3;
    Group pagesGroup;
    float groupX, groupY, groupW, groupH;
    Runnable runnable;

    public TipBaseDialog(String name) {
        super(name);

        left = group.findActor("left", Touchable.enabled);
        left.clearListeners();
        left.addListener(new ClickListener() {
            @Override
            public void clicked(InputEvent event, float x, float y) {
                page--;
                setPage();
            }
        });

        right = group.findActor("right", Touchable.enabled);
        right.clearListeners();
        right.addListener(new ClickListener() {
            @Override
            public void clicked(InputEvent event, float x, float y) {
                page++;
                setPage();
            }
        });

        initPagesGroup();

        setPage();
    }

    protected abstract void initPagesGroup();

    protected void setPage() {
        if (page < 1) {
            page = 1;
        } else if (page > pageSize) {
            page = pageSize;
        }
        showPage(page);
        showPageSign(page);
    }

    protected abstract void showPageSign(int page);

    protected abstract void showPage(int page);

    protected ActorGestureListener getMoveClick() {
        return new ActorGestureListener() {
            boolean isFling;
            float startX;

            @Override
            public void touchDown(InputEvent event, float x, float y, int pointer, int button) {
                isFling = false;
                Actor actor = event.getListenerActor();
                startX = actor.getX();
            }

            @Override
            public void pan(InputEvent event, float x, float y, float deltaX, float deltaY) {
                Actor actor = event.getListenerActor();
                actor.setX(actor.getX() + deltaX);
            }

            @Override
            public void fling(InputEvent event, float velocityX, float velocityY, int button) {
                if (velocityX >= 500) {
                    page--;
                    setPage();
                    isFling = true;
                } else if (velocityX <= -500) {
                    page++;
                    setPage();
                    isFling = true;
                }
            }

            @Override
            public void touchUp(InputEvent event, float x, float y, int pointer, int button) {
                if (!isFling) {
                    Actor actor = event.getListenerActor();
                    if (actor.getX() > startX && actor.getX() - startX > groupW / 2f) {
                        page--;
                        setPage();
                    } else if (actor.getX() < startX && startX - actor.getX() > groupW / 2f) {
                        page++;
                        setPage();
                    } else {
                        setPage();
                    }
                }
            }
        };
    }

    public void setCloseFinish(Runnable runnable) {
        this.runnable = runnable;
    }

    @Override
    public void close() {
        super.close();
        if (runnable != null) {
            runnable.run();
        }
    }
}
