package com.mobilegame.dominoes.dialogs;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.InputEvent;
import com.badlogic.gdx.scenes.scene2d.Touchable;
import com.badlogic.gdx.scenes.scene2d.ui.Label;
import com.badlogic.gdx.utils.Align;
import com.esotericsoftware.spine.SkeletonData;
import com.esotericsoftware.spine.SkeletonJson;
import com.esotericsoftware.spine.SkeletonRenderer;
import com.esotericsoftware.spine.attachments.AtlasAttachmentLoader;
import com.mobilegame.dominoes.DominoGame;
import com.mobilegame.dominoes.assets.Assets;
import com.mobilegame.dominoes.audio.AudioManager;
import com.mobilegame.dominoes.data.Classes.Design;
import com.mobilegame.dominoes.data.DiamondDataUtil;
import com.mobilegame.dominoes.data.FileUtil;
import com.mobilegame.dominoes.flurry.DeltaDNAManager;
import com.mobilegame.dominoes.flurry.FlurryManager;
import com.mobilegame.dominoes.utils.listener.ButtonListener2;
import com.spine.MySpineGroup;

/**
 * Created by qwe on 2020/1/10.
 */

public class UnlockDialog extends BaseDialog {
    private Unlocklistenr listener;
    private Design design;
    private DesignDialogNew.DesignType designType;


    public UnlockDialog(String name) {
        super(name);
        initButtons();
        addListenr();
    }

    public void setDesign(Design design) {
        this.design = design;

        SkeletonJson sJsonAct1 = new SkeletonJson(new AtlasAttachmentLoader(Assets.newUI));
        SkeletonData skeletonDataLoading1 = sJsonAct1.readSkeletonData(Gdx.files.internal("animation/jewel.json"));
        MySpineGroup jewelActor = new MySpineGroup(new SkeletonRenderer(), skeletonDataLoading1);
        jewelActor.setScale(0.58f);
        jewelActor.setPosition(360, 772, Align.center);
        jewelActor.setAnimation("animation", true);
        group.addActor(jewelActor);

        if (design.type.contains("dominoes")) {
            designType = DesignDialogNew.DesignType.dominoes;
        } else {
            designType = DesignDialogNew.DesignType.background;
        }

        if (DiamondDataUtil.getDiamondNum() < design.Unlocking_condition) {
            group.findActor("btn_ok").setVisible(false);
            AudioManager.PlaySound(AudioManager.AudioType.locked);
        } else {
            group.findActor("notEnough").setVisible(false);
        }

        ((Label) group.findActor("diamond_num")).setText("" + design.Unlocking_condition);
    }

    private void initButtons() {
        Actor btn = group.findActor("btn_ok", Touchable.enabled);
        if (btn != null) {
            btn.addListener(new ButtonListener2() {
                @Override
                public void clicked(InputEvent event, float x, float y) {
                    super.clicked(event, x, y);
                    AudioManager.PlaySound(AudioManager.AudioType.unlock);
                    FlurryManager.flurryLog_design_switch();
                    if (listener != null) {
                        listener.ok();
                        String type;

                        DesignDialogNew.isUnlock = design.ID;
                        if (designType == DesignDialogNew.DesignType.dominoes) {
                            DiamondDataUtil.useDesign(design.ID, designType);
                            DominoGame.showDominoes(design.Topic_name + "-");
                            type = "dominoes";
                        } else {
                            DiamondDataUtil.useDesign(design.ID, designType);
                            DominoGame.showBg(design.ID);
                            type = "backGround";
                        }

                        /////////////////////////// DNA ////////////////////////////
                        DeltaDNAManager.addSkinUsed(type, design.ID + "", true, DiamondDataUtil.getDiamondNum());
                        DeltaDNAManager.addUiInteraction("unlock", "button", "buySkin", FileUtil.unlockOk);
                        ////////////////////////////////////////////////////////////
                    }
                }
            });
        }
    }

    public void addListenr() {
        listener = new Unlocklistenr() {
            @Override
            public void ok() {
                FlurryManager.flurryLog_design_unlock();
                FlurryManager.flurryLog_design_unlock_u();
                FlurryManager.flurryLog_design_unlock_f(design.ID);
                DiamondDataUtil.usdDiamond(design.Unlocking_condition);
                DiamondDataUtil.unlockDesign(design.ID);
                DominoGame.refreshDesignDialog();
                close();
            }
        };
    }

    public interface Unlocklistenr {
        void ok();
    }

}
