package com.mobilegame.dominoes.objects;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.g2d.Batch;
import com.badlogic.gdx.graphics.g2d.PolygonSpriteBatch;
import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.Touchable;
import com.badlogic.gdx.utils.Align;
import com.esotericsoftware.spine.*;
import com.mobilegame.dominoes.DominoGame;
import com.mobilegame.dominoes.spine.MAtlasAttachmentLoader;

public class AniActor extends Actor {
    private Skeleton skeleton;
    private AnimationState state;
    SkeletonRenderer renderer;
    private float scale = 1;
    public AniActor(Actor old,String name){
        if(old!=null) {
            setPosition(old.getX(Align.center), old.getY(Align.center));
            setSize(old.getWidth(), old.getHeight());
            setOrigin(old.getOriginX(),old.getOriginY());
        }
        this.name = name;
        init();
        setTouchable(Touchable.disabled);
    }

    public AniActor(String name){
        this.name = name;
        init();
    }


    private void init() {
        renderer = DominoGame.getSkeletonRenderer();
        SkeletonJson skeletonJson = DominoGame.getSkeletonBinary();
        skeletonJson.setScale(scale);
        SkeletonData data = skeletonJson.readSkeletonData(Gdx.files.internal(name));
        AnimationStateData animationStateData = new AnimationStateData(data);
        state = new AnimationState(animationStateData);
        skeleton = new Skeleton(data);
        skeleton.setPosition(x, y);
        MAtlasAttachmentLoader.prefix = "";
    }


    public void setAnimation(String name,boolean loop){
        skeleton.setToSetupPose();
        state.setAnimation(0,name,loop);
    }

    public AnimationState getState() {
        return state;
    }

    @Override
    public void act(float delta) {
        super.act(delta);
        if(isVisible()) {
            state.update(delta);
        }
    }

    @Override
    public void draw(Batch batch, float parentAlpha) {
        if(!isVisible()) return;
        super.draw(batch, parentAlpha);
        int c1 = batch.getBlendSrcFunc();
        int c2 = batch.getBlendDstFunc();
        int c3=batch.getBlendSrcFuncAlpha();
        int c4 =batch.getBlendDstFuncAlpha();
        state.apply(skeleton);
        skeleton.setPosition(x,y);
        skeleton.updateWorldTransform();
        skeleton.setColor(getColor());
        renderer.draw((PolygonSpriteBatch)batch, skeleton);
        batch.setBlendFunctionSeparate(c1,c2,c3,c4);
    }
}
