package com.mobilegame.dominoes.objects;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.g2d.NinePatch;
import com.badlogic.gdx.math.Interpolation;
import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.Group;
import com.badlogic.gdx.scenes.scene2d.InputEvent;
import com.badlogic.gdx.scenes.scene2d.Touchable;
import com.badlogic.gdx.scenes.scene2d.actions.Actions;
import com.badlogic.gdx.scenes.scene2d.ui.Image;
import com.badlogic.gdx.scenes.scene2d.ui.Label;
import com.badlogic.gdx.utils.Align;
import com.esotericsoftware.spine.SkeletonData;
import com.esotericsoftware.spine.SkeletonJson;
import com.esotericsoftware.spine.SkeletonRenderer;
import com.esotericsoftware.spine.attachments.AtlasAttachmentLoader;
import com.mobilegame.dominoes.DominoGame;
import com.mobilegame.dominoes.assets.Assets;
import com.mobilegame.dominoes.data.Classes.Design;
import com.mobilegame.dominoes.data.DiamondDataUtil;
import com.mobilegame.dominoes.data.FileUtil;
import com.mobilegame.dominoes.dialogs.DesignDialogNew;
import com.mobilegame.dominoes.flurry.DeltaDNAManager;
import com.mobilegame.dominoes.flurry.FlurryManager;
import com.mobilegame.dominoes.utils.listener.ButtonListenrNoSound;
import com.spine.MySpineActor;


/**
 * Created by qwe on 2020/1/7.
 */

public class DesignDominoesGroup extends Group {
    private Image own;
    private Image inUse;
    private Image show;
    private Image inChoose;
    private Image designBack;
    private Group diamond;
    private DesignDialogNew.DesignType designType;
    private Design design;

    private String skinName;
    private String skinNameF;

    private MySpineActor inuseSpineActor;
    private MySpineActor demoSpineActor;

    public DesignDominoesGroup(Design desig) {

        setSize(100, 213);
        this.design = desig;
        float offsetH;

        boolean isUnlock = DiamondDataUtil.isUnlockDesign(design.ID);
        if (isUnlock) {
            SkeletonJson sJsonAct = new SkeletonJson(new AtlasAttachmentLoader(Assets.instance.designAnimation));
            SkeletonData skeletonDataLoading = sJsonAct.readSkeletonData(Gdx.files.internal("animation/inuse.json"));
            inuseSpineActor = new MySpineActor(new SkeletonRenderer(), skeletonDataLoading);
        }
        SkeletonJson sJsonAct2 = new SkeletonJson(new AtlasAttachmentLoader(Assets.instance.designAnimation));
        if (design.type.contains("dominoes")) {
            designType = DesignDialogNew.DesignType.dominoes;
            if (isUnlock) {
                SkeletonData skeletonDataLoading2 = sJsonAct2.readSkeletonData(Gdx.files.internal("animation/skin_dom.json"));
                demoSpineActor = new MySpineActor(new SkeletonRenderer(), skeletonDataLoading2);
            }
            offsetH = 7;
            designBack = new Image(Assets.instance.design.findRegion("blacks1"));
            show = new Image(Assets.instance.design.findRegion(design.Topic_name + "_board")) {
                @Override
                public void setScale(float scaleX, float scaleY) {
                    super.setScale(scaleX, scaleY);
                    designBack.setScale(scaleX, scaleY);
                }
            };

            skinName = "skinTile";
            skinNameF = FileUtil.skinTileDomino;
        } else {
            designType = DesignDialogNew.DesignType.background;
            if (isUnlock) {
                SkeletonData skeletonDataLoading2 = sJsonAct2.readSkeletonData(Gdx.files.internal("animation/skin_back.json"));
                demoSpineActor = new MySpineActor(new SkeletonRenderer(), skeletonDataLoading2);
            }
            offsetH = 8;
            designBack = new Image(Assets.instance.design.findRegion("blacks2"));
            show = new Image(Assets.instance.design.findRegion(design.Topic_name)) {
                @Override
                public void setScale(float scaleX, float scaleY) {
                    super.setScale(scaleX, scaleY);
                    designBack.setScale(scaleX, scaleY);
                }
            };

            skinName = "skin2";
            skinNameF = FileUtil.skinTileBackGround;
        }
//        show = new Image(Assets.instance.design.findRegion(design.Topic_name+"-board"));
        inUse = new Image(Assets.instance.design.findRegion("inuse"));
        inChoose = new Image(Assets.instance.design.findRegion("inChoose"));
        own = new Image(Assets.instance.design.findRegion("own"));
        diamond = new Group();
        diamond.setSize(own.getWidth(), own.getHeight());

        Image diamondBg = new Image(new NinePatch(Assets.instance.newUI.findRegion("baoshi_bg"), 13, 13, 13, 13));
        diamondBg.setSize(107, 58);

        Image crystal = new Image(new NinePatch(Assets.instance.newUI.findRegion("designzuanshi"), 10, 10, 10, 10));
        crystal.setOrigin(Align.center);
        crystal.setScale(0.65f);
        crystal.setRotation(-10);
        crystal.setPosition(12, 1);

        Image x = new Image(new NinePatch(Assets.instance.newUI.findRegion("_1")));
        x.setScale(0.625f);
        x.setPosition(51, 20);

        Label.LabelStyle style = new Label.LabelStyle();
        Assets.instance.fonts[2].getData().ascent = 0;
        Assets.instance.fonts[2].setUseIntegerPositions(false);
        Assets.instance.fonts[2].getData().capHeight = Assets.instance.fonts[2].getData().lineHeight;
        style.font = Assets.instance.fonts[2];
        style.fontColor = new Color(0xffe700ff);
        Label m = new Label("" + design.Unlocking_condition, style);
        m.setFontScale(0.6f);
        m.setScale(0.6f);
        m.setPosition(69, -17);

        diamond.addActor(diamondBg);
        diamond.addActor(m);
        diamond.addActor(x);
        diamond.addActor(crystal);

        show.setPosition(getWidth() / 2, (213 + inUse.getHeight()) / 2, Align.center);
        designBack.setOrigin(Align.center);
        designBack.setSize(show.getWidth() + 3, show.getHeight() + 3);
        designBack.setPosition(show.getX(Align.center), show.getY(Align.center), Align.center);
        if (demoSpineActor != null) {
            demoSpineActor.setPosition(show.getX(Align.center), show.getY(Align.center) + offsetH, Align.center);
        }

        designBack.setColor(33 / 255f, 37 / 255f, 41 / 255f, 0.7f);
        designBack.setTouchable(Touchable.disabled);

        inChoose.setPosition(getWidth(), getHeight() - 120, Align.center);
        inUse.setPosition(
                50,
                inUse.getHeight() / 2,
                Align.center
        );
        if (inuseSpineActor != null) {
            inuseSpineActor.setPosition(50, inUse.getHeight() / 2, Align.center);
        }

        own.setPosition(
                50,
                own.getHeight() / 2,
                Align.center
        );
        diamond.setPosition(
                50,
                own.getHeight() / 2,
                Align.center
        );


        addActor(show);
        addActor(designBack);

        addActor(own);
        addActor(inUse);
        if (inuseSpineActor != null) {
            addActor(inuseSpineActor);
        }
        addActor(diamond);
        addActor(inChoose);
        if (demoSpineActor != null) {
            addActor(demoSpineActor);
        }

        own.setVisible(false);
        inUse.setVisible(false);
        inChoose.setVisible(false);

        show.clearListeners();
        show.setOrigin(Align.center);
        show.addListener(new ButtonListenrNoSound() {
            @Override
            public void clicked(InputEvent event, float x, float y) {
                super.clicked(event, x, y);
                DominoGame.showUnlockDialgo(design);
                //////////////////////////// DNA ////////////////////////////////
                DeltaDNAManager.addUiInteraction(skinName, "skin", design.ID + "", skinNameF + design.ID);
                /////////////////////////////////////////////////////////////////
            }
        });

        if (DiamondDataUtil.isUnlockDesign(design.ID) || desig.Unlocking_condition == 0) {
            setOwn();
        }

        if (DiamondDataUtil.getUseDesign(designType) == design.ID) {
            setInUse();

            if (DesignDialogNew.isUnlock > 0 && DesignDialogNew.isUnlock == design.ID) {
                DesignDialogNew.isUnlock = -1;
                designBack.clearActions();
                designBack.addAction(
                        Actions.sequence(
                                Actions.alpha(0, 0.333f),
                                Actions.hide()
                        )
                );
                if (demoSpineActor != null) {
                    demoSpineActor.setAnimation("animation", false);
                }
                if (inuseSpineActor != null) {
                    inuseSpineActor.setAnimation("animation", false);
                }
            } else {
                designBack.setVisible(false);
            }
        }

//        if (demoSpineActor != null) {
//            demoSpineActor.setAnimation("animation", false);
//        }
//
//        if (inuseSpineActor != null) {
//            inuseSpineActor.setAnimation("animation", false);
//        }
    }


    public void setOwn() {
        designBack.setVisible(false);
        diamond.setVisible(false);
        own.setVisible(true);
        inUse.setVisible(false);

        show.clearListeners();
        show.addListener(new ButtonListenrNoSound() {

            @Override
            public void clicked(InputEvent event, float x, float y) {
                super.clicked(event, x, y);
                FlurryManager.flurryLog_design_switch();
                String type;
                String sid;
                if (designType == DesignDialogNew.DesignType.dominoes) {

                    DiamondDataUtil.useDesign(design.ID, designType);
                    DominoGame.showDominoes(design.Topic_name + "-");
                    type = "dominoe";
                } else {
                    DiamondDataUtil.useDesign(design.ID, designType);
                    DominoGame.showBg(design.ID);
                    type = "background";
                }
                /////////////////////////// DNA ////////////////////////////
                DeltaDNAManager.addSkinUsed(type, design.ID + "", false, DiamondDataUtil.getDiamondNum());
                DeltaDNAManager.addUiInteraction(skinName, "skin", "skin " + design.ID, skinNameF + design.ID);
                ////////////////////////////////////////////////////////////
            }
        });
    }

    public void setInUse() {
        diamond.setVisible(false);

        inChoose.setVisible(true);
        inChoose.setOrigin(Align.center);
        setScaleAnimation(inChoose, 0.298f, 1, 0, 0.3f, null, Interpolation.swingIn);
        own.setOrigin(Align.center);
        setScaleAnimation(own, 1, 0.298f, 0, 0.15f, new Runnable() {
            @Override
            public void run() {
                own.setVisible(false);
            }
        }, Interpolation.sineIn);
        inUse.setVisible(true);
        inUse.setOrigin(Align.center);
        setScaleAnimation(inUse, 0.298f, 1, 0.15f, 0.15f, null, Interpolation.sineOut);
    }

    public void setScaleAnimation(Actor actor, float startScale, float endScale, float deltaTiem, float time, Runnable runnable, Interpolation interpolation) {
        setScaleAnimation(actor, startScale, startScale, endScale, endScale, deltaTiem, time, runnable, interpolation);
    }

    private void setScaleAnimation(Actor actor, float startScaleX, float startScaleY, float endScaleX, float endScaleY, float delta, float time, Runnable runnable, Interpolation interpolation) {
        if (actor == null) return;
        actor.clearActions();
        actor.setScale(startScaleX, startScaleY);
        if (runnable != null) {
            actor.addAction(
                    Actions.sequence(
                            Actions.delay(delta, Actions.scaleTo(endScaleX, endScaleY, time, interpolation)),
                            Actions.run(runnable)
                    )
            );
        } else {
            actor.addAction(
                    Actions.delay(delta, Actions.scaleTo(endScaleX, endScaleY, time, interpolation))
            );
        }
    }
}
