package com.mobilegame.dominoes.objects;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.g2d.Batch;
import com.badlogic.gdx.graphics.g2d.NinePatch;
import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.math.Interpolation;
import com.badlogic.gdx.math.MathUtils;
import com.badlogic.gdx.scenes.scene2d.Group;
import com.badlogic.gdx.scenes.scene2d.actions.Actions;
import com.badlogic.gdx.scenes.scene2d.actions.SequenceAction;
import com.badlogic.gdx.scenes.scene2d.ui.Image;
import com.badlogic.gdx.utils.Align;
import com.esotericsoftware.spine.SkeletonData;
import com.esotericsoftware.spine.SkeletonJson;
import com.esotericsoftware.spine.SkeletonRenderer;
import com.esotericsoftware.spine.attachments.AtlasAttachmentLoader;
import com.mobilegame.dominoes.Actor.NinePathTexture;
import com.mobilegame.dominoes.DominoGame;
import com.mobilegame.dominoes.assets.Assets;
import com.mobilegame.dominoes.flurry.ABTestManager;
import com.mobilegame.dominoes.handles.GameConfig;
import com.mobilegame.dominoes.screens.GameScreen;
import com.spine.MySpineActor;

public class DominoCard extends Group implements DominoCardBase {
    private Image bg;
    private Image bottom1;
    private Image bottom;
    private Image[] points = new Image[2];
    private int[] values;
    float deltaX = 0;
    float deltaY = 0;
    float scale = 1f;
    float positionY = 0;

//    private Image lightGreen;
//    private Image lightYellow;
//    private Image lightBlue;

    private Image doublePoint;
    private NinePathTexture lineLightTop;
    private NinePathTexture lineLightBottm;
    private NinePathTexture lineLightLeft;
    private NinePathTexture lineLightRight;

    private NewPointLabel newPointLabel;

    private boolean showSpinner = false;
    private boolean showAllfive = false;
    private boolean showTop = false;
    private boolean showBottom = false;
    private boolean showLeft = false;
    private boolean showRight = false;
    float offsetX = 0;

    public DominoCard(int[] values) {
        this.values = values;

        scale = DominoGame.dominoesScale;
        bg = new Image(new NinePatch(Assets.instance.dominoes.createSprite(DominoGame.dominoeName + "board-dominoes1")));
        bg.setSize(253.5f * scale, 127.5f * scale);

        setSize(bg.getWidth() + 8, bg.getHeight() + 8);
        bg.setPosition(getWidth() / 2, getHeight() / 2, Align.center);

        bottom = new Image(new NinePatch(Assets.instance.dominoes.findRegion(DominoGame.dominoeName + "button"), 30, 30, 0, 0));
        bottom.setSize(127.5f * scale, 30 * scale);
        bottom1 = new Image(new NinePatch(Assets.instance.dominoes.findRegion(DominoGame.dominoeName + "button1"), 30, 30, 0, 0));
        bottom1.setSize(253.5f * scale, 30 * scale);

        bg.setName("bg");
        addActor(bottom);
        addActor(bottom1);
        addActor(bg);

        bottom.setVisible(false);
        bottom1.setX(getWidth() / 2, Align.center);
        bottom1.setY(bg.getY() + 6 * scale, Align.center);
        bottom.setX(getWidth() / 2, Align.center);
        bottom.setY(-2);

        for (int i = 0; i < 2; i++) {
            if (values[i] == 0)
                continue;
            if (values[i] != 0) {
                points[i] = new Image(Assets.instance.dominoes.createSprite(DominoGame.dominoeName + "point-" + values[i]));
                points[i].setSize(90 * scale, 90 * scale);
                points[i].setOrigin(Align.center);
                points[i].setRotation(90);
                points[i].setVisible(true);
            }
            points[i].setOrigin(Align.center);
            addActor(points[i]);
        }
        if (points[0] != null) {
            points[0].setPosition(getWidth() / 4, getHeight() / 2, Align.center);
        }
        if (points[1] != null) {
            points[1].setPosition(getWidth() / 4 * 3, getHeight() / 2, Align.center);
        }
        setOrigin(getWidth() / 4F, getHeight() / 2);

        doublePoint = new Image(Assets.instance.dominoes.findRegion("spinner"));
        doublePoint.setVisible(false);
        doublePoint.setOrigin(Align.center);
        doublePoint.setScale(1.8f);
        doublePoint.setPosition(getWidth() / 2f, getHeight() / 2f, Align.center);
        addActor(doublePoint);

        if (ABTestManager.currentTest != ABTestManager.C) {
            TextureRegion lineLightRegion = Assets.dominoes.findRegion("guang");
            NinePatch lineLightNine = new NinePatch(lineLightRegion, 1, 1, 0, 0);
            float width = bg.getHeight();
            float height = width / lineLightRegion.getRegionWidth() * lineLightRegion.getRegionHeight();
            lineLightTop = new NinePathTexture(lineLightNine);
            lineLightTop.setVisible(false);
            lineLightTop.setSize(width, height);
            lineLightTop.setOrigin(Align.bottom);
            lineLightTop.setRotation(90);
            lineLightTop.setPosition(-offsetX, bg.getHeight() / 2f + 4, Align.bottom);

            lineLightBottm = new NinePathTexture(lineLightNine);
            lineLightBottm.setVisible(false);
            lineLightBottm.setSize(width, height);
            lineLightBottm.setOrigin(Align.bottom);
            lineLightBottm.setRotation(-90);
            lineLightBottm.setPosition(bg.getWidth() + 6 + offsetX, bg.getHeight() / 2f + 4, Align.bottom);

            lineLightLeft = new NinePathTexture(lineLightNine);
            lineLightLeft.setVisible(false);
            lineLightLeft.setSize(width, height);
            lineLightLeft.setOrigin(Align.bottom);
            lineLightLeft.setPosition(bg.getWidth() / 2f + 3.5f, bg.getHeight() + 7 + offsetX, Align.bottom);

            lineLightRight = new NinePathTexture(lineLightNine);
            lineLightRight.setVisible(false);
            lineLightRight.setSize(width, height);
            lineLightRight.setOrigin(Align.bottom);
            lineLightRight.setRotation(180);
            lineLightRight.setPosition(bg.getWidth() / 2f + 3.5f, -7 - offsetX, Align.bottom);
        }

        refreshShow();
    }

    public void gray(boolean set) {
        findActor("gray").setVisible(set);
    }

    public void setSpinner() {
        showSpinner = true;
    }

    public void highLight(int state) {
//        if (ABTestManager.currentTest == ABTestManager.A
//                || ABTestManager.currentTest == ABTestManager.E
//                || GameConfig.gameType != GameConfig.ALL_FIVES_TYPE) {
//            if (state == DominoCardCommon.GREEN) {
//                lightYellow.setVisible(false);
//                lightGreen.setVisible(true);
//            } else if (state == DominoCardCommon.YELLOW) {
//                lightGreen.setVisible(false);
//                lightYellow.setVisible(true);
//            } else {
//                lightYellow.setVisible(false);
//                lightGreen.setVisible(false);
//            }
//        }

        if (state == -1) {
            removeAllFiveAddPointAction();
            GameScreen.isAddPoint = false;
        } else {
            if (GameScreen.isAddPoint) {
                addAllfiveAddPointAction();
            }
        }
    }

    public void updatePosition(int x, int y, final float rotation, final Runnable runnable) {
        float tempX = getWidth() / 4 * x;
        float tempY = getWidth() / 4 * y;
        GameConfig.aniTime = 0.002F * MathUtils.length(getX(), getY(), tempX - originX, tempY - originY);
        if (GameConfig.aniTime < 0.6F) {
            GameConfig.aniTime = 0.6F;
        }
        if (GameConfig.aniTime > 0.7F) {
            GameConfig.aniTime = 0.7F;
        }

        GameConfig.aniTime = 0.7f;

        bg.setOrigin(Align.center);
        if (rotation == 180) {

            bottom1.setVisible(true);
            bottom.setVisible(false);
            deltaX = 0;
            deltaY = -2f;

            bottom1.setOrigin(Align.center);
            bottom1.setRotation(180);
            bottom1.setY(bg.getHeight() + bg.getY() - 6f * scale, Align.center);
            bottom1.setX(getWidth() / 2, Align.center);

            if (lineLightTop != null) {
                lineLightTop.setPosition(-offsetX, bg.getHeight() / 2f + 4, Align.bottom);
            }
            if (lineLightBottm != null) {
                lineLightBottm.setPosition(bg.getWidth() + 6 + offsetX, bg.getHeight() / 2f + 4, Align.bottom);
            }
            if (lineLightLeft != null) {
                lineLightLeft.setPosition(bg.getWidth() / 2f + 3.5f, bg.getHeight() + 13 + offsetX, Align.bottom);
            }
            if (lineLightRight != null) {
                lineLightRight.setPosition(bg.getWidth() / 2f + 3.5f, -1 - offsetX, Align.bottom);
            }
        } else if (rotation == 90 || rotation == -90) {

            bottom.setVisible(true);
            bottom1.setVisible(false);

            deltaX = 1.5f * rotation / 90;
            deltaY = 0f;
            bottom.setOrigin(Align.center);

            if (rotation == 90) {
                bottom.setRotation(90);
                bottom.setPosition(bg.getX() + bg.getWidth() - 5f * scale, bg.getY() + bg.getHeight() / 2, Align.center);
                if (lineLightTop != null) {
                    lineLightTop.setPosition(-offsetX, bg.getHeight() / 2f + 4, Align.bottom);
                }
                if (lineLightBottm != null) {
                    lineLightBottm.setPosition(bg.getWidth() + 13 + offsetX, bg.getHeight() / 2f + 4, Align.bottom);
                }
                if (lineLightLeft != null) {
                    lineLightLeft.setPosition(bg.getWidth() / 2f + 3.5f, bg.getHeight() + 7 + offsetX, Align.bottom);
                }
                if (lineLightRight != null) {
                    lineLightRight.setPosition(bg.getWidth() / 2f + 3.5f, 1 - offsetX, Align.bottom);
                }
            } else {
                bottom.setRotation(-90);
                bottom.setPosition(bg.getX() + 5f * scale, bg.getY() + bg.getHeight() / 2, Align.center);
                if (lineLightTop != null) {
                    lineLightTop.setPosition(-offsetX - 7, bg.getHeight() / 2f + 4, Align.bottom);
                }
                if (lineLightBottm != null) {
                    lineLightBottm.setPosition(bg.getWidth() + 6 + offsetX, bg.getHeight() / 2f + 4, Align.bottom);
                }
                if (lineLightLeft != null) {
                    lineLightLeft.setPosition(bg.getWidth() / 2f + 3.5f, bg.getHeight() + 7 + offsetX, Align.bottom);
                }
                if (lineLightRight != null) {
                    lineLightRight.setPosition(bg.getWidth() / 2f + 3.5f, 1 - offsetX, Align.bottom);
                }
            }
        } else {
            bottom1.setVisible(true);
            bottom.setVisible(false);
            bottom1.setRotation(0);
            deltaX = 0;
            deltaY = 2f;
            bottom1.setX(getWidth() / 2, Align.center);
            bottom1.setY(9f * scale, Align.center);

            if (lineLightTop != null) {
                lineLightTop.setPosition(-offsetX, bg.getHeight() / 2f + 4, Align.bottom);
            }
            if (lineLightBottm != null) {
                lineLightBottm.setPosition(bg.getWidth() + 6 + offsetX, bg.getHeight() / 2f + 4, Align.bottom);
            }
            if (lineLightLeft != null) {
                lineLightLeft.setPosition(bg.getWidth() / 2f + 3.5f, bg.getHeight() + 7 + offsetX, Align.bottom);
            }
            if (lineLightRight != null) {
                lineLightRight.setPosition(bg.getWidth() / 2f + 3.5f, -7 - offsetX, Align.bottom);
            }
        }

        float endX = tempX - getOriginX();
        float endY = tempY - originY;

        if (runnable != null) {
            toFront();
        }

        positionY = tempY - originY;
        addAction(Actions.rotateTo(-rotation, GameConfig.aniTime, Interpolation.pow4Out));
        SequenceAction sequenceAction = Actions.sequence(
                Actions.moveTo(endX, endY, GameConfig.aniTime, Interpolation.pow4Out),
                Actions.run(new Runnable() {
                    @Override
                    public void run() {
                        if (showSpinner) {
                            doublePoint.setVisible(true);
                        }
                        if (showAllfive) {
                            showAllfive = false;
                        }

                        if (showTop) { //上
                            if (lineLightTop != null) {
                                lineLightTop.setVisible(true);
                            }
                            showTop = false;
                        }

                        if (showBottom) {
                            if (lineLightBottm != null) {
                                lineLightBottm.setVisible(true);
                            }
                            showBottom = false;
                        }

                        if (showLeft) {
                            if (lineLightLeft != null) {
                                lineLightLeft.setVisible(true);
                            }
                            showLeft = false;
                        }

                        if (showRight) {
                            if (lineLightRight != null) {
                                lineLightRight.setVisible(true);
                            }
                            showRight = false;
                        }

                        if (runnable != null) {
                            runnable.run();
                        }
                    }
                })
        );
        addAction(sequenceAction);
    }

    public void refreshShow() {
        boolean isBgsetVisible = bg.isVisible();
        boolean isBottomsetVisible = bottom.isVisible();
        boolean isBottom1setVisible = bottom1.isVisible();
        float boX = bottom.getX();
        float boY = bottom.getY();
        float bo1X = bottom1.getX();
        float bo1Y = bottom1.getY();
        float bgRotation = bg.getRotation();
        float boRotation = bottom.getRotation();
        float bo1Rotation = bottom1.getRotation();

        removeActor(bg);
        removeActor(points[0]);
        removeActor(points[1]);
        removeActor(bottom);
        removeActor(bottom1);

        bg = new Image(new NinePatch(Assets.instance.dominoes.createSprite(DominoGame.dominoeName + "board-dominoes1")));
        bottom = new Image(new NinePatch(Assets.instance.dominoes.findRegion(DominoGame.dominoeName + "button")));
        bottom1 = new Image(new NinePatch(Assets.instance.dominoes.findRegion(DominoGame.dominoeName + "button1")));

        bg.setName("bg");
        addActor(bottom);
        addActor(bottom1);
        addActor(bg);

        bg.setSize(253.5f * scale, 127.5f * scale);
        bottom.setSize(127.5f * scale, 30 * scale);
        bottom1.setSize(253.5f * scale, 30 * scale);

        bottom.setPosition(boX, boY);
        bottom.setOrigin(Align.center);
        bottom.setVisible(isBottomsetVisible);
        bottom.setRotation(boRotation);

        bottom1.setPosition(bo1X, bo1Y);
        bottom1.setOrigin(Align.center);
        bottom1.setVisible(isBottom1setVisible);
        bottom1.setRotation(bo1Rotation);

        bg.setVisible(isBgsetVisible);
        bg.setOrigin(Align.center);
        bg.setRotation(bgRotation);

        setSize(bg.getWidth() + 6, bg.getHeight() + 8);
        bg.setPosition(getWidth() / 2, getHeight() / 2, Align.center);

        for (int i = 0; i < 2; i++) {
            if (values[i] == 0) continue;
            if (values[i] != 0) {
                points[i] = new Image(new NinePatch(Assets.instance.dominoes.createSprite(DominoGame.dominoeName + "point-" + values[i])));
                points[i].setSize(90 * scale, 90 * scale);
                points[i].setOrigin(Align.center);
                points[i].setRotation(90);
                points[i].setVisible(true);
            }
            points[i].setOrigin(Align.center);
            addActor(points[i]);
        }
        if (points[0] != null) {
            points[0].setPosition(getWidth() / 4, getHeight() / 2, Align.center);
        }
        if (points[1] != null) {
            points[1].setPosition(getWidth() / 4 * 3, getHeight() / 2, Align.center);
        }

        if (lineLightTop != null) {
            addActorBefore(bg, lineLightTop);
        }
        if (lineLightBottm != null) {
            addActorBefore(bg, lineLightBottm);
        }
        if (lineLightLeft != null) {
            addActorBefore(bg, lineLightLeft);
        }
        if (lineLightRight != null) {
            addActorBefore(bg, lineLightRight);
        }
        addActor(doublePoint);
    }

    public void addAllfiveAction() {
        String[] strings = {"lan", "lv", "zi", "hui", "zong", "huang"};
        SkeletonJson sJsonAct;
        SkeletonData skeletonDataLoading;
        sJsonAct = new SkeletonJson(new AtlasAttachmentLoader(Assets.instance.kuang));
        skeletonDataLoading = sJsonAct.readSkeletonData(Gdx.files.internal("animation/kuang.json"));
        MySpineActor mySpineActor = new MySpineActor(new SkeletonRenderer(), skeletonDataLoading);
        mySpineActor.setSpeed(0.6f);
        int index = DominoGame.bgNum - 10;
        mySpineActor.getAnimationState().addAnimation(0, strings[index], false, 0);

        mySpineActor.setPosition(getWidth() / 2, getHeight() / 2);
        addActor(mySpineActor);
    }

    public void addAllfiveAddPointAction() {
//        addPointActor.setVisible(true);
//        addPointGuangActor.setVisible(true);
    }

    public void removeAllFiveAddPointAction() {
//        if (addPointActor != null) {
//            addPointActor.setVisible(false);
//        }
//        if (addPointGuangActor != null) {
//            addPointGuangActor.setVisible(false);
//        }
    }

    private NewPointLabel getNewPointLabel(boolean isLabel) {
        if (newPointLabel == null) {
            newPointLabel = new NewPointLabel(bg.getWidth(), bg.getHeight(), isLabel);
        }
        return newPointLabel;
    }

    public void setNewPointLabelAlpha(boolean isLabel, boolean isFar) {
        getNewPointLabel(isLabel).setImageBgFar(isFar);
    }

    public void addPointHint(int point, int bearing, boolean isDouble, boolean isLabel) {
        NewPointLabel newPointLabel = getNewPointLabel(isLabel);
        newPointLabel.setPoint(point);
        if (point != 0 && point % 5 == 0) {
            newPointLabel.setColor(true);
        } else {
            newPointLabel.setColor(false);
        }
        newPointLabel.setBgRotation(0);
        if (bearing == 0) {
            if (isDouble) {
                newPointLabel.setBgRotation(90);
                newPointLabel.setPosition(getWidth() + getHeight() / 2f, getHeight() / 2f, Align.center);
            } else {
                newPointLabel.setPosition(getWidth(), getHeight() / 2f, Align.left);
            }
        } else if (bearing == 1) {
            if (isDouble) {
                newPointLabel.setBgRotation(90);
                newPointLabel.setPosition(-getHeight() / 2f, getHeight() / 2f, Align.center);
            } else {
                newPointLabel.setPosition(0, getHeight() / 2f, Align.right);
            }
        } else if (bearing == 2) {
            if (isDouble) {
                newPointLabel.setPosition(getWidth() / 2f, getHeight(), Align.bottom);
            } else {
                newPointLabel.setBgRotation(90);
                newPointLabel.setPosition(getWidth() / 2f, getHeight() + newPointLabel.getWidth() / 2f, Align.center);
            }
        } else if (bearing == 3) {
            if (isDouble) {
                newPointLabel.setPosition(getWidth() / 2f, 0, Align.top);
            } else {
                newPointLabel.setBgRotation(90);
                newPointLabel.setPosition(getWidth() / 2f, -newPointLabel.getWidth() / 2f, Align.center);
            }
        }
        addActor(newPointLabel);
    }

    public void removePointHint(boolean isLabel) {
        if (getNewPointLabel(isLabel) != null) {
            getNewPointLabel(isLabel).remove();
            newPointLabel = null;
        }
    }

    public void setPointLabelRotation(boolean isLabel, float rotation) {
        if (getNewPointLabel(isLabel) != null) {
            getNewPointLabel(isLabel).setLabelRotation(rotation);
        }
    }

    @Override
    public void draw(Batch batch, float parentAlpha) {
        if (transform) applyTransform(batch, computeTransform());
        drawChildren(batch, parentAlpha);

//        if(frameBg!=null){
//            frameBg.setOriginBasedPosition(getWidth()/2 + 1F+deltaX,getHeight()/2 + 0.5F+deltaY);
////            frameBg.setRotation(getRotation());
//            frameBg.draw(batch);
//        }
        if (transform) resetTransform(batch);
    }

    public void setLineLightPosition(boolean top, boolean bottom, boolean left, boolean right, boolean isDelay) {
        if (top) { //上
            if (isDelay) {
                showTop = true;
            } else {
                if (lineLightTop != null) {
                    lineLightTop.setVisible(true);
                }
                showTop = false;
            }
        }

        if (bottom) {
            if (isDelay) {
                showBottom = true;
            } else {
                if (lineLightBottm != null) {
                    lineLightBottm.setVisible(true);
                }
                showBottom = false;
            }
        }

        if (left) {
            if (isDelay) {
                showLeft = true;
            } else {
                if (lineLightLeft != null) {
                    lineLightLeft.setVisible(true);
                }
                showLeft = false;
            }
        }

        if (right) {
            if (isDelay) {
                showRight = true;
            } else {
                if (lineLightRight != null) {
                    lineLightRight.setVisible(true);
                }
                showRight = false;
            }
        }
    }

    public void setHideLineLight(boolean isShow) {
        if (lineLightTop != null) {
            lineLightTop.setVisible(isShow);
        }
        showTop = false;

        if (lineLightBottm != null) {
            lineLightBottm.setVisible(isShow);
        }
        showBottom = false;

        if (lineLightLeft != null) {
            lineLightLeft.setVisible(isShow);
        }
        showLeft = false;

        if (lineLightRight != null) {
            lineLightRight.setVisible(isShow);
        }
        showRight = false;
    }

    public float getPositionY() {
        return positionY;
    }
}
