package com.mobilegame.dominoes.objects;

import com.badlogic.gdx.graphics.g2d.Batch;
import com.badlogic.gdx.graphics.g2d.NinePatch;
import com.badlogic.gdx.math.MathUtils;
import com.badlogic.gdx.math.Vector2;
import com.badlogic.gdx.scenes.scene2d.Group;
import com.badlogic.gdx.scenes.scene2d.InputEvent;
import com.badlogic.gdx.scenes.scene2d.actions.Actions;
import com.badlogic.gdx.scenes.scene2d.ui.Image;
import com.badlogic.gdx.scenes.scene2d.utils.ActorGestureListener;
import com.badlogic.gdx.utils.Align;
import com.badlogic.gdx.utils.ArrayMap;
import com.mobilegame.dominoes.DominoGame;
import com.mobilegame.dominoes.assets.Assets;
import com.mobilegame.dominoes.handles.GameConfig;

public class DominoCardPlayer extends Group implements DominoCardBase {
    final public int dominoId;
    private Image bg;
    private Image layer;
    //    private Sprite layer;
    private Image bottom;
    private Image[] points;
    private int[] values;
    private DominoCardPlayerListener cardListener;
    boolean gray;
    float sourceX, sourceY;
    private ArrayMap<Integer, Vector2> possibleUIPos;
    private int possibleMoveIndex = 0;

    public DominoCardPlayer(int id, int[] values, final DominoCardPlayerListener listener) {
//        debug();
        this.dominoId = id;
        this.cardListener = listener;

        this.values = values;

        bg = new Image(new NinePatch(Assets.instance.dominoes.findRegion(DominoGame.dominoeName + "board-dominoes"), 20, 20, 20, 20));
//        Texture maskTexture = new Texture(Gdx.files.internal("image/mask(1).png"), Pixmap.Format.RGBA8888, false);
//        maskTexture.setFilter(Texture.TextureFilter.Linear, Texture.TextureFilter.Linear);
        layer = new Image(new NinePatch(Assets.instance.dominoes.findRegion("mask"), 20, 20, 20, 20));//Assets.instance.dominoes.findRegion("mask")
        bottom = new Image(new NinePatch(Assets.instance.dominoes.findRegion(DominoGame.dominoeName + "button"), 20, 20, 0, 0));
        bg.setSize(89, 177);
        bottom.setWidth(89);
        setSize(bg.getWidth(), bg.getHeight() + bottom.getHeight() / 2);
//        layer.setOrigin(Align.center);
//        layer.setScale(getWidth() / layer.getWidth(), (getHeight() - 5) / layer.getHeight());
        layer.addAction(Actions.alpha(0.6f));
        layer.setSize(getWidth(), getHeight() - 5);

        points = new Image[2];
        for (int i = 0; i < 2; i++) {
            if (values[i] != 0) {
                points[i] = new Image(Assets.instance.dominoes.createSprite(DominoGame.dominoeName + "point-" + values[i]));
                points[i].setSize(60, 60);
            } else {
                points[i] = new Image(Assets.instance.dominoes.createSprite(DominoGame.dominoeName + "point-" + values[i]));
                points[i].setSize(60, 60);
                points[i].setVisible(false);
            }

        }

        float centerX = getWidth() / 2;
        float centerY = getHeight() / 2;
        bg.setPosition(centerX, centerY, Align.center);
        bottom.setY(bg.getX() - 1);
        layer.setX(bg.getX());
        addActor(bottom);
        addActor(bg);
        if (points[0] != null) {
            points[0].setPosition(centerX, centerY + getHeight() / 4, Align.center);
            addActor(points[0]);
        }
        if (points[1] != null) {
            points[1].setPosition(centerX, centerY - getHeight() / 4, Align.center);
            addActor(points[1]);
        }
        sourceX = getX();
        sourceY = getY();
        addActor(layer);
        layer.setVisible(false);

        if (listener != null) {
            addListener(new ActorGestureListener() {
                long touchTime;
                boolean isMorePoint;

                @Override
                public void touchDown(InputEvent event, float x, float y, int pointer, int button) {
                    super.touchDown(event, x, y, pointer, button);
                    if (pointer > 0) {
                        isMorePoint = true;
                        return;
                    }

                    isMorePoint = false;
                    possibleUIPos = null;

                    if (gray) {
                        return;
                    } else {
                        possibleUIPos = cardListener.touchdown(dominoId, possibleMoveIndex);
                        calculatePossibleIndex();
                        for (int i = 0; i < possibleUIPos.size; i++) {
                            Vector2 temp = new Vector2(possibleUIPos.getValueAt(i));
//                            Gdx.app.log("yxk",temp.x+":"+temp.y);
                        }

                        if ((possibleUIPos == null || possibleUIPos.size == 0) && !gray) {
                            gray(true);
                        } else {
                            touchTime = System.currentTimeMillis();
                            setZIndex(Integer.MAX_VALUE);
                            Vector2 closePos = possibleUIPos.getValueAt(possibleMoveIndex);
                            Vector2 pos = getParent().localToStageCoordinates(new Vector2(getX(Align.center), getY(Align.center)));
                            float offsetX = closePos.x - pos.x;
                            offsetX = MathUtils.clamp(offsetX, -30, 30);
                            addAction(Actions.moveBy(offsetX, getHeight() / 4 * 3, 0.1F));

                        }

                    }
                    listener.touchDrag(dominoId, possibleMoveIndex);
                }

                //
//                @Override
//                public void touchDragged(InputEvent event, float x, float y, int pointer) {
//                    super.touchDragged(event, x, y, pointer);
//                    if (gray) return;
//                    if(System.currentTimeMillis() - touchTime < 200 && !( y-starty > 200)){
//                        return;
//                    }
////                    if(isMorePoint) return;
//
//                    setPosition((getX() + x - startx) , (getY() + y-starty));
//                    calculatePossibleIndex();
////                    if(getY() - sourceY > getHeight()){
//                    listener.touchDrag(dominoId, possibleMoveIndex);
//                }
//
                @Override
                public void pan(InputEvent event, float x, float y, float deltaX, float deltaY) {
                    super.pan(event, x, y, deltaX, deltaY);
                    if (gray) return;
//                    if(isMorePoint) return;

                    setPosition((getX() + deltaX), (getY() + deltaY));
                    calculatePossibleIndex();
//                    if(getY() - sourceY > getHeight()){
                    listener.touchDrag(dominoId, possibleMoveIndex);
//                    }


                }

                @Override
                public void touchUp(InputEvent event, float x, float y, int pointer, int button) {
                    super.touchUp(event, x, y, pointer, button);
                    if (pointer > 0)
                        return;
                    if (gray)
                        return;

                    float currentY = getY();
//                    System.out.println("********* touchup　*********"+currentY+"  "+sourceY+"  "+getHeight()+"  "+possibleUIPos.size+"  "+System.currentTimeMillis()+"  "+touchTime+"  "+possibleUIPos.size);
                    if (currentY - sourceY >= getHeight() || (System.currentTimeMillis() - touchTime < 200 && possibleUIPos.size == 1 && GameConfig.gameType != GameConfig.ALL_FIVES_TYPE)) {
                        getActions().clear();
                        Vector2 position = getParent().localToStageCoordinates(new Vector2(getX(Align.center), getY(Align.center)));
                        cardListener.touchUp(position, dominoId, possibleMoveIndex);
                    } else {
                        clearActions();
                        addAction(Actions.moveTo(sourceX, sourceY, 0.05F));
                    }
                    listener.release();
                }
            });
        }
    }

    private void calculatePossibleIndex() {
        possibleMoveIndex = 0;
        if (possibleUIPos == null || possibleUIPos.size == 0) {
            return;
        }
        float sourceX = getX() + getWidth() / 2;
        float sourceY = getY() + getHeight() / 2 + 20;

//        Vector2 temp = new Vector2(possibleUIPos.getValueAt(0));
//        stageToLocalCoordinates(temp);
        float length = 1000000000;
        possibleMoveIndex = 0;
        for (int i = 0; i < possibleUIPos.size; i++) {
            Vector2 temp = new Vector2(possibleUIPos.getValueAt(i));
            Vector2 temp1 = new Vector2(sourceX, sourceY);
//            Gdx.app.log("temp2"+i,"x:"+temp.x+",y:"+temp.y);
//            Gdx.app.log("temp3"+i,"x:"+sourceX+",y:"+sourceY);
//            stageToLocalCoordinates(temp);

            float tempLength = MathUtils.length(sourceX * 2, sourceY, temp.x * 2, temp.y);

            if (tempLength < length) {
                possibleMoveIndex = i;
                length = tempLength;
            }
        }
    }

    @Override
    protected void positionChanged() {
        super.positionChanged();
//        float centerX = getX(Align.center);
//        float centerY = getY(Align.center);
//       layer.setOriginBasedPosition(centerX, centerY+5 );
//        points[0].setOriginBasedPosition(centerX, centerY + getHeight() / 4);
//        points[1].setOriginBasedPosition(centerX, centerY - getHeight() / 4);
    }

    @Override
    public void draw(Batch batch, float parentAlpha) {
        super.draw(batch, parentAlpha);
//        bg.draw(batch);
//        if(points[0]!= null){
//            points[0].draw(batch);
//        }
//        if(points[1]!= null){
//            points[1].draw(batch);
//        }

    }

    public interface DominoCardPlayerListener {
        ArrayMap<Integer, Vector2> touchdown(int dominoId, int closestIndex);

        void touchDrag(int id, int closestIndex);

        void touchUp(Vector2 position, int id, int connectId);

        void release();
    }

    public void refresh() {

        removeActor(bg);
        removeActor(points[0]);
        removeActor(points[1]);
        removeActor(bottom);
        removeActor(layer);
        layer.setVisible(gray);
        bg = new Image(new NinePatch(Assets.instance.dominoes.findRegion(DominoGame.dominoeName + "board-dominoes")));
        bottom = new Image(new NinePatch(Assets.instance.dominoes.findRegion(DominoGame.dominoeName + "button"), 20, 20, 0, 0));
        bg.setSize(89, 177);
        bottom.setWidth(89);

        for (int i = 0; i < 2; i++) {
            if (values[i] != 0) {
                points[i] = new Image(Assets.instance.dominoes.createSprite(DominoGame.dominoeName + "point-" + values[i]));
                points[i].setSize(60, 60);
            }

        }

        float centerX = getWidth() / 2;
        float centerY = getHeight() / 2;

        setSize(bg.getWidth(), bg.getHeight() + bottom.getHeight() / 2);
        bg.setPosition(centerX, centerY, Align.center);
        bottom.setY(bg.getX() - 1);
        addActor(bottom);
        addActor(bg);
        if (points[0] != null) {
            points[0].setPosition(centerX, centerY + getHeight() / 4, Align.center);
            addActor(points[0]);
        }
        if (points[1] != null) {
            points[1].setPosition(centerX, centerY - getHeight() / 4, Align.center);
            addActor(points[1]);
        }
        addActor(layer);
        layer.setX(bg.getX());
        layer.setX(centerX, Align.center);
        layer.setPosition(getWidth()/2f, getHeight()/2f - 2.5f, Align.center);
//        layer.moveBy(0, -2);
    }


    @Override
    public void gray(boolean set) {
        this.gray = set;
        layer.setVisible(set);
    }

    public void changePosition(float x, float y) {
        setPosition(x, y);
        sourceX = x;
        sourceY = y;
    }

    public void resumePosition() {
        refresh();
        setPosition(sourceX, sourceY);
    }

}
