package com.mobilegame.dominoes.objects;

import com.badlogic.gdx.graphics.g2d.Batch;
import com.badlogic.gdx.graphics.g2d.NinePatch;
import com.badlogic.gdx.graphics.glutils.ShaderProgram;
import com.badlogic.gdx.math.MathUtils;
import com.badlogic.gdx.math.Vector2;
import com.badlogic.gdx.scenes.scene2d.Group;
import com.badlogic.gdx.scenes.scene2d.InputEvent;
import com.badlogic.gdx.scenes.scene2d.actions.Actions;
import com.badlogic.gdx.scenes.scene2d.ui.Image;
import com.badlogic.gdx.scenes.scene2d.utils.ActorGestureListener;
import com.badlogic.gdx.utils.Align;
import com.badlogic.gdx.utils.ArrayMap;
import com.mobilegame.dominoes.Actor.DominoActor;
import com.mobilegame.dominoes.DominoGame;
import com.mobilegame.dominoes.assets.Assets;
import com.mobilegame.dominoes.flurry.ABTestManager;
import com.mobilegame.dominoes.handles.GameConfig;

public class DominoCardPlayer2 extends Group implements DominoCardBase {
    final public int dominoId;
    private DominoActor dominoActor;
    private boolean isShader = false;
    private DominoCardPlayerListener cardListener;
    boolean gray;
    boolean isAddPoint;
    float sourceX, sourceY;
    private ArrayMap<Integer, Vector2> possibleUIPos;
    private int possibleMoveIndex = 0;
    private Vector2 v = new Vector2();

    //    MySpineGroup addPointGroup;
    Image addPointGuang;

    public DominoCardPlayer2(int id, int[] values, final DominoCardPlayerListener listener) {
        this.dominoId = id;
        this.cardListener = listener;

        dominoActor = new DominoActor(Assets.dominoes, values) {
            @Override
            public void draw(Batch batch, float parentAlpha) {
                ShaderProgram spp = batch.getShader();
                if (isShader) {
                    batch.setShader(DominoGame.sp);
                }
                super.draw(batch, parentAlpha);
                batch.setShader(spp);
            }
        };
        addActor(dominoActor);

//        SkeletonJson sJsonAct = new SkeletonJson(new AtlasAttachmentLoader(new TextureAtlas("animation/pg.atlas")));
//        SkeletonData skeletonDataLoading = sJsonAct.readSkeletonData(Gdx.files.internal("animation/pg.json"));
//        addPointGroup = new MySpineGroup(new SkeletonRenderer(), skeletonDataLoading);
//        addPointGroup.setRotation(90);
//        addPointGroup.setPosition(dominoActor.getWidth() / 2f, dominoActor.getHeight() / 2f);
//        addPointGroup.setScale(0.61f);
//        addPointGroup.setAnimation("animation1");
//        addPointGroup.setVisible(false);
//        addActor(addPointGroup);

        setSize(dominoActor.getWidth(), dominoActor.getHeight());

        if (ABTestManager.currentTest != ABTestManager.B) {
            addPointGuang = new Image(new NinePatch(Assets.dominoes.findRegion("kuang"), 16, 16, 16, 16));
            addPointGuang.setVisible(false);
            addPointGuang.setSize(getWidth(), getHeight());
            addActor(addPointGuang);
        }

        dominoActor.setPosition(0, 0);

        sourceX = getX();
        sourceY = getY();


        if (listener != null) {
            addListener(new ActorGestureListener() {
                long touchTime;
                int isMorePoint = 0;

                Vector2 oldXY = new Vector2();

                @Override
                public void touchDown(InputEvent event, float x, float y, int pointer, int button) {
                    isMorePoint = pointer;
                    if (pointer > 0) {
                        return;
                    }
                    oldXY.set(event.getListenerActor().getX(), event.getListenerActor().getY());

                    possibleUIPos = null;

                    if (gray) {
                        return;
                    } else {
                        possibleUIPos = cardListener.touchdown(dominoId, possibleMoveIndex);
                        calculatePossibleIndex();
//                        for (int i = 0; i < possibleUIPos.size; i++) {
//                            Vector2 temp = new Vector2(possibleUIPos.getValueAt(i));
////                            Gdx.app.log("yxk",temp.x+":"+temp.y);
//                        }

                        if ((possibleUIPos == null || possibleUIPos.size == 0) && !gray) {
                            gray(true);
                        } else {
                            touchTime = System.currentTimeMillis();
                            setZIndex(Integer.MAX_VALUE);
                            Vector2 closePos = possibleUIPos.getValueAt(possibleMoveIndex);
                            Vector2 pos = getParent().localToStageCoordinates(new Vector2(getX(Align.center), getY(Align.center)));
                            float offsetX = closePos.x - pos.x;
                            offsetX = MathUtils.clamp(offsetX, -30, 30);
                            addAction(Actions.moveBy(offsetX, getHeight() / 4 * 3, 0.1F));
                        }
                    }
                    listener.touchDrag(dominoId, possibleMoveIndex);

//                    if (GameScreen.isAddPoint) {
//                        addAllfiveAddPointAction();
//                        GameScreen.isAddPoint = false;
//                    }
                }

                @Override
                public void pan(InputEvent event, float x, float y, float deltaX, float deltaY) {
                    if (isMorePoint > 0) return;
                    if (gray) return;
                    setPosition((getX() + deltaX), (getY() + deltaY));
                    calculatePossibleIndex();
                    listener.touchDrag(dominoId, possibleMoveIndex);
                }

                @Override
                public void touchUp(InputEvent event, float x, float y, int pointer, int button) {
                    if (pointer > 0) {
                        isMorePoint--;
                        return;
                    }
                    if (gray)
                        return;

                    float currentY = getY();
                    if (currentY - sourceY >= getHeight() || (System.currentTimeMillis() - touchTime < 200 && possibleUIPos.size == 1 && GameConfig.gameType != GameConfig.ALL_FIVES_TYPE)) {
                        getActions().clear();
                        Vector2 position = getParent().localToStageCoordinates(new Vector2(getX(Align.center), getY(Align.center)));
                        cardListener.touchUp(position, dominoId, possibleMoveIndex);
                    } else {
                        clearActions();
                        addAction(Actions.moveTo(oldXY.x, oldXY.y, 0.05F));
                        cardListener.touchNoConnect();
                    }
                    listener.release();

//                    removeAllFiveAddPointAction();
                }
            });
        }

    }

//    private Array<Actor> actors = new Array<>();
//
//    private void clearDebugActors() {
//        for (int i = 0; i < actors.size; ++i) {
//            getStage().getRoot().removeActor(actors.get(i));
//        }
//    }

    private void calculatePossibleIndex() {
        possibleMoveIndex = 0;
        if (possibleUIPos == null || possibleUIPos.size == 0) {
            return;
        }
        float sourceX = getX() + getWidth() / 2;
        float sourceY = getY() + getHeight() / 2;
        v.set(sourceX, sourceY);
        v = getParent().localToStageCoordinates(v);
        float length = 1000000000;
        possibleMoveIndex = 0;
        for (int i = 0; i < possibleUIPos.size; i++) {
            Vector2 temp = new Vector2(possibleUIPos.getValueAt(i));

            float tempLength = MathUtils.length(v.x, v.y, temp.x, temp.y);

            if (tempLength < length) {
                possibleMoveIndex = i;
                length = tempLength;
            }
        }
    }

    @Override
    protected void positionChanged() {
        super.positionChanged();
    }

    @Override
    public void draw(Batch batch, float parentAlpha) {
        super.draw(batch, parentAlpha);
    }

    public interface DominoCardPlayerListener {
        ArrayMap<Integer, Vector2> touchdown(int dominoId, int closestIndex);

        void touchDrag(int id, int closestIndex);

        void touchUp(Vector2 position, int id, int connectId);

        void touchNoConnect();

        void release();
    }

    public void refresh() {
        dominoActor.refresh();
    }


    @Override
    public void gray(boolean set) {
        this.gray = set;
        isShader = set;

        if (!set && isAddPoint) {
            if (addPointGuang != null) {
                addPointGuang.setVisible(true);
            }
        } else {
            if (addPointGuang != null) {
                addPointGuang.setVisible(false);
            }
        }
    }

    public void changePosition(float x, float y) {
        setPosition(x, y);
        sourceX = x;
        sourceY = y;
    }

    public void resumePosition() {
        refresh();
        setPosition(sourceX, sourceY);
    }

    public void addAllfiveAddPointAction() {
        isAddPoint = true;
        if (gray) {
            if (addPointGuang != null) {
                addPointGuang.setVisible(false);
            }
        } else {
            if (addPointGuang != null) {
                addPointGuang.setVisible(true);
            }
        }
    }

    public void removeAllFiveAddPointAction() {
//        if (addPointGroup != null) {
//            addPointGroup.setVisible(false);
//        }
        isAddPoint = false;
        if (addPointGuang != null) {
            addPointGuang.setVisible(false);
        }

    }
}
