package com.mobilegame.dominoes.objects;

import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.Group;
import com.badlogic.gdx.scenes.scene2d.InputEvent;
import com.badlogic.gdx.scenes.scene2d.Touchable;
import com.badlogic.gdx.scenes.scene2d.ui.ScrollPane;
import com.badlogic.gdx.utils.Align;
import com.badlogic.gdx.utils.Timer;
import com.mobilegame.dominoes.DominoGame;
import com.mobilegame.dominoes.assets.Configuration;
import com.mobilegame.dominoes.flurry.FlurryManager;
import com.mobilegame.dominoes.utils.LeagueUtil;
import com.mobilegame.dominoes.utils.Player;
import com.mobilegame.dominoes.utils.listener.ButtonListener2;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;

/**
 * Created by qwe on 2020/10/26.
 * 本轮排行榜的结算
 */

public class LeagueEnd extends Group {

    private Group root;
    private Group table ;
    private ScrollPane pane ;
    private ArrayList<PlayerInfo> playerInfoList = new ArrayList<PlayerInfo>();
    public boolean isShow;
    public boolean isAction;
    public Actor back;

    public Actor btn_adsClaim;
    public Actor btn_Claim;
    public LeagueMain.LeagueListener listener;

    public LeagueEnd(Group root,LeagueMain.LeagueListener listener) {
        this.root = root;
        this.listener = listener;

        addActor(root);
        init();
    }

    private void init() {

        table = new Group();
        pane = new ScrollPane(table);
        isAction = false;

        //返回按钮
        back = root.findActor("btn_back");

        back.setTouchable(Touchable.enabled);
        back.addListener(new ButtonListener2() {
            @Override
            public void clicked(InputEvent event, float x, float y) {
                super.clicked(event, x, y);
                close();
            }
        });

        //看视频按钮
        btn_adsClaim = root.findActor("btn_adsClaim");

        btn_adsClaim.setTouchable(Touchable.enabled);
        btn_adsClaim.addListener(new ButtonListener2() {
            @Override
            public void clicked(InputEvent event, float x, float y) {
                super.clicked(event, x, y);
                if(!isShow)
                    return;
                if(DominoGame.launcherListener.isRewardAdReady()){
                    DominoGame.launcherListener.showRewardAd();
                    DominoGame.adsType = DominoGame.AdsType.Claim;
                    FlurryManager.flurryLog_Video_click_jiesuan();
                    Claim();
                }


            }
        });
        //CLAIM REWARDS 按钮
        btn_Claim = root.findActor("btn_Claim");
        btn_Claim.setTouchable(Touchable.enabled);
        btn_Claim.addListener(new ButtonListener2() {
            @Override
            public void clicked(InputEvent event, float x, float y) {
                super.clicked(event, x, y);
                if(!isShow)
                    return;
                Claim();
            }
        });



        changePositon(back);

        Group title = root.findActor("title");
        changePositon(title);

        Group maptitle = root.findActor("maptitle");
        changePositon(maptitle);

        Actor rankBg =  root.findActor("rank");
        changeHeight(rankBg);

        Actor Bg =  root.findActor("bg");
        changeHeight(Bg);

        pane.setSize(
                root.findActor("rank").getWidth(),
                root.findActor("rank").getHeight()-120
        );
        pane.setPosition(
                root.findActor("rank").getX(Align.center),
                root.findActor("rank").getY(Align.center),
                Align.center);


        update();
        addActor(pane);

    }

    private void Claim() {

        int rank = LeagueUtil.user.getRank();

        if(rank <= LeagueUtil.heartNum.length){
            LeagueUtil.addLife(LeagueUtil.heartNum[rank-1]);
        }

        if(rank <= LeagueUtil.coinNum.length) {
            LeagueUtil.addCoins(LeagueUtil.coinNum[rank - 1]);
        }

        FlurryManager.flurryLog_score_league(LeagueUtil.user.getScore());
        FlurryManager.flurryLog_league_rankLeague(rank);

        close();
        LeagueUtil.EndLeague();
        FlurryManager.flurryLog_Step(13);
        listener.claim();

        Timer.instance().scheduleTask(new Timer.Task() {
            @Override
            public void run() {
                DominoGame.showLeague();
            }
        },0.3f);

    }


    public void update(){
        playerInfoList.clear();
        table.clear();

        float playerInfoHeight = 0;
        for (Player player: LeagueUtil.players){
            PlayerInfo playerInfo = new PlayerInfo(player);
            playerInfo.upDate();
            playerInfoList.add(playerInfo);
            playerInfoHeight = playerInfo.getHeight();
        }

        table.setSize(
                pane.getWidth(),
                playerInfoList.size()*playerInfoHeight
        );

        Collections.sort(playerInfoList,new Comparator<PlayerInfo>() {

            @Override
            public int compare(PlayerInfo playerInfo, PlayerInfo t1) {

                int diff = playerInfo.rank - t1.rank;
                if (diff > 0) {
                    return -1;
                }else if (diff < 0) {
                    return 1;
                }
                return 0; //相等为0
            }
        });

        float posY = 0;
        for (PlayerInfo playerInfo : playerInfoList){
            table.addActor(playerInfo);
            playerInfo.setX(pane.getWidth()/2,Align.center);
            playerInfo.setY(posY);
            posY+=playerInfo.getHeight();
        }

    }

    public void close() {
        isShow = false;
        listener.close();
        setVisible(false);
    }

    @Override
    public void act(float delta) {
        super.act(delta);
        root.findActor("unclicked").setVisible(!DominoGame.launcherListener.isRewardAdReady());
    }

    public void show(){
        if(isShow)
            return;
        LeagueUtil.loadPlayer();
        update();
        isShow = true;
        setVisible(true);
    }

    public void changePositon(Actor actor){
        float changeY = actor.getY();
        changeY += Configuration.heightOffset ;
        actor.setY(changeY);

    }

    public void changeHeight(Actor actor){

        float changeY = actor.getHeight();
        changeY += Configuration.heightOffset;
        actor.setHeight(changeY);

    }
}
