package com.mobilegame.dominoes.objects;

import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.Group;
import com.badlogic.gdx.scenes.scene2d.InputEvent;
import com.badlogic.gdx.scenes.scene2d.Touchable;
import com.badlogic.gdx.scenes.scene2d.actions.Actions;
import com.badlogic.gdx.scenes.scene2d.ui.Image;
import com.badlogic.gdx.scenes.scene2d.utils.ClickListener;
import com.badlogic.gdx.utils.Align;
import com.badlogic.gdx.utils.Timer;
import com.mobilegame.dominoes.DominoGame;
import com.mobilegame.dominoes.assets.Assets;
import com.mobilegame.dominoes.assets.Configuration;
import com.mobilegame.dominoes.data.FileUtil;
import com.mobilegame.dominoes.dialogs.DialogInterface;
import com.mobilegame.dominoes.flurry.DeltaDNAManager;
import com.mobilegame.dominoes.flurry.FlurryManager;
import com.mobilegame.dominoes.handles.GameConfig;
import com.mobilegame.dominoes.utils.listener.ButtonListener2;


/**
 * Created by Doodle on 2021/1/8.
 */

public class MainHelpNew extends Group implements DialogInterface {
    private MainHelpListener listener;
    private Actor back;
    private Actor startNewButton;
    Image[] info = new Image[3];
    private boolean isShow;

    float changeX = -1;
    float changeY = 0;

    private Group scene;
    private Image screenBG;
    private Actor middleBG;
    private Actor backClick;
    private Actor topBg;

    private Actor[] modeClicks;
    private Actor[] modeNoClick;
    private Actor[] modeLabel;
    private Actor[] scoreClicks;
    private Actor[] scoreNoClicks;
    private Actor[] scoreLabel;
    private Actor[] difficultClicks;
    private Actor[] difficultNoClicks;
    private Actor[] difficultLabel;
    private Actor[] modeClickScopes;
    private Actor[] scoreClickScopes;
    private Actor[] difficultClickScopes;

    private Group upGroup;
    private Group middleGroup;

    private float oldUpGroupY;
    private float oldBackX;

    String[] name = new String[]{"drawInfo", "blockInfo", "allfivesInfo"};

    public MainHelpNew(Group root, DominoGame game, MainHelpListener listener) {
        scene = root;
        Assets.instance.assetManager.load("image/endBG.png", Texture.class);
        Assets.instance.assetManager.finishLoading();
        screenBG = new Image(Assets.instance.assetManager.get("image/endBG.png", Texture.class));
        addActor(screenBG);

        this.listener = listener;
        addActor(root);

        topBg = root.findActor("tuceng_3");
        upGroup = new Group();
        scene.addActor(upGroup);

        upGroup.setSize(topBg.getWidth(), topBg.getHeight());
        upGroup.setPosition(topBg.getX(), topBg.getY());
        oldUpGroupY = upGroup.getY();

        topBg.setY(topBg.getY() - upGroup.getY());
        upGroup.addActor(topBg);

        Actor titleActor = root.findActor("BitmapFontLabel_3");
        titleActor.setY(titleActor.getY() - upGroup.getY());
        upGroup.addActor(titleActor);

        back = root.findActor("back", Touchable.enabled);
        back.setY(back.getY() - upGroup.getY());
        oldBackX = back.getX();
        backClick = new Actor() {
            @Override
            public void setScale(float scaleX, float scaleY) {
                super.setScale(scaleX, scaleY);
                back.setScale(scaleX, scaleY);
            }
        };
        upGroup.addActor(back);
        backClick.setSize(back.getWidth() + 30, back.getHeight() + 30);
        upGroup.addActor(backClick);


        middleBG = root.findActor("tuceng_2");
        startNewButton = root.findActor("button_play", Touchable.enabled);

        middleGroup = new Group();
        middleGroup.setSize(720, 315);
        middleGroup.setPosition(360, 459, Align.center);

        scene.addActor(middleGroup);
        modeClicks = new Actor[]{
                scene.findActor("drawC"),
                scene.findActor("blockC"),
                scene.findActor("allfiveC")
        };

        modeNoClick = new Actor[]{
                scene.findActor("drawN"),
                scene.findActor("blockN"),
                scene.findActor("allfiveN")
        };

        modeLabel = new Actor[]{
                scene.findActor("drawL"),
                scene.findActor("blockL"),
                scene.findActor("allfiveL")
        };

        modeClickScopes = new Actor[modeClicks.length];
        for (int i = 0; i < modeClickScopes.length; ++i) {
            modeClickScopes[i] = new Actor();
            modeClickScopes[i].setSize(modeNoClick[i].getWidth() - 5, modeNoClick[i].getHeight() + 10);
            modeClickScopes[i].setPosition(modeNoClick[i].getX(Align.center), modeClicks[i].getY(Align.center), Align.center);
            modeClickScopes[i].setTouchable(Touchable.enabled);

            modeClickScopes[i].setY(modeClickScopes[i].getY() - middleGroup.getY());
            modeClicks[i].setY(modeClicks[i].getY() - middleGroup.getY());
            modeNoClick[i].setY(modeNoClick[i].getY() - middleGroup.getY());
            modeLabel[i].setY(modeLabel[i].getY() - middleGroup.getY());

            middleGroup.addActor(modeClickScopes[i]);
            middleGroup.addActor(modeClicks[i]);
            middleGroup.addActor(modeNoClick[i]);
            middleGroup.addActor(modeLabel[i]);
        }

        scoreClicks = new Actor[]{
                scene.findActor("50C"),
                scene.findActor("100C"),
                scene.findActor("150C")
        };

        scoreNoClicks = new Actor[]{
                scene.findActor("50N"),
                scene.findActor("100N"),
                scene.findActor("150N")
        };

        scoreLabel = new Actor[]{
                scene.findActor("50L"),
                scene.findActor("100L"),
                scene.findActor("150L")
        };

        scoreClickScopes = new Actor[scoreNoClicks.length];
        for (int i = 0; i < scoreClickScopes.length; ++i) {
            scoreClickScopes[i] = new Actor();
            scoreClickScopes[i].setSize(scoreNoClicks[i].getWidth() - 5, scoreNoClicks[i].getHeight() + 10);
            scoreClickScopes[i].setPosition(scoreNoClicks[i].getX(Align.center), scoreNoClicks[i].getY(Align.center), Align.center);
            scoreClickScopes[i].setTouchable(Touchable.enabled);

            scoreClickScopes[i].setY(scoreClickScopes[i].getY() - middleGroup.getY());
            scoreClicks[i].setY(scoreClicks[i].getY() - middleGroup.getY());
            scoreNoClicks[i].setY(scoreNoClicks[i].getY() - middleGroup.getY());
            scoreLabel[i].setY(scoreLabel[i].getY() - middleGroup.getY());

            middleGroup.addActor(scoreClickScopes[i]);
            middleGroup.addActor(scoreClicks[i]);
            middleGroup.addActor(scoreNoClicks[i]);
            middleGroup.addActor(scoreLabel[i]);
        }

        difficultClicks = new Actor[]{
                scene.findActor("easyC"),
                scene.findActor("normalC"),
                scene.findActor("hardC")
        };

        difficultNoClicks = new Actor[]{
                scene.findActor("easyN"),
                scene.findActor("normalN"),
                scene.findActor("hardN")
        };

        difficultLabel = new Actor[]{
                scene.findActor("easyL"),
                scene.findActor("normalL"),
                scene.findActor("hardL")
        };

        difficultClickScopes = new Actor[difficultNoClicks.length];
        for (int i = 0; i < difficultClickScopes.length; ++i) {
            difficultClickScopes[i] = new Actor();
            difficultClickScopes[i].setSize(difficultNoClicks[i].getWidth() - 5, difficultNoClicks[i].getHeight() + 10);
            difficultClickScopes[i].setPosition(difficultNoClicks[i].getX(Align.center), difficultNoClicks[i].getY(Align.center), Align.center);
            difficultClickScopes[i].setTouchable(Touchable.enabled);

            difficultClickScopes[i].setY(difficultClickScopes[i].getY() - middleGroup.getY());
            difficultClicks[i].setY(difficultClicks[i].getY() - middleGroup.getY());
            difficultNoClicks[i].setY(difficultNoClicks[i].getY() - middleGroup.getY());
            difficultLabel[i].setY(difficultLabel[i].getY() - middleGroup.getY());

            middleGroup.addActor(difficultClickScopes[i]);
            middleGroup.addActor(difficultClicks[i]);
            middleGroup.addActor(difficultNoClicks[i]);
            middleGroup.addActor(difficultLabel[i]);
        }

        Actor modeLabel = root.findActor("BitmapFontLabel_1");
        modeLabel.setY(modeLabel.getY() - middleGroup.getY());
        middleGroup.addActor(modeLabel);

        Actor scoresLabel = root.findActor("BitmapFontLabel_1_0");
        scoresLabel.setY(scoresLabel.getY() - middleGroup.getY());
        middleGroup.addActor(scoresLabel);

        Actor difficultLabel = root.findActor("BitmapFontLabel_1_0_0");
        difficultLabel.setY(difficultLabel.getY() - middleGroup.getY());
        middleGroup.addActor(difficultLabel);

        Actor shu1 = root.findActor("xingzhuang_575_kaobei_0");
        shu1.setY(shu1.getY() - middleGroup.getY());
        middleGroup.addActor(shu1);

        Actor shu2 = root.findActor("xingzhuang_575_kaobei_0_0");
        shu2.setY(shu2.getY() - middleGroup.getY());
        middleGroup.addActor(shu2);

        for (int i = 0; i < name.length; i++) {
            info[i] = new Image(Assets.dominoes.findRegion(name[i]));
            root.addActor(info[i]);
        }

        init();
        initButtons();
        resize((int) Configuration.width, (int) Configuration.height);
        root.setTouchable(Touchable.enabled);
        isShow = false;
    }

    private void clickButton(Actor[] clicks, Actor[] noClicks, Actor[] labels, int clickid) {
        for (int k = 0; k < 3; ++k) {
            if (k == clickid) {
                clicks[k].setVisible(true);
                noClicks[k].setVisible(false);
                labels[k].setColor(158 / 255f, 86 / 255f, 15 / 255f, 1);
            } else {
                clicks[k].setVisible(false);
                noClicks[k].setVisible(true);
                labels[k].setColor(171 / 255f, 201 / 255f, 1, 1);
            }
        }
    }

    private void init() {
        for (int i = 0; i < 3; ++i) {
            modeClickScopes[i].clearListeners();
            final int finalI = i;
            modeClickScopes[i].addListener(new ClickListener() {
                @Override
                public void clicked(InputEvent event, float x, float y) {

                    if (finalI == 0) {
                        GameConfig.setGameType(GameConfig.DRAW_TYPE);
                        setGameTypeHelp(finalI);
                    } else if (finalI == 1) {
                        GameConfig.setGameType(GameConfig.BLOCK_TYPE);
                        setGameTypeHelp(finalI);
                    } else {
                        GameConfig.setGameType(GameConfig.ALL_FIVES_TYPE);
                        setGameTypeHelp(finalI);
                    }

                    clickButton(modeClicks, modeNoClick, modeLabel, finalI);
                }
            });

            scoreClickScopes[i].clearListeners();
            scoreClickScopes[i].addListener(new ClickListener() {
                @Override
                public void clicked(InputEvent event, float x, float y) {
                    GameConfig.setGameSocre(finalI);
                    clickButton(scoreClicks, scoreNoClicks, scoreLabel, finalI);
                }
            });

            difficultClickScopes[i].clearListeners();
            difficultClickScopes[i].addListener(new ClickListener() {
                @Override
                public void clicked(InputEvent event, float x, float y) {
                    GameConfig.setGameHard(finalI);
                    clickButton(difficultClicks, difficultNoClicks, difficultLabel, finalI);
                }
            });
        }
    }

    private void initButtons() {
        backClick.clearListeners();
        backClick.addListener(new ButtonListener2() {
            @Override
            public void clicked(InputEvent event, float x, float y) {
                super.clicked(event, x, y);
                close();
            }
        });

        startNewButton.clearListeners();
        startNewButton.addListener(new ButtonListener2() {
            @Override
            public void clicked(InputEvent event, float x, float y) {
                super.clicked(event, x, y);
                GameConfig.loadData = false;
                FlurryManager.flurryLog_Click(GameConfig.gameType + 1, 2);
                FlurryManager.flurryLog_Clicknew();
                FlurryManager.flurryLog_ClickStartGametype(GameConfig.gameType + 1);
                FlurryManager.flurryLog_Step(5);

                listener.startGame();
                Timer.instance().scheduleTask(new Timer.Task() {
                    @Override
                    public void run() {
                        close();
                    }
                }, 1.1f);

                //////////////////////////// DNA ////////////////////////////////
                DeltaDNAManager.addUiInteraction("menuPage", "button", "startNew", FileUtil.menupageStartNew);
                /////////////////////////////////////////////////////////////////
            }
        });
    }

    public void close() {
        changeX = -1;
        addAction(Actions.sequence(
                Actions.moveTo(changeX * Configuration.width, changeY * Configuration.height, 0.3f),
                Actions.run(new Runnable() {
                    @Override
                    public void run() {
                        setVisible(false);
                        isShow = false;
                    }
                })
        ));

    }

    public void show() {
        int type = FileUtil.getGameType();
        int score = FileUtil.getGameSocre();
        int hard = FileUtil.getGameHard();

        GameConfig.setGameType(type);
        GameConfig.setGameSocre(score);
        GameConfig.setGameHard(hard);

        clickButton(modeClicks, modeNoClick, modeLabel, type);
        clickButton(scoreClicks, scoreNoClicks, scoreLabel, score);
        clickButton(difficultClicks, difficultNoClicks, difficultLabel, hard);
        setGameTypeHelp(type);

        show(type);
    }

    public void show(int gameType) {
        DominoGame.isBoneyard = false;
        setVisible(true);
        isShow = true;

        changeX = 1;
        setX(0);

        addAction(Actions.sequence(
                Actions.moveTo(changeX * Configuration.width, changeY * Configuration.height, 0.01f),
                Actions.fadeIn(0.01f),
                Actions.moveTo(0, 0, 0.3f),
                Actions.run(new Runnable() {
                    @Override
                    public void run() {
                        addTutorHelp();
                    }
                })
        ));
    }

    public void setGameTypeHelp(int id) {
        for (int i = 0; i < name.length; i++) {
            if (id == i) {
                info[i].setVisible(true);
            } else {
                info[i].setVisible(false);
            }
        }
    }

    @Override
    public void resize(int width, int height) {
        //长屏手机

        float offsetH = Configuration.heightOffset / 2f;
        float offsetW = Configuration.widthOffset / 2f;
        upGroup.setY(oldUpGroupY + offsetH);

        float middleH = height / 1280f * 492;
        if (middleH < 492) {
            middleH = 492;
        }
        middleBG.setHeight(middleH);
        middleBG.setY(upGroup.getY(), Align.top);

        startNewButton.setPosition(360, Configuration.height / 1280f * 158 - offsetH, Align.bottom);
        middleGroup.setY((middleBG.getY() - startNewButton.getY(Align.top)) / 2f + startNewButton.getY(Align.top), Align.center);

        topBg.setWidth(Configuration.width);
        topBg.setX(360, Align.center);

        middleBG.setWidth(Configuration.width);
        middleBG.setX(360, Align.center);

        back.setX(oldBackX - offsetW);
        backClick.setPosition(back.getX(Align.center), back.getY(Align.center), Align.center);

        screenBG.setSize(Configuration.width, Configuration.height);
        screenBG.setPosition(-offsetW, -offsetH);

        for (int i = 0; i < name.length; ++i) {
            info[i].setPosition(360, middleBG.getY(Align.center), Align.center);
            if (i == name.length - 1) {
                info[i].moveBy(0, -4.5f);
            }
        }
    }

    public interface MainHelpListener {
        void startGame();
    }

    public boolean isShow() {
        return isShow;
    }

    public void addTutorHelp() {
        if (FileUtil.getIsTutorFinished(3)) {
            addTutorStart();
            return;
        }
//        tutorManager = new TutorManager(getStage(), 3);
        info[0].addListener(new ClickListener() {
            @Override
            public void clicked(InputEvent event, float x, float y) {
                super.clicked(event, x, y);
//                tutorManager.hide();

                addTutorStart();

            }
        });

//        tutorManager.bg.addListener(new ClickListener() {
//            @Override
//            public void clicked(InputEvent event, float x, float y) {
//                super.clicked(event, x, y);
//                tutorManager.hide();
//
//                addTutorStart();
//            }
//        });
    }

    public void addTutorStart() {
        if (FileUtil.getIsTutorFinished(4)) {
            return;
        }
//        tutorManager = new TutorManager(getStage(), 4);
//        Vector2 pos = tutorManager.showActor(root.findActor("btn_new"));
//
//        tutorManager.addoldMan(600, pos.y + 400);
//        tutorManager.addWord(100, pos.y + 400, "Let's start!");
//        tutorManager.addHand(pos.x + 300, pos.y - 130);

    }
}
