package com.mobilegame.dominoes.screens;

import com.badlogic.gdx.*;
import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.Group;
import com.badlogic.gdx.scenes.scene2d.Stage;
import com.badlogic.gdx.scenes.scene2d.Touchable;
import com.badlogic.gdx.scenes.scene2d.actions.Actions;
import com.badlogic.gdx.scenes.scene2d.ui.Label;
import com.badlogic.gdx.utils.Align;
import com.badlogic.gdx.utils.Timer;
import com.mobilegame.dominoes.DominoGame;
import com.mobilegame.dominoes.assets.Assets;
import com.mobilegame.dominoes.assets.Configuration;
import com.mobilegame.dominoes.audio.AudioManager;
import com.mobilegame.dominoes.data.FileUtil;
import com.mobilegame.dominoes.dialogs.BaseDialog;
import com.mobilegame.dominoes.dialogs.DialogAdapter;
import com.mobilegame.dominoes.dialogs.DialogListener;
import com.mobilegame.dominoes.flurry.FlurryManager;
import com.mobilegame.dominoes.handles.GameConfig;
import com.mobilegame.dominoes.utils.LayerUtils;
import com.mobilegame.dominoes.utils.Log;

import org.freyja.libgdx.cocostudio.ui.parser.group.CCScene;

import java.util.Iterator;
import java.util.Stack;

/**
 * Created by lilexiang on 17-8-15.
 */
public class BaseScreen implements Screen {
    protected DominoGame game;
    protected Stage stage;
    protected Actor layer;
    protected Stack<BaseDialog> dialogs = new Stack<>();
    protected String TAG = "";
    protected Group dialogsGroup;
    protected boolean screenPaused;
    protected ScreenState state;
    protected Class pendingScreen;
    float stateTime;
    protected float outTime = 0.3F;
    protected float inTime = 0.3F;

    protected boolean isCanTouch = true;

    public BaseScreen(DominoGame game) {
        this.game = game;
        stage = new Stage(game.getViewport(), DominoGame.getBatch()) {
            @Override
            public boolean touchDown(int screenX, int screenY, int pointer, int button) {
                return isCanTouch && super.touchDown(screenX, screenY, pointer, button);
            }
        };
        stage.getRoot().setPosition(Configuration.widthOffset / 2f, Configuration.heightOffset / 2f);

        dialogsGroup = new Group();
        dialogsGroup.setTouchable(Touchable.childrenOnly);
        stage.addActor(dialogsGroup);
        layer = LayerUtils.createLayer();
    }

    @Override
    public void show() {
        AudioManager.playMusic();
        state = ScreenState.show;
        stateTime = 0;
        if (!FileUtil.isnewPlayer()) {
            showBanner();
        } else {
            closeBanner();
        }
//        inAni();
    }

    protected void start() {
        if (state == ScreenState.start) return;
        state = ScreenState.start;
        DominoGame.addInputProcessor(stage);
        DominoGame.resumeInputProcessor();
        initDebug();
        if (layer != null) {
            layer.remove();
        }
    }

    private void initDebug() {
        InputAdapter keyAdapter = new InputAdapter() {
            @Override
            public boolean keyUp(int keycode) {
                if (keycode == Input.Keys.ESCAPE || keycode == Input.Keys.BACK) {
                    back();
                }
                if (Configuration.debug || Gdx.app.getType() == Application.ApplicationType.Desktop) {
                    if (keycode == Input.Keys.R) {
                        Configuration.resetData();
                    }
                }
                return super.keyDown(keycode);
            }
        };
        DominoGame.addInputProcessor(keyAdapter);
    }


    public void update() {
        Log.log("BaseScreen()", "update() --->" + getClass());
        Iterator<BaseDialog> iterator = dialogs.iterator();
        while (iterator.hasNext()) {
            iterator.next().update();
        }
    }

    @Override
    public void dispose() {
        if (stage != null) {
            stage.dispose();
            stage = null;
        }
    }

    int logTime = 0;
    int checkBannerTime;


    @Override
    public void render(float delta) {
        if (stage != null) {
            stage.act();
        }
        if (stage != null) {
            stage.draw();
        }


        if (!screenPaused)
            stateTime += delta;
        logTime++;
        if (logTime > 10 * 60) {
            //           Gdx.app.log("BaseScreen", "render()----> stateTime = " + stateTime + "  state = " + state + "  assetManager update " + Assets.instance.assetManager.update());
            logTime = 0;
        }
        if (state == ScreenState.end && Assets.instance.assetManager.update(10) && stateTime > outTime + 0.02F) {
            state = ScreenState.out;
            out();
        }

        if (state == ScreenState.show && stateTime > inTime + 0.02F) {
            start();
        }
        checkBannerTime++;


        if (!FileUtil.isnewPlayer()) {
            DominoGame.AdTime += delta;
            if (DominoGame.AdTime > 60) {
                DominoGame.isAd = true;
                DominoGame.AdTime = 0;
            }
        }

        DominoGame.playTime += delta;
        if (DominoGame.playTime > 60) {
            FileUtil.putTime();
            DominoGame.playTime = 0;
            if (FileUtil.getTime() > 3) {
                FileUtil.moreThan15Minutes();
            }
        }

        DominoGame.rateTime += delta;
        if (DominoGame.rateTime > 60) {
            FileUtil.putRateTime();

            DominoGame.rateTime = 0;
            if (FileUtil.getRateTime() >= 12) {
                DominoGame.isRate = true;
            }
        }

        DominoGame.deltaTime += delta;
        GameConfig.firstTimeTotal += delta;

        int numberOfSteps = 60;
        if (FileUtil.isFirstLogin) {

            if (GameConfig.firstTimeTotal > numberOfSteps) {
                GameConfig.firstFLurryTotal++;
                GameConfig.firstTimeTotal = 0;
                FlurryManager.flurryLog_PlayTime_firstBase(GameConfig.firstFLurryTotal * numberOfSteps);
            }

        }
    }

    public <T extends BaseDialog> T showDialog(Class<? extends BaseDialog> tClass, String name) {
        return showDialog(tClass, name, null);
    }


    public <T extends BaseDialog> T showDialog(Class<? extends BaseDialog> tClass, String name, DialogListener listener) {
        try {
            T dialog = (T) (tClass.getConstructor(String.class).newInstance(name));
            dialog.addListener(dialogListener);
            if (listener != null) {
                dialog.addListener(listener);
            }
            dialogs.push(dialog);
            if (name.contains("design")) {
                FlurryManager.flurryLog_design_view();
                dialog.setGroupBottom(dialogsGroup);
            } else {
                dialog.setGroup(dialogsGroup);
            }
            dialogsGroup.setZIndex(Integer.MAX_VALUE);
            return dialog;
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }


    protected void removeDialog() {
        if (!dialogs.empty()) {
            dialogs.pop();
            update();
        }
    }

    protected boolean back() {
        if (dialogs.size() > 0) {
            BaseDialog dialog = dialogs.peek();
            if (dialog != null) {
                dialog.close();
                return false;
            }
        }
        if (DominoGame.isAction) {
            return false;
        }
        return true;
    }


    @Override
    public void resize(int width, int height) {
        stage.getRoot().setPosition(Configuration.widthOffset / 2f, Configuration.heightOffset / 2f);
        dialogsGroup.setSize(Configuration.width, Configuration.height);
        dialogsGroup.setPosition(0, 0);
        Iterator<BaseDialog> iterator = dialogs.iterator();
        while (iterator.hasNext()) {
            BaseDialog b = iterator.next();
            b.layout();
            b.resize(width, height);
        }
        layer.setPosition(0, 0);
        layer.setSize(Configuration.width, Configuration.height);
    }

    @Override
    public void hide() {

    }

    @Override
    public void pause() {
        screenPaused = true;
        AudioManager.pauseMusic();
    }


    @Override
    public void resume() {
        Assets.instance.loadFinish();
        update();
        AudioManager.resumeMusic();

        screenPaused = false;
    }

    protected void out() {
        Log.log("BaseScreen", "out() ---> " + pendingScreen);
//        if (pendingScreen == MenuScreen.class) {
//            game.gotoMenuScreen();
//        }
        dispose();
    }

    private void loadResource() {
    }

    public void end(Class screen) {
        if (state == ScreenState.end) return;
        if (screen == null) return;
        Log.log("BaseScreen", "end()----> " + screen);
        stateTime = 0;
        closeBanner();
        stateTime = 0;
        pendingScreen = screen;
        state = ScreenState.end;
        DominoGame.clearInputProcessor();
        loadResource();
        endAni();
    }

    protected void inAni() {
        layer.remove();
        layer.setColor(0, 0, 0, 1);
        stage.addActor(layer);
        layer.getActions().clear();
        layer.addAction(Actions.sequence(Actions.fadeOut(outTime)));
    }

    protected void endAni() {
        layer.remove();
        layer.setColor(0, 0, 0, 0);
        stage.addActor(layer);
        layer.getActions().clear();
        layer.addAction(Actions.sequence(Actions.fadeIn(outTime)));
    }


    public Group parseScene(String name) {
        return CCScene.parse(game.getCocoStudioUIEditor(), name);
    }

    private Group msgGroup;


    public void showMessage(String message) {
        showMessage(message, 100);
    }

    public void showMessage(String message, float height) {
        if (msgGroup == null) {
            msgGroup = parseScene("ui/objects/tip.json");
            msgGroup.setName("tip");
        }
        Label label = msgGroup.findActor("content_label");
        if (msgGroup.getParent() != null) {
            msgGroup.setColor(1, 1, 1, 1);
            msgGroup.getActions().clear();
        } else {
            //    msgGroup.addAction(Actions.fadeIn(0.1F));
        }
        msgGroup.setColor(1, 1, 1, 1);
        label.setText(message);
        label.layout();
        float textWidth = label.getGlyphLayout().width;
        Actor bg = msgGroup.findActor("bg");
        bg.setWidth(textWidth + 60);
        bg.setPosition(msgGroup.getWidth() / 2, msgGroup.getHeight() / 2, Align.center);
        stage.addActor(msgGroup);
        msgGroup.setPosition(Configuration.width / 2, height, Align.center);
        msgGroup.addAction(Actions.sequence(Actions.delay(1.5F), Actions.fadeOut(0.3F), Actions.removeActor()));
    }

    public <T extends Actor> T findActor(String name) {
        return findActor(stage.getRoot(), name);
    }

    public <T extends Actor> T findActor(Group parent, String name) {
        return parent.findActor(name);
    }

    public enum ScreenState {
        show, start, end, dark, out
    }


    protected DialogListener dialogListener = new DialogAdapter() {
        @Override
        public void closed() {
            removeDialog();
        }

        @Override
        public void update(boolean updated) {
            BaseScreen.this.update();
        }

        @Override
        public void showDialog(Class<? extends BaseDialog> tClass) {
            super.showDialog(tClass);
            //   BaseScreen.this.showDialog(tClass);
        }
    };

    public void showAds() {

        if (DominoGame.isAd) {
            DominoGame.launcherListener.showFullScreenAd();
            if (DominoGame.launcherListener.isInsReady()) {
                DominoGame.isAd = false;
                DominoGame.showIns = true;
            }

        }

    }

    public void showAction() {
        DominoGame.isAction = true;

//        stage.addActor(Actionlayer);
//        if (mySpineActor != null) {
//            stage.addActor(mySpineActor);
//            mySpineActor.setAnimation("1", true);
//            mySpineActor.setVisible(true);
//        }
//        Actionlayer.setVisible(true);
        Timer.instance().scheduleTask(new Timer.Task() {
            @Override
            public void run() {
                showAds();
            }
        }, 0.5f);

//        Timer.instance().scheduleTask(new Timer.Task() {
//            @Override
//            public void run() {
//                DominoGame.isAction = false;
//                if (mySpineActor != null) {
//                    mySpineActor.setVisible(false);
//                }
//                Actionlayer.setVisible(false);
//            }
//        }, 0.9f);

    }

    public void showBanner() {
        DominoGame.launcherListener.showBanner(true);
    }

    public void closeBanner() {
        DominoGame.launcherListener.showBanner(false);
    }

}
