package com.mobilegame.dominoes.screens;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.GL20;
import com.badlogic.gdx.graphics.g2d.TextureAtlas;
import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.actions.Actions;
import com.badlogic.gdx.scenes.scene2d.actions.SequenceAction;
import com.badlogic.gdx.scenes.scene2d.ui.Image;
import com.badlogic.gdx.utils.Align;
import com.mobilegame.dominoes.DominoGame;
import com.mobilegame.dominoes.assets.Assets;
import com.mobilegame.dominoes.assets.Configuration;
import com.mobilegame.dominoes.data.Classes.Design;
import com.mobilegame.dominoes.data.DiamondDataUtil;
import com.mobilegame.dominoes.data.GameDataCsv;
import com.mobilegame.dominoes.dialogs.DesignDialogNew;
import com.mobilegame.dominoes.flurry.FlurryManager;
import com.mobilegame.dominoes.utils.LeagueUtil;


/**
 * Created by Doodle on 2021/1/5.
 */

public class LoadingScreenNew extends BaseScreen {
    private Actor titleActor;
    private Actor[] dianbgs;

    public LoadingScreenNew(DominoGame game) {
        super(game);
        Assets.instance.assetManager.load("image/loadScreen.atlas", TextureAtlas.class);
        Assets.instance.assetManager.finishLoading();

        if (Assets.newUI != null) {
            TextureRegion bgRegion = Assets.newUI.findRegion("bg");
            Actor bg = new Image(bgRegion);
            bg.setSize(Configuration.width, Configuration.height);
            bg.moveBy(-Configuration.widthOffset / 2f, -Configuration.heightOffset / 2f);
            stage.addActor(bg);

            titleActor = new Image(Assets.newUI.findRegion("title"));
            titleActor.setPosition(360, 684, Align.center);
            stage.addActor(titleActor);
        }

        TextureAtlas loadScreenAtlas = Assets.instance.assetManager.get("image/loadScreen.atlas", TextureAtlas.class);
        loadAnimation(loadScreenAtlas);
    }

    private void loadAnimation(TextureAtlas textureAtlas) {
        float x = 234, y = titleActor.getY() - 100;

        TextureRegion dianbgRegion = textureAtlas.findRegion("dianbg");
        TextureRegion baidianRegion = textureAtlas.findRegion("dian");
        dianbgs = new Actor[6];
        Actor baidianActor = new Image(baidianRegion);
        baidianActor.setPosition(x, y);

        for (int i = 0, j = dianbgs.length; i < j; ++i) {
            dianbgs[i] = new Image(dianbgRegion);
            dianbgs[i].setPosition(x, y);
            x += 44;
            stage.addActor(dianbgs[i]);
        }

        stage.addActor(baidianActor);
        baidianActor.clearActions();
        baidianActor.addAction(
                Actions.forever(
                        getSequenceAction(dianbgs)
                )
        );
    }

    private SequenceAction getSequenceAction(Actor[] dianbgs) {
        SequenceAction sequenceAction = Actions.sequence();
        for (int i = 0; i < dianbgs.length; ++i) {
            sequenceAction.addAction(
                    Actions.moveTo(dianbgs[i].getX(), dianbgs[i].getY())
            );

            sequenceAction.addAction(
                    Actions.delay(0.3333f)
            );
        }

        return sequenceAction;
    }

    private int step = 4;
    private float time = 0;
    private int progress = 0;
    private int next = 0;

    private void process(float delta) {
        float reality_p;
        time += delta;

        reality_p = Assets.instance.assetManager.getProgress() * 100f;

        if (progress <= 10)
            progress += step;
        else if (progress <= 50 && progress <= reality_p) {
            progress += step;
        } else if (progress <= 70 && progress <= reality_p) {
            progress += step;
        } else if (progress <= 100 && progress <= reality_p) {
            progress += step;
        } else if (reality_p == 100f) {
            progress += step;
        }

        if (progress >= 100) {
            progress = 100;
            next += 1;
        }

        if (time > 4.5 && Assets.instance.assetManager.update() && next >= 2) {
            loadingOver();
        }

    }

    private void initGameResource() {

    }

    public void loadingOver() {

        game.CheckTime();
        LeagueUtil.init();
        game.loadingOver();
        FlurryManager.flurryLog_Step(3);
//        DiamondDataUtil.addDiamondNum(100);
    }

    @Override
    public void render(float delta) {
        // TODO Auto-generated method stub
        Gdx.gl.glClearColor(1, 1, 1, 1);
        Gdx.gl.glClear(GL20.GL_COLOR_BUFFER_BIT | GL20.GL_DEPTH_BUFFER_BIT);
        process(delta);


        stage.act();
        stage.draw();
    }

    @Override
    public void show() {
        initGameResource();
        time = 0;
        Assets.instance.loadSync();
        Assets.instance.loadFinish();

        int dominoesId = DiamondDataUtil.getUseDesign(DesignDialogNew.DesignType.dominoes);
        Design dominoes = GameDataCsv.getDesign(dominoesId);
        if (dominoes != null) {
            DominoGame.dominoeName = dominoes.Topic_name + "-";
        }

        int bgId = DiamondDataUtil.getUseDesign(DesignDialogNew.DesignType.background);
        if (bgId != 0) {
            DominoGame.bgNum = bgId;
        }

        FlurryManager.flurryLog_Step(2);
    }

    @Override
    public void hide() {
        // TODO Auto-generated method stub

    }

    @Override
    public void pause() {
        // TODO Auto-generated method stub

    }

    @Override
    public void resume() {
        // TODO Auto-generated method stub
//        AudioManager.pauseAllMusic();
    }

    @Override
    public void dispose() {
        // TODO Auto-generated method stub
        if (stage != null)
            stage.dispose();
    }

}
