package com.mobilegame.dominoes.utils;

import com.badlogic.gdx.math.EarClippingTriangulator;
import com.badlogic.gdx.utils.FloatArray;
import com.badlogic.gdx.utils.ShortArray;

import java.awt.geom.Point2D;
import java.util.List;


public class MathUtil {
    private static EarClippingTriangulator earClippingTriangulator = new EarClippingTriangulator();


    public static double getAngle(float x1, float y1, float x2, float y2) {
        float length = (float) Math.sqrt((x2 - x1) * (x2 - x1) + (y2 - y1) * (y2 - y1));
        double angle = Math.asin((y2 - y1) / (length + Float.MIN_VALUE));
        if (x2 - x1 >= 0) return angle;
        else if (y2 - y1 >= 0) return Math.PI - angle;
        else return Math.PI - angle;
    }

    public static FloatArray cubicSplineInterpolation(FloatArray vertex) {
        FloatArray result = new FloatArray();
        int size = vertex.size;
        if (size >= 8) {
            result.add(vertex.get(0), vertex.get(1));
            for (int i = 0; i < size; i += 2) {
                if (i < size - 8) {
                    for (float t = 0; t < 1; t += 0.1F) {
                        float a1 = (float) Math.pow((1 - t), 3) / 6;
                        float a2 = (float) ((3 * Math.pow(t, 3) - 6 * Math.pow(t, 2) + 4) / 6);
                        float a3 = (float) ((-3 * Math.pow(t, 3) + 3 * Math.pow(t, 2) + 3 * t + 1) / 6);
                        float a4 = (float) (Math.pow(t, 3) / 6);
                        float x = a1 * vertex.get(i) + a2 * vertex.get(i + 2) + a3 * vertex.get(i + 4) + a4 * vertex.get(i + 6);
                        float y = a1 * vertex.get(i + 1) + a2 * vertex.get(i + 3) + a3 * vertex.get(i + 5) + a4 * vertex.get(i + 7);
                        result.add(x, y);
                    }
                }
            }
            result.add(vertex.get(size - 2), vertex.get(size - 1));
        } else {
            for (int i = 0; i < size; i += 2) {
                result.add(vertex.get(i), vertex.get(i + 1));
            }
        }
        return result;
    }

    public static float[] merge(float[] x, float[] y) {
        float[] v = new float[x.length + y.length];
        for (int i = 0, k = 0; i < x.length; i++) {
            v[k++] = x[i];
            v[k++] = y[i];
        }
        return v;
    }

    public static boolean isOnSameLine(float x1, float y1, float x2, float y2, float x3, float y3) {
        if (x2 - x1 == 0 && x3 - x1 == 0) return true;
        else if (x2 - x1 == 0 || x3 - x1 == 0) return false;
        //       return (y2 - y1) / (x2 - x1) == (y3 - y1) / (x3 - x1);
        return Math.abs((y2 - y1) / (x2 - x1) - (y3 - y1) / (x3 - x1)) < 0.002F;
    }

    public static ShortArray computeTriangles(float[] vertices) {
        return earClippingTriangulator.computeTriangles(vertices);
    }


    public static boolean checkOverlapping(float[] polygon1, float[] polygon2) {
        if (polygon1.length < 6 || polygon2.length < 6) {
            return false;
        }
        if (polygon1.length % 2 != 0 || polygon2.length % 2 != 0) {
            return false;
        }
        for (int i = 0; i < polygon1.length; i = i + 2) {
            for (int j = 0; j < polygon2.length; j = j + 2) {
                if ((polygon1[i] == polygon2[j] && polygon1[i + 1] == polygon2[j + 1]&&!(polygon1[(i + 2) % polygon1.length] == polygon2[(j + 2) % polygon2.length] && polygon1[(i + 3) % polygon1.length] == polygon2[(j + 3) % polygon2.length]))
                        || (polygon1[i] == polygon2[(j + 2) % polygon2.length] && polygon1[i + 1] == polygon2[(j + 3) % polygon2.length]&&!(polygon1[(i + 2) % polygon1.length] == polygon2[j] && polygon1[(i + 3) % polygon1.length] == polygon2[j + 1]))
                        || (polygon1[(i + 2) % polygon1.length] == polygon2[j] && polygon1[(i + 3) % polygon1.length] == polygon2[j + 1]&&!(polygon1[i] == polygon2[(j + 2) % polygon2.length] && polygon1[i + 1] == polygon2[(j + 3) % polygon2.length]))
                        || (polygon1[(i + 2) % polygon1.length] == polygon2[(j + 2) % polygon2.length] && polygon1[(i + 3) % polygon1.length] == polygon2[(j + 3) % polygon2.length]&&!(polygon1[i] == polygon2[j] && polygon1[i + 1] == polygon2[j + 1]))) {
                    continue;
                }
                if (linesIntersect(polygon1[i], polygon1[i + 1], polygon1[(i + 2) % polygon1.length], polygon1[(i + 3) % polygon1.length],
                        polygon2[j], polygon2[j + 1], polygon2[(j + 2) % polygon2.length], polygon2[(j + 3) % polygon2.length])) {
                    return true;
                }
            }
        }
        return false;
    }

    public static boolean linesIntersect(double x1, double y1,
                                         double x2, double y2,
                                         double x3, double y3,
                                         double x4, double y4) {
        return ((relativeCCW(x1, y1, x2, y2, x3, y3) *
                relativeCCW(x1, y1, x2, y2, x4, y4) <= 0)
                && (relativeCCW(x3, y3, x4, y4, x1, y1) *
                relativeCCW(x3, y3, x4, y4, x2, y2) <= 0));
    }

    public static int relativeCCW(double x1, double y1,
                                  double x2, double y2,
                                  double px, double py) {
        x2 -= x1;
        y2 -= y1;
        px -= x1;
        py -= y1;
        double ccw = px * y2 - py * x2;
        if (ccw == 0.0) {
            ccw = px * x2 + py * y2;
            if (ccw > 0.0) {
                px -= x2;
                py -= y2;
                ccw = px * x2 + py * y2;
                if (ccw < 0.0) {
                    ccw = 0.0;
                }
            }
        }
        return (ccw < 0.0) ? -1 : ((ccw > 0.0) ? 1 : 0);
    }

    /**
     * 判断点是否在多边形内
     *
     * @param point 检测点
     * @param pts   多边形的顶点
     * @return 点在多边形内返回true, 否则返回false
     */
    public static boolean IsPtInPoly(Point2D.Double point, List<Point2D.Double> pts) {

        int N = pts.size();
        boolean boundOrVertex = true; //如果点位于多边形的顶点或边上，也算做点在多边形内，直接返回true
        int intersectCount = 0;//cross points count of x
        double precision = 2e-10; //浮点类型计算时候与0比较时候的容差
        Point2D.Double p1, p2;//neighbour bound vertices
        Point2D.Double p = point; //当前点

        p1 = pts.get(0);//left vertex
        for (int i = 1; i <= N; ++i) {//check all rays
            if (p.equals(p1)) {
                return boundOrVertex;//p is an vertex
            }

            p2 = pts.get(i % N);//right vertex
            if (p.x < Math.min(p1.x, p2.x) || p.x > Math.max(p1.x, p2.x)) {//ray is outside of our interests
                p1 = p2;
                continue;//next ray left point
            }

            if (p.x > Math.min(p1.x, p2.x) && p.x < Math.max(p1.x, p2.x)) {//ray is crossing over by the algorithm (common part of)
                if (p.y <= Math.max(p1.y, p2.y)) {//x is before of ray
                    if (p1.x == p2.x && p.y >= Math.min(p1.y, p2.y)) {//overlies on a horizontal ray
                        return boundOrVertex;
                    }

                    if (p1.y == p2.y) {//ray is vertical
                        if (p1.y == p.y) {//overlies on a vertical ray
                            return boundOrVertex;
                        } else {//before ray
                            ++intersectCount;
                        }
                    } else {//cross point on the left side
                        double xinters = (p.x - p1.x) * (p2.y - p1.y) / (p2.x - p1.x) + p1.y;//cross point of y
                        if (Math.abs(p.y - xinters) < precision) {//overlies on a ray
                            return boundOrVertex;
                        }

                        if (p.y < xinters) {//before ray
                            ++intersectCount;
                        }
                    }
                }
            } else {//special case when ray is crossing through the vertex
                if (p.x == p2.x && p.y <= p2.y) {//p crossing over p2
                    Point2D.Double p3 = pts.get((i + 1) % N); //next vertex
                    if (p.x >= Math.min(p1.x, p3.x) && p.x <= Math.max(p1.x, p3.x)) {//p.x lies between p1.x & p3.x
                        ++intersectCount;
                    } else {
                        intersectCount += 2;
                    }
                }
            }
            p1 = p2;//next ray left point
        }

        if (intersectCount % 2 == 0) {//偶数在多边形外
            return false;
        } else { //奇数在多边形内
            return true;
        }

    }
    public static float length(float x1,float y1,float x2,float y2){
        float x = x2-x1;
        float y = y2-y1;
        return (float) Math.sqrt(x*x+y*y);
    }

    private void initInt(int[][] temp, int size, int defaultValue) {
        for (int i = 0; i < temp.length; i++) {
            temp[i] = new int[size];
            for (int j = 0; j < temp[i].length; j++) {
                temp[i][j] = defaultValue;
            }
        }
    }

    private void initFloat(float[] temp, float defaultValue) {
        for (int i = 0; i < temp.length; i++) {
            temp[i] = defaultValue;
        }
    }

//    public static boolean checkOverlapping(float[] polygon1,float[] polygon2){
//        if(polygon1.length<6||polygon2.length<6) {
//            System.out.println("点太少!!!!!");
//            return false;
//        }
//        if(polygon1.length%2!=0||polygon2.length%2!=0) {
//            System.out.println("点不对!!!!!");
//            return false;
//        }
//        polygon.setVertices(polygon1);
//        for (int i = 0;i<polygon2.length/2;i+=2){
//            if(polygon.contains(polygon2[i*2],polygon2[i*2+1])) return true;
//        }
//        return false;
//    }
}
