package com.mobilegame.dominoes.utils;

import com.badlogic.gdx.graphics.g2d.Batch;
import com.badlogic.gdx.graphics.g2d.ParticleEffect;
import com.badlogic.gdx.graphics.g2d.ParticleEmitter;
import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.utils.Disposable;

public class ParticleEffectActor extends Actor implements Disposable {

	public ParticleEffect particleEffect;

	public ParticleEffectActor(ParticleEffect particleEffect) {
		this.particleEffect = particleEffect;

		this.particleEffect.start();
	}

	public ParticleEffectActor(ParticleEffect particleEffect, String main){

		this.particleEffect = particleEffect;

		ParticleEmitter mainEmitter = particleEffect.findEmitter(main);
		float highMax = mainEmitter.getXScale().getHighMax();
		float highMin = mainEmitter.getXScale().getHighMin();
		float width = highMax > highMin ? highMax : highMin;

		highMax = mainEmitter.getYScale().getHighMax();
		highMin = mainEmitter.getYScale().getHighMin();
		float height = highMax > highMin ? highMax : highMin;
		height = height == 0 ? width : height;  // 粒子特效默认行为：height 没设置的话，会默认为 width

		setSize(width, height);

		this.particleEffect.start();

	}

	@Override
	public void act (float delta) {
		try {
			super.act(delta);
			if(!isVisible()) {
                return;
            }
			particleEffect.setPosition(getX() + getWidth() / 2F, getY() + getHeight() / 2F);
			particleEffect.update(delta);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	@Override
	public void draw (Batch batch, float parentAlpha) {
		int blendDstFunc=batch.getBlendDstFunc();
		int blendSrcFunc=batch.getBlendSrcFunc();
		particleEffect.draw(batch);
		batch.setBlendFunction(blendSrcFunc, blendDstFunc);
	}

	@Override
	public void dispose() {
		particleEffect.dispose();
	}
}
