package com.mobilegame.dominoes.utils.downloader;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.Net;
import com.badlogic.gdx.files.FileHandle;
import com.badlogic.gdx.utils.Array;
import com.mobilegame.dominoes.assets.Configuration;
import com.mobilegame.dominoes.utils.Log;
import com.mobilegame.dominoes.utils.net.Net2;
import com.mobilegame.dominoes.utils.net.NetJavaImpl2;
import com.mobilegame.dominoes.utils.net.NetJavaImpl3;

import java.io.*;
import java.util.Enumeration;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;


public class BaseDownloader implements Downloader {
    public Net2 net;

    public void download(final DownLoadInfo info) {
        if (net == null) {
            if (Configuration.APILevel < 12) {
                net = new NetJavaImpl3(1);
            } else {
                net = new NetJavaImpl2(1);
            }
        }
        final Net.HttpRequest request = new Net.HttpRequest(Net.HttpMethods.GET);
        request.setUrl(info.URL);
        final Net.HttpResponseListener listener = new Net.HttpResponseListener() {
            @Override
            public void handleHttpResponse(Net.HttpResponse httpResponse) {
                try {
                    Log.log("BaseDownloader", "file name" + info.path);
                    int size = Integer.MAX_VALUE;
                    String header = httpResponse.getHeader("Content-Length");
                    try {
                        if(header!=null) {
                            size = Integer.parseInt(header);
                        }else{
                            info.listener.failure(info);
                            return;
                        }
                    }catch (Exception e){}
                    InputStream inputStream = httpResponse.getResultAsStream();
                    File file = Gdx.files.local(info.path).file();
                    File dir = new File(file.getParent());
                    if(!dir.exists()) dir.mkdir();
//                    File file = new File(info.path);
                    if (file.exists()) {
                     //   file.delete();
                    }
                    file.createNewFile();
                    RandomAccessFile randomAccessFile = new RandomAccessFile(file, "rwd");
                    randomAccessFile.seek(0);
                    byte[] picBytes = new byte[1024];
                    int readNum = -1;
                    int doneSize = 0;
                    while ((readNum = inputStream.read(picBytes)) != -1) {
                        randomAccessFile.write(picBytes, 0, readNum);
                        doneSize += readNum;
                        Log.log("Download","down size " + doneSize);
                        info.listener.progress(info, doneSize, size);
                    }
                    randomAccessFile.close();
                    inputStream.close();
                    info.listener.success(info);
                } catch (Exception e) {
                    e.printStackTrace();
                    info.listener.failure(info);
                }
            }

            @Override
            public void failed(Throwable t) {
                info.listener.failure(info);
            }

            @Override
            public void cancelled() {
                info.listener.cancel(info);
            }
        };
        net.sendHttpRequest(request, listener);
    }

    @Override
    public void download(Array<DownLoadInfo> infos) {
        for (int i = 0; i < infos.size; i++) {
            download(infos.get(i));
        }
    }

    public void unzip(String zipFileName,String path) {
        try {
            File files = Gdx.files.local(zipFileName).file();
            ZipFile zipFile = new ZipFile(files);
            Enumeration e = zipFile.entries();
            while (e.hasMoreElements()) {
                ZipEntry entry = (ZipEntry) e.nextElement();
                InputStream is = zipFile.getInputStream(entry);
                String entryName = entry.getName();
                FileHandle local = Gdx.files.local(path + entryName);
                File dstFile = local.file();
                if (entryName.endsWith("/")) {
                    boolean mk = dstFile.mkdirs();
                    continue;
                }
                if (!local.parent().exists()) {
                    local.parent().mkdirs();
                }
                System.err.println(dstFile.getAbsolutePath());
                FileOutputStream fos = new FileOutputStream(dstFile);
                byte[] buffer = new byte[8192];
                int count;
                while ((count = is.read(buffer, 0, buffer.length)) != -1) {
                    fos.write(buffer, 0, count);
                }
                fos.close();
                is.close();
            }
            files.delete();

        } catch (IOException e) {
            e.printStackTrace();
        }
    }


}
