/******************************************************************************
 * Spine Runtimes Software License v2.5
 *
 * Copyright (c) 2013-2016, Esoteric Software
 * All rights reserved.
 *
 * You are granted a perpetual, non-exclusive, non-sublicensable, and
 * non-transferable license to use, install, execute, and perform the Spine
 * Runtimes software and derivative works solely for personal or internal
 * use. Without the written permission of Esoteric Software (see Section 2 of
 * the Spine Software License Agreement), you may not (a) modify, translate,
 * adapt, or develop new applications using the Spine Runtimes or otherwise
 * create derivative works or improvements of the Spine Runtimes or (b) remove,
 * delete, alter, or obscure any trademarks or any copyright, trademark, patent,
 * or other intellectual property or proprietary rights notices on or in the
 * Software, including any copy thereof. Redistributions in binary or source
 * form must include this license and terms.
 *
 * THIS SOFTWARE IS PROVIDED BY ESOTERIC SOFTWARE "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL ESOTERIC SOFTWARE BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES, BUSINESS INTERRUPTION, OR LOSS OF
 * USE, DATA, OR PROFITS) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *****************************************************************************/

package com.spine;

import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.g2d.Batch;
import com.badlogic.gdx.graphics.g2d.PolygonSpriteBatch;
import com.badlogic.gdx.math.Rectangle;
import com.badlogic.gdx.math.Vector2;
import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.InputEvent;
import com.badlogic.gdx.scenes.scene2d.utils.ClickListener;
import com.badlogic.gdx.utils.Array;
import com.esotericsoftware.spine.AnimationState;
import com.esotericsoftware.spine.AnimationStateData;
import com.esotericsoftware.spine.Bone;
import com.esotericsoftware.spine.Skeleton;
import com.esotericsoftware.spine.SkeletonData;
import com.esotericsoftware.spine.SkeletonRenderer;
import com.esotericsoftware.spine.Slot;
import com.esotericsoftware.spine.attachments.RegionAttachment;

import java.util.ArrayList;
import java.util.HashMap;

/**
 * A scene2d actor that draws a skeleton.
 */
@SuppressWarnings("rawtypes")
public class MySpineActor extends Actor {
    protected AnimationState state;
    protected boolean clip;
    protected SkeletonRenderer renderer;
    protected Skeleton skeleton;

    /**
     * Creates an uninitialized SkeletonActor. The renderer, skeleton, and animation state must be set before use.
     */

    public MySpineActor(SkeletonRenderer renderer, SkeletonData data) {
        this.renderer = renderer;
        skeleton = new Skeleton(data);
        state = new AnimationState(new AnimationStateData(data));
        this.addListener(xClickListener);
    }

    public MySpineActor(SkeletonRenderer renderer, Skeleton skeleton, AnimationState state) {
        this.renderer = renderer;
        this.skeleton = skeleton;
        this.state = state;
        this.addListener(xClickListener);
    }

    public MySpineActor(SkeletonRenderer renderer2, MySpineStatus mysp1) {
        this.renderer = renderer2;
        this.skeleton = mysp1.skeleton;
        this.state = mysp1.animationState;
        this.addListener(xClickListener);

    }

    public void act(float delta) {
        super.act(delta);
//        float oldscaleX = skeleton.getData().findBone("root").getScaleX();
//        float oldscaleY = skeleton.getData().findBone("root").getScaleY();

//        skeleton.getData().findBone("root").setScale(oldscaleX * getScaleX(), oldscaleY * getScaleY());

        state.update(delta);
        state.apply(skeleton);

        skeleton.setPosition(getX(), getY());
//        skeleton.getRootBone().setRotation(getRotation());
//        skeleton.updateWorldTransform();

//        skeleton.getData().findBone("root").setScale(oldscaleX, oldscaleY);
    }

    private ArrayList<ClickListener> boneClickListeners = new ArrayList<ClickListener>();
    private ArrayList<String> boneClick = new ArrayList<String>();
    private HashMap<String, Vector2> boneTouchScale = new HashMap<String, Vector2>();
    private float[] vertices = new float[8];
    private Rectangle boundBox = new Rectangle();
    private int focusBoneIndex = -1;

    private ClickListener xClickListener = new ClickListener() {
        public void clicked(InputEvent event, float x, float y) {
            if (focusBoneIndex < 0 || focusBoneIndex >= boneClickListeners.size()) {
                return;
            }
            ClickListener listener = boneClickListeners.get(focusBoneIndex);
            if (listener != null) {
                listener.clicked(event, x, y);
            }
        }

        @Override
        public boolean touchDown(InputEvent event, float x, float y, int pointer, int button) {
            if (focusBoneIndex < 0 || focusBoneIndex >= boneClickListeners.size()) {
                return false;
            }
            ClickListener listener = boneClickListeners.get(focusBoneIndex);
            if (listener != null) {
                listener.touchDown(event, x, y, pointer, button);
            }
            return true;
        }

        @Override
        public void touchUp(InputEvent event, float x, float y, int pointer, int button) {
            if (focusBoneIndex < 0 || focusBoneIndex >= boneClickListeners.size()) {
                return;
            }
            ClickListener listener = boneClickListeners.get(focusBoneIndex);
            if (listener != null) {
                listener.touchUp(event, x, y, pointer, button);
            }
        }
    };

    public void addBoneClickListener(String boneName, Vector2 touchScale, ClickListener listener) {
        for (int i = 0; i < boneClick.size(); i++) {
            if (boneClick.get(i).equals(boneName)) {
                return;
            }
        }
        boneTouchScale.put(boneName, touchScale);
        boneClick.add(boneName);
        boneClickListeners.add(listener);
    }

    public void addBoneClickListener(String boneName, ClickListener listener) {
        for (int i = 0; i < boneClick.size(); i++) {
            if (boneClick.get(i).equals(boneName)) {
                return;
            }
        }
        boneClick.add(boneName);
        boneClickListeners.add(listener);
    }

    public void clearBoneClickListeners() {
        boneClick.clear();
        boneClickListeners.clear();
    }

    @Override
    public Actor hit(float x, float y, boolean touchable) {
        Array<Slot> slotsArray = skeleton.getSlots();
        focusBoneIndex = -1;
        for (int i = 0; i < boneClick.size(); i++) {
            String boneName = boneClick.get(i);
            Bone bone = this.skeleton.findBone(boneName);
            RegionAttachment region = null;
            for (int j = 0; j < slotsArray.size; j++) {
                Slot item = slotsArray.get(j);
                if (item.getBone() != bone)
                    continue;
                if (item.getAttachment() != null) {
                    if (item.getAttachment() instanceof RegionAttachment) {
                        region = (RegionAttachment) (item.getAttachment());
                    }
                    break;
                }
            }
            if (region != null) {
                region.computeWorldVertices(bone, vertices, 0, 2);
                getBoundingRectangle(vertices, boundBox);
//                System.out.println(boneName + "----- " + boundBox + "  " + " " + x + "  " + y + "  " + this.x + "  " + this.y + "  " + boundBox.x+" "+boundBox.y);
//                System.out.println("boneWordx");
                boundBox.x -= this.getX();
                boundBox.y -= this.getY();
                if (boneTouchScale.containsKey(boneName)) {
                    Vector2 touchScale = boneTouchScale.get(boneName);
                    float twidth = boundBox.getWidth() * touchScale.x;
                    float theight = boundBox.getHeight() * touchScale.y;
                    boundBox.set(boundBox.x - (twidth - boundBox.getWidth()) / 2, boundBox.y - (theight - boundBox.getHeight()) / 2, twidth, theight);
                }
//                System.out.println(boneName + "----- " + boundBox + "  " + temp);
                if (boundBox.contains(touchTemp.set(x, y))) {   //计算点击区域是否在矩形区域内
                    focusBoneIndex = i;
                    return this;
                }
            }
        }

        return super.hit(x, y, touchable);
    }

    public Vector2 touchTemp = new Vector2();


    public static void getBoundingRectangle(float[] vertices, Rectangle result) {
        float minX = vertices[0];
        float minY = vertices[1];
        float maxX = vertices[0];
        float maxY = vertices[1];

        final int numFloats = vertices.length;
        for (int i = 2; i < numFloats; i += 2) {
            minX = minX > vertices[i] ? vertices[i] : minX;
            minY = minY > vertices[i + 1] ? vertices[i + 1] : minY;
            maxX = maxX < vertices[i] ? vertices[i] : maxX;
            maxY = maxY < vertices[i + 1] ? vertices[i + 1] : maxY;
        }

        result.x = minX;
        result.y = minY;
        result.width = maxX - minX;
        result.height = maxY - minY;
    }

    @SuppressWarnings("unchecked")
    public void draw(Batch batch, float parentAlpha) {
        Color color = skeleton.getColor();
        float oldAlpha = color.a;
        skeleton.getColor().a *= parentAlpha;

        int b1 = batch.getBlendDstFunc();
        int b2 = batch.getBlendSrcFunc();
        skeleton.setPosition(getX() + getWidth() / 2, getY()+getHeight()/2);
        skeleton.updateWorldTransform();
        if (clip) {
            batch.flush();
            clipBegin();
            if (batch instanceof PolygonSpriteBatch)
                renderer.draw((PolygonSpriteBatch) batch, skeleton);
            else
                renderer.draw(batch, skeleton);
            batch.flush();
            clipEnd();
        } else {
            if (batch instanceof PolygonSpriteBatch)
                renderer.draw((PolygonSpriteBatch) batch, skeleton);
            else
                renderer.draw(batch, skeleton);
        }
        batch.setBlendFunction(b2, b1);

        color.a = oldAlpha;
    }


    @Override
    public void setScale(float scaleX, float scaleY) {
        super.setScale(scaleX, scaleY);
        skeleton.setScale(scaleX, scaleY);
    }

    public SkeletonRenderer getRenderer() {
        return renderer;
    }

    public void setRenderer(SkeletonRenderer renderer) {
        this.renderer = renderer;
    }

    public Skeleton getSkeleton() {
        return skeleton;
    }

    public void setSkeleton(Skeleton skeleton) {
        this.skeleton = skeleton;
    }

    public AnimationState getAnimationState() {
        return state;
    }

    public void setAnimationState(AnimationState state) {
        this.state = state;
    }

    public void setAnimation(String animation) {
        this.setAnimation(animation, true, 0);
    }

    public void setAnimation(String animation, boolean loop) {
        this.setAnimation(animation, loop, 0);
    }

    public void setAnimation(String animation, boolean loop, int trackIndex) {
        this.state.setAnimation(trackIndex, animation, loop);
    }

    public void setClip(boolean b) {
        clip = b;
    }

    public void setSpeed(float delta){
        state.setTimeScale(delta);
    }
}
